@@ -1,78 +1,84 @@
-# LaTeX2HTML 2002-2-1 (1.71)
+# LaTeX2HTML 2008 (1.71)
 # Associate images original text with physical files.
 
 
+$key = q/tau_{3}^{};MSF=1.6;LFS=12;AAT/;
+$cached_env_img{$key} = q|<IMG
+ WIDTH="20" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
+ SRC="|."$dir".q|img12.png"
+ ALT="$ \tau_{3}^{}$">|; 
+
+$key = q/displaystyletau_{2}^{};MSF=1.6;LFS=12;AAT/;
+$cached_env_img{$key} = q|<IMG
+ WIDTH="20" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
+ SRC="|."$dir".q|img6.png"
+ ALT="$\displaystyle \tau_{2}^{}$">|; 
+
+$key = q/tau_{2}^{};MSF=1.6;LFS=12;AAT/;
+$cached_env_img{$key} = q|<IMG
+ WIDTH="20" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
+ SRC="|."$dir".q|img11.png"
+ ALT="$ \tau_{2}^{}$">|; 
+
+$key = q/tau_{1}^{};MSF=1.6;LFS=12;AAT/;
+$cached_env_img{$key} = q|<IMG
+ WIDTH="20" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
+ SRC="|."$dir".q|img10.png"
+ ALT="$ \tau_{1}^{}$">|; 
+
+$key = q/displaystyle{frac{{k-1}}{{nr3}}};MSF=1.6;LFS=12;AAT/;
+$cached_env_img{$key} = q|<IMG
+ WIDTH="52" HEIGHT="61" ALIGN="MIDDLE" BORDER="0"
+ SRC="|."$dir".q|img7.png"
+ ALT="$\displaystyle {\frac{{k-1}}{{nr3}}}$">|; 
+
+$key = q/displaystyle{frac{{j-1}}{{nr2}}};MSF=1.6;LFS=12;AAT/;
+$cached_env_img{$key} = q|<IMG
+ WIDTH="50" HEIGHT="61" ALIGN="MIDDLE" BORDER="0"
+ SRC="|."$dir".q|img5.png"
+ ALT="$\displaystyle {\frac{{j-1}}{{nr2}}}$">|; 
+
 $key = q/displaystyletau_{1}^{};MSF=1.6;LFS=12;AAT/;
 $cached_env_img{$key} = q|<IMG
  WIDTH="20" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
- SRC="|."$dir".q|img3.png"
+ SRC="|."$dir".q|img4.png"
  ALT="$\displaystyle \tau_{1}^{}$">|; 
 
-$key = q/displaystyle{frac{{i-1}}{{nr1}}};MSF=1.6;LFS=12;AAT/;
+$key = q/displaystyletau_{3}^{};MSF=1.6;LFS=12;AAT/;
 $cached_env_img{$key} = q|<IMG
- WIDTH="48" HEIGHT="61" ALIGN="MIDDLE" BORDER="0"
+ WIDTH="20" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
+ SRC="|."$dir".q|img8.png"
+ ALT="$\displaystyle \tau_{3}^{}$">|; 
+
+$key = q/le;MSF=1.6;LFS=12;AAT/;
+$cached_env_img{$key} = q|<IMG
+ WIDTH="19" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
  SRC="|."$dir".q|img2.png"
- ALT="$\displaystyle {\frac{{i-1}}{{nr1}}}$">|; 
+ ALT="$ \le$">|; 
+
+$key = q/leq;MSF=1.6;LFS=12;AAT/;
+$cached_env_img{$key} = q|<IMG
+ WIDTH="19" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
+ SRC="|."$dir".q|img14.png"
+ ALT="$ \leq$">|; 
 
 $key = q/sim;MSF=1.6;LFS=12;AAT/;
 $cached_env_img{$key} = q|<IMG
  WIDTH="19" HEIGHT="19" ALIGN="BOTTOM" BORDER="0"
- SRC="|."$dir".q|img12.png"
+ SRC="|."$dir".q|img13.png"
  ALT="$ \sim$">|; 
 
 $key = q/tau_{i}^{};MSF=1.6;LFS=12;AAT/;
 $cached_env_img{$key} = q|<IMG
  WIDTH="18" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
- SRC="|."$dir".q|img8.png"
+ SRC="|."$dir".q|img9.png"
  ALT="$ \tau_{i}^{}$">|; 
 
-$key = q/displaystyletau_{3}^{};MSF=1.6;LFS=12;AAT/;
+$key = q/displaystyle{frac{{i-1}}{{nr1}}};MSF=1.6;LFS=12;AAT/;
 $cached_env_img{$key} = q|<IMG
- WIDTH="20" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
- SRC="|."$dir".q|img7.png"
- ALT="$\displaystyle \tau_{3}^{}$">|; 
-
-$key = q/tau_{3}^{};MSF=1.6;LFS=12;AAT/;
-$cached_env_img{$key} = q|<IMG
- WIDTH="20" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
- SRC="|."$dir".q|img11.png"
- ALT="$ \tau_{3}^{}$">|; 
-
-$key = q/le;MSF=1.6;LFS=12;AAT/;
-$cached_env_img{$key} = q|<IMG
- WIDTH="19" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
- SRC="|."$dir".q|img1.png"
- ALT="$ \le$">|; 
-
-$key = q/displaystyle{frac{{j-1}}{{nr2}}};MSF=1.6;LFS=12;AAT/;
-$cached_env_img{$key} = q|<IMG
- WIDTH="50" HEIGHT="61" ALIGN="MIDDLE" BORDER="0"
- SRC="|."$dir".q|img4.png"
- ALT="$\displaystyle {\frac{{j-1}}{{nr2}}}$">|; 
-
-$key = q/tau_{2}^{};MSF=1.6;LFS=12;AAT/;
-$cached_env_img{$key} = q|<IMG
- WIDTH="20" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
- SRC="|."$dir".q|img10.png"
- ALT="$ \tau_{2}^{}$">|; 
-
-$key = q/displaystyle{frac{{k-1}}{{nr3}}};MSF=1.6;LFS=12;AAT/;
-$cached_env_img{$key} = q|<IMG
- WIDTH="52" HEIGHT="61" ALIGN="MIDDLE" BORDER="0"
- SRC="|."$dir".q|img6.png"
- ALT="$\displaystyle {\frac{{k-1}}{{nr3}}}$">|; 
-
-$key = q/tau_{1}^{};MSF=1.6;LFS=12;AAT/;
-$cached_env_img{$key} = q|<IMG
- WIDTH="20" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
- SRC="|."$dir".q|img9.png"
- ALT="$ \tau_{1}^{}$">|; 
-
-$key = q/displaystyletau_{2}^{};MSF=1.6;LFS=12;AAT/;
-$cached_env_img{$key} = q|<IMG
- WIDTH="20" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
- SRC="|."$dir".q|img5.png"
- ALT="$\displaystyle \tau_{2}^{}$">|; 
+ WIDTH="48" HEIGHT="61" ALIGN="MIDDLE" BORDER="0"
+ SRC="|."$dir".q|img3.png"
+ ALT="$\displaystyle {\frac{{i-1}}{{nr1}}}$">|; 
 
 1;
 
