@@ -112,7 +112,7 @@
       USE constants,          ONLY: pi, fpi
       USE mp,                 ONLY: mp_sum
       USE io_global,          ONLY: stdout, ionode
-      USE mp_global,          ONLY: intra_bgrp_comm, nbgrp, inter_bgrp_comm, me_bgrp
+      USE mp_global,          ONLY: intra_bgrp_comm, nbgrp, inter_bgrp_comm, me_bgrp, nproc_bgrp
       USE funct,              ONLY: dft_is_meta
       USE cg_module,          ONLY: tcg
       USE cp_interfaces,      ONLY: stress_kin, enkin
@@ -121,7 +121,8 @@
       USE cp_interfaces,      ONLY: checkrho, ennl, calrhovan, dennl
       USE cp_main_variables,  ONLY: iprint_stdout, descla
       USE wannier_base,       ONLY: iwf
-      USE cp_main_variables,  ONLY: rhopr ! Lingzhu Kong
+      USE exx_module,         ONLY: rhopr 
+      USE input_parameters,   ONLY: tcpbo ! BS
 !
       IMPLICIT NONE
       INTEGER nfi
@@ -147,6 +148,11 @@
       REAL(DP) :: rsumr(2), rsumg(2), sa1, sa2, detmp(6), mtmp(3,3)
       REAL(DP) :: rnegsum, rmin, rmax, rsum
       COMPLEX(DP) :: ci,fp,fm
+#if defined(__INTEL_COMPILER)
+#if __INTEL_COMPILER  >= 1300
+!dir$ attributes align: 4096 :: psi, psis, drhovan
+#endif
+#endif
       COMPLEX(DP), ALLOCATABLE :: psi(:), psis(:)
       REAL(DP), ALLOCATABLE :: drhovan(:,:,:,:,:)
 
@@ -219,11 +225,12 @@
       !
       COMPUTE_CHARGE: IF( trhor .AND. ( .NOT. thdyn ) ) THEN
          !
-         !   non self-consistent calculation  
-         !   charge density is read from unit 47
+         !     ==================================================================
+         !     non self-consistent charge: charge density is read from unit 47
+         !     ==================================================================
+         !
+         ! Lingzhu Kong
          !
-!=============================================================
-! Lingzhu Kong
          IF( first ) THEN
             CALL read_rho( nspin, rhor )
             rhopr = rhor
@@ -231,9 +238,9 @@
          ELSE
             rhor = rhopr
          END IF
-!=============================================================
-!
+
          ALLOCATE( psi( dfftp%nnr ) )
+
          IF(nspin.EQ.1)THEN
             iss=1
             DO ir=1,dfftp%nnr
@@ -259,8 +266,9 @@
          ENDIF
 
          DEALLOCATE( psi )
-!
+
       ELSE
+         !
          !     ==================================================================
          !     self-consistent charge
          !     ==================================================================
@@ -297,42 +305,9 @@
                rhos(ir,iss1)=rhos(ir,iss1) + sa1*( DBLE(psis(ir)))**2
             END DO
             !
-         ELSE IF( dffts%have_task_groups ) THEN
+         ELSE 
             !
-            CALL loop_over_states_tg()
-            !
-         ELSE
-            !
-            ALLOCATE( psis( dffts%nnr ) ) 
-            !
-            DO i = 1, nbsp_bgrp, 2
-               !
-               CALL c2psi( psis, dffts%nnr, c_bgrp( 1, i ), c_bgrp( 1, i+1 ), ngw, 2 )
-
-               CALL invfft('Wave',psis, dffts )
-               !
-               iss1 = ispin_bgrp(i)
-               sa1  = f_bgrp(i) / omega
-               IF ( i .NE. nbsp_bgrp ) THEN
-                  iss2 = ispin_bgrp(i+1)
-                  sa2  = f_bgrp(i+1) / omega
-               ELSE
-                  iss2 = iss1
-                  sa2  = 0.0d0
-               END IF
-               !
-               DO ir = 1, dffts%nnr
-                  rhos(ir,iss1) = rhos(ir,iss1) + sa1 * ( DBLE(psis(ir)))**2
-                  rhos(ir,iss2) = rhos(ir,iss2) + sa2 * (AIMAG(psis(ir)))**2
-               END DO
-               !
-            END DO
-            !
-            IF( nbgrp > 1 ) THEN
-               call mp_sum( rhos, inter_bgrp_comm )
-            END IF
-            !
-            DEALLOCATE( psis ) 
+            CALL loop_over_states()
             !
          END IF
          !
@@ -342,26 +317,34 @@
          !
          IF(nspin.EQ.1)THEN
             iss=1
+!$omp parallel do
             DO ir=1,dffts%nnr
                psis(ir)=CMPLX(rhos(ir,iss),0.d0,kind=DP)
             END DO
+!$omp end parallel do
             CALL fwfft('Smooth', psis, dffts )
+!$omp parallel do
             DO ig=1,ngms
                rhog(ig,iss)=psis(nls(ig))
             END DO
+!$omp end parallel do
          ELSE
             isup=1
             isdw=2
+!$omp parallel do
              DO ir=1,dffts%nnr
                psis(ir)=CMPLX(rhos(ir,isup),rhos(ir,isdw),kind=DP)
             END DO
+!$omp end parallel do
             CALL fwfft('Smooth',psis, dffts )
+!$omp parallel do private(fp,fm)
             DO ig=1,ngms
                fp= psis(nls(ig)) + psis(nlsm(ig))
                fm= psis(nls(ig)) - psis(nlsm(ig))
                rhog(ig,isup)=0.5d0*CMPLX( DBLE(fp),AIMAG(fm),kind=DP)
                rhog(ig,isdw)=0.5d0*CMPLX(AIMAG(fp),-DBLE(fm),kind=DP)
             END DO
+!$omp end parallel do
          ENDIF
          !
          ALLOCATE( psi( dfftp%nnr ) )
@@ -372,14 +355,18 @@
             ! 
             iss=1
             psi (:) = (0.d0, 0.d0)
+!$omp parallel do
             DO ig=1,ngms
                psi(nlm(ig))=CONJG(rhog(ig,iss))
                psi(nl (ig))=      rhog(ig,iss)
             END DO
+!$omp end parallel do
             CALL invfft('Dense',psi, dfftp )
+!$omp parallel do
             DO ir=1,dfftp%nnr
                rhor(ir,iss)=DBLE(psi(ir))
             END DO
+!$omp end parallel do
             !
          ELSE 
             !
@@ -388,15 +375,19 @@
             isup=1
             isdw=2
             psi (:) = (0.d0, 0.d0)
+!$omp parallel do
             DO ig=1,ngms
                psi(nlm(ig))=CONJG(rhog(ig,isup))+ci*CONJG(rhog(ig,isdw))
                psi(nl(ig))=rhog(ig,isup)+ci*rhog(ig,isdw)
             END DO
+!$omp end parallel do
             CALL invfft('Dense',psi, dfftp )
+!$omp parallel do
             DO ir=1,dfftp%nnr
                rhor(ir,isup)= DBLE(psi(ir))
                rhor(ir,isdw)=AIMAG(psi(ir))
             END DO
+!$omp end parallel do
          ENDIF
          !
          IF ( dft_is_meta() ) CALL kedtauofr_meta( c_bgrp, psi, SIZE( psi ), psis, SIZE( psis ) ) ! METAGGA
@@ -424,8 +415,12 @@
 !
 !     here to check the integral of the charge density
 !
+! BS: I have turned off computing and printing integrated electronic density at
+! every iprint_stdout steps during CP-BO calculations ... 
+!     IF( ( iverbosity > 1 ) .OR. ( nfi == 0 ) .OR. &
+!         ( MOD(nfi, iprint_stdout) == 0 ) .AND. ( .NOT. tcg ) ) THEN
       IF( ( iverbosity > 1 ) .OR. ( nfi == 0 ) .OR. &
-          ( MOD(nfi, iprint_stdout) == 0 ) .AND. ( .NOT. tcg ) ) THEN
+          ( MOD(nfi, iprint_stdout) == 0 ) .AND. ( .NOT. tcg ) .AND. (.NOT.tcpbo )) THEN
 
          IF( iverbosity > 1 ) THEN
             CALL checkrho( dfftp%nnr, nspin, rhor, rmin, rmax, rsum, rnegsum )
@@ -486,12 +481,12 @@
          RETURN
       END SUBROUTINE
 
-      !
       !
 
-      SUBROUTINE loop_over_states_tg
+      SUBROUTINE loop_over_states
          !
          USE parallel_include
+         USE fft_parallel,           ONLY: pack_group_sticks, fw_tg_cft3_z, fw_tg_cft3_scatter, fw_tg_cft3_xy
          !
          !        MAIN LOOP OVER THE EIGENSTATES
          !           - This loop is also parallelized within the task-groups framework
@@ -500,11 +495,19 @@
          IMPLICIT NONE
          !
          INTEGER :: from, ii, eig_index, eig_offset
+
+#if defined(__INTEL_COMPILER)
+#if __INTEL_COMPILER  >= 1300
+!dir$ attributes align: 4096 :: tmp_rhos, aux
+#endif
+#endif
          REAL(DP), ALLOCATABLE :: tmp_rhos(:,:)
+         COMPLEX(DP), ALLOCATABLE :: aux(:)
 
          ALLOCATE( psis( dffts%tg_nnr * dffts%nogrp ) ) 
+         ALLOCATE( aux( dffts%tg_nnr * dffts%nogrp ) ) 
          !
-         ALLOCATE( tmp_rhos ( dffts%nr1x*dffts%nr2x*dffts%tg_npp( me_bgrp + 1 ), nspin ) )
+         ALLOCATE( tmp_rhos ( dffts%nr1x * dffts%nr2x * dffts%tg_npp( me_bgrp + 1 ), nspin ) )
          !
          tmp_rhos = 0_DP
 
@@ -514,16 +517,15 @@
             !  The size of psis is nnr: which is equal to the total number
             !  of local fourier coefficients.
             !
-!$omp parallel default(shared), private(eig_offset, ig, eig_index )
-            !
-!$omp do
-            do ig = 1, SIZE(psis)
-               psis (ig) = (0.d0, 0.d0)
-            end do
+
+#ifdef __MPI
+
+            aux = (0.d0, 0.d0)
             !
             !  Loop for all local g-vectors (ngw)
-            !  c: stores the Fourier expansion coefficients
-            !     the i-th column of c corresponds to the i-th state
+            !  ci_bgrp: stores the Fourier expansion coefficients
+            !     the i-th column of c_bgrp corresponds to the i-th state (in
+            !     this band group)
             !  nlsm and nls matrices: hold conversion indices form 3D to
             !     1-D vectors. Columns along the z-direction are stored contigiously
             !
@@ -535,36 +537,42 @@
             do eig_index = 1, 2*dffts%nogrp, 2   
                !
                !  here we pack 2*nogrp electronic states in the psis array
+               !  note that if nogrp == nproc_bgrp each proc perform a full 3D
+               !  fft and the scatter phase is local (without communication)
                !
                IF ( ( i + eig_index - 1 ) <= nbsp_bgrp ) THEN
                   !
-                  !  Outer loop for eigenvalues
                   !  The  eig_index loop is executed only ONCE when NOGRP=1.
-                  !  Equivalent to the case with no task-groups
-                  !  dfft%nsw(me) holds the number of z-sticks for the current processor per wave-function
-                  !  We can either send these in the group with an mpi_allgather...or put the
-                  !  in the PSIS vector (in special positions) and send them with them.
-                  !  Otherwise we can do this once at the beginning, before the loop.
-                  !  we choose to do the latter one.
-
-!$omp do
-                  do ig=1,ngw
-                     psis(nlsm(ig)+eig_offset*dffts%tg_nnr)=conjg(c_bgrp(ig,i+eig_index-1))+ci*conjg(c_bgrp(ig,i+eig_index))
-                     psis(nls(ig)+eig_offset*dffts%tg_nnr)=c_bgrp(ig,i+eig_index-1)+ci*c_bgrp(ig,i+eig_index)
-                  end do
+                  !
+                  CALL c2psi( aux(eig_offset*dffts%tg_nnr+1), dffts%tg_nnr, &
+                        c_bgrp( 1, i+eig_index-1 ), c_bgrp( 1, i+eig_index ), ngw, 2 )
                   !
                   eig_offset = eig_offset + 1
                   !
                ENDIF
                !
             end do
-!$omp end parallel
-
+            !
             !  2*NOGRP are trasformed at the same time
             !  psis: holds the fourier coefficients of the current proccesor
             !        for eigenstates i and i+2*NOGRP-1
             !
-            CALL invfft( 'Wave', psis, dffts )
+            !  now redistribute data
+            !
+            CALL pack_group_sticks( aux, psis, dffts )
+            !
+            CALL fw_tg_cft3_z( psis, dffts, aux )
+            CALL fw_tg_cft3_scatter( psis, dffts, aux )
+            CALL fw_tg_cft3_xy( psis, dffts )
+#else
+
+            psis = (0.d0, 0.d0)
+
+            CALL c2psi( psis, dffts%nnr, c_bgrp( 1, i ), c_bgrp( 1, i+1 ), ngw, 2 )
+
+            CALL invfft('Wave',psis, dffts )
+
+#endif
             !
             ! Now the first proc of the group holds the first two bands
             ! of the 2*nogrp bands that we are processing at the same time,
@@ -619,7 +627,6 @@
             IF( ir > SIZE( psis ) ) &
                CALL errore( ' rhoofr ', ' psis size too small ', ir )
 
-!$omp parallel do default(shared)
             do ir = 1, dffts%nr1x*dffts%nr2x*dffts%tg_npp( me_bgrp + 1 )
                tmp_rhos(ir,iss1) = tmp_rhos(ir,iss1) + sa1*( real(psis(ir)))**2
                tmp_rhos(ir,iss2) = tmp_rhos(ir,iss2) + sa2*(aimag(psis(ir)))**2
@@ -631,6 +638,11 @@
             CALL mp_sum( tmp_rhos, inter_bgrp_comm )
          END IF
 
+         !ioff = 0
+         !DO ip = 1, nproc_bgrp
+         !   CALL MPI_REDUCE( rho(1+ioff*nr1*nr2,1), rhos(1,1), dffts%nnr, MPI_DOUBLE_PRECISION, MPI_SUM, ip-1, intra_bgrp_comm, ierr)
+         !   ioff = ioff + dffts%npp( ip )
+         !END DO
          IF ( dffts%nogrp > 1 ) THEN
             CALL mp_sum( tmp_rhos, gid = dffts%ogrp_comm )
          ENDIF
@@ -651,10 +663,11 @@
          ENDDO
 
          DEALLOCATE( tmp_rhos )
+         DEALLOCATE( aux ) 
          DEALLOCATE( psis ) 
 
          RETURN
-      END SUBROUTINE loop_over_states_tg
+      END SUBROUTINE loop_over_states
 
 !-----------------------------------------------------------------------
    END SUBROUTINE rhoofr_cp
@@ -683,6 +696,11 @@
 ! output
       real(DP) ::    gradr( dfftp%nnr, 3, nspin )
 ! local
+#if defined(__INTEL_COMPILER)
+#if __INTEL_COMPILER  >= 1300
+!dir$ attributes align: 4096 :: v
+#endif
+#endif
       complex(DP), allocatable :: v(:)
       complex(DP) :: ci
       integer     :: iss, ig, ir
@@ -817,6 +835,11 @@ SUBROUTINE drhov(irb,eigrb,rhovan,drhovan,rhog,rhor,drhog,drhor)
      &     isa, ia, ir, ijs
       REAL(DP) :: asumt, dsumt
       COMPLEX(DP) fp, fm, ci
+#if defined(__INTEL_COMPILER)
+#if __INTEL_COMPILER  >= 1300
+!dir$ attributes align: 4096 :: v, dqgbt,qv
+#endif
+#endif
       COMPLEX(DP), ALLOCATABLE :: v(:)
       COMPLEX(DP), ALLOCATABLE:: dqgbt(:,:)
       COMPLEX(DP), ALLOCATABLE :: qv(:)
@@ -1115,6 +1138,11 @@ SUBROUTINE rhov(irb,eigrb,rhovan,rhog,rhor)
       INTEGER     :: isup, isdw, nfft, ifft, iv, jv, ig, ijv, is, iss, isa, ia, ir, i, j
       REAL(DP)    :: sumrho
       COMPLEX(DP) :: ci, fp, fm, ca
+#if defined(__INTEL_COMPILER)
+#if __INTEL_COMPILER  >= 1300
+!dir$ attributes align: 4096 :: qgbt, v, qv
+#endif
+#endif
       COMPLEX(DP), ALLOCATABLE :: qgbt(:,:)
       COMPLEX(DP), ALLOCATABLE :: v(:)
       COMPLEX(DP), ALLOCATABLE :: qv(:)
