@@ -19,7 +19,6 @@ MODULE cp_main_variables
   USE pres_ai_mod,       ONLY : abivol, abisur, jellium, t_gauss, rho_gaus, &
                                 v_vol, posv, f_vol
   USE descriptors,       ONLY : la_descriptor
-  USE control_flags,     ONLY : lwfnscf, lwfpbe0, lwfpbe0nscf  ! Lingzhu Kong
   !
   IMPLICIT NONE
   SAVE
@@ -88,7 +87,6 @@ MODULE cp_main_variables
   COMPLEX(DP), ALLOCATABLE :: rhog(:,:)
   REAL(DP),    ALLOCATABLE :: rhor(:,:), rhos(:,:)
   REAL(DP),    ALLOCATABLE :: vpot(:,:)
-  REAL(DP),    ALLOCATABLE :: rhopr(:,:)   ! Lingzhu Kong
   !
   ! derivative wrt cell
   !
@@ -107,38 +105,6 @@ MODULE cp_main_variables
                              ! for the present run
   INTEGER :: iprint_stdout=1 ! define how often CP writes verbose information to stdout
   !
-  !==========================================================================
-  ! Lingzhu Kong
-            
-     INTEGER  :: my_nbspx
-     INTEGER  :: nord2            ! order of expansion ( points on one side)
-     INTEGER  :: lap_neig(3,3)    ! new directions
-     REAL(DP) :: lap_dir_step(3)  ! step in the new directions
-     INTEGER  :: lap_dir_num      ! number of new directions
-     REAL(DP) :: b_lap(6)         ! coefficients of the directions
-     INTEGER  :: lap_dir(3)       ! activeness of the new directions
-
-     INTEGER  np_in_sp, np_in_sp2  ! number of grid points in the 1st sphere and the shell between 1st and 2nd sphere
-
-! conversion between 3D index (i,j,k) and 1D index np
-     INTEGER,     ALLOCATABLE     :: odtothd_in_sp(:,:)
-     INTEGER,     ALLOCATABLE     :: thdtood_in_sp(:,:,:)
-     INTEGER,     ALLOCATABLE     :: thdtood(:,:,:)
-     REAL(DP),    ALLOCATABLE     :: xx_in_sp(:)
-     REAL(DP),    ALLOCATABLE     :: yy_in_sp(:)
-     REAL(DP),    ALLOCATABLE     :: zz_in_sp(:)
-
-     REAL(DP),    ALLOCATABLE     :: selfv(:,:,:)
-     REAL(DP),    ALLOCATABLE     :: pairv(:,:,:,:)
-     REAL(DP),    ALLOCATABLE     :: exx_potential(:, :)
-
-     REAL(DP),    ALLOCATABLE     :: clm(:,:)
-     REAL(DP),    ALLOCATABLE     :: coeke(:,:)
-     REAL(DP),    ALLOCATABLE     :: vwc(:,:)
-     INTEGER  ::  lmax
-     INTEGER  ::  n_exx =0
-!==========================================================================
-
   CONTAINS
     !
     !------------------------------------------------------------------------
@@ -152,15 +118,6 @@ MODULE cp_main_variables
                              me_bgrp, ortho_comm_id
       USE mp,          ONLY: mp_max, mp_min
       USE descriptors, ONLY: la_descriptor, descla_init
-!==============================================================================
-!Lingzhu Kong
-      USE mp_global,               ONLY  : nproc_image
-      USE fft_base,                ONLY  : dffts
-      USE electrons_base,          ONLY  : nbsp
-      USE wannier_base,            ONLY  : neigh, exx_ps_rcut, exx_me_rcut, vnbsp
-      USE control_flags,           ONLY  : lwfnscf, lwfpbe0, lwfpbe0nscf
-!===============================================================================
-
       !
       INTEGER,           INTENT(IN) :: ngw, ngw_g, ngb, ngs, ng, nr1,nr2,nr3, &
                                        nnr, nrxxs, nat, nax, nsp, nspin, &
@@ -323,32 +280,6 @@ MODULE cp_main_variables
       IF( ALLOCATED( rhor ) )    DEALLOCATE( rhor )
       IF( ALLOCATED( rhos ) )    DEALLOCATE( rhos )
       IF( ALLOCATED( rhog ) )    DEALLOCATE( rhog )
-!====================================================================
-!Lingzhu Kong
-      IF ( lwfpbe0 )THEN
-         IF( ALLOCATED( selfv ) )          DEALLOCATE( selfv )
-      ENDIF
-
-      IF ( lwfpbe0nscf .or. lwfnscf)THEN
-         IF( ALLOCATED( rhopr ) )          DEALLOCATE( rhopr )
-      ENDIF
-         
-      IF ( lwfpbe0nscf )THEN
-         IF( ALLOCATED( vwc)    )          DEALLOCATE( vwc )
-      ENDIF
-      IF ( lwfpbe0 .or. lwfpbe0nscf ) THEN
-         IF( ALLOCATED( pairv ) )          DEALLOCATE( pairv )
-         IF( ALLOCATED( exx_potential ) )  DEALLOCATE( exx_potential )
-         IF( ALLOCATED( odtothd_in_sp ) )  DEALLOCATE(odtothd_in_sp )
-         IF( ALLOCATED( thdtood_in_sp ) )  DEALLOCATE(thdtood_in_sp )
-         IF( ALLOCATED( thdtood  ))        DEALLOCATE(thdtood)
-         IF( ALLOCATED( xx_in_sp ))        DEALLOCATE(xx_in_sp )
-         IF( ALLOCATED( yy_in_sp ))        DEALLOCATE(yy_in_sp )
-         IF( ALLOCATED( zz_in_sp ))        DEALLOCATE(zz_in_sp )
-         IF( ALLOCATED( clm )     )        DEALLOCATE(clm)
-         IF( ALLOCATED( coeke)    )        DEALLOCATE(coeke)
-      END IF
-!===================================================================
       IF( ALLOCATED( drhog ) )   DEALLOCATE( drhog )
       IF( ALLOCATED( drhor ) )   DEALLOCATE( drhor )
       IF( ALLOCATED( bec_bgrp ) )     DEALLOCATE( bec_bgrp )
