@@ -54,6 +54,11 @@
       !
       !
      real(DP) ::  verl1, verl2, verl3
+#if defined(__INTEL_COMPILER)
+#if __INTEL_COMPILER  >= 1300
+!dir$ attributes align: 4096 :: emadt2, emaver, c2, c3, c2tmp, c3tmp, tg_rhos, ftmp, itmp
+#endif
+#endif
      real(DP),    allocatable :: emadt2(:)
      real(DP),    allocatable :: emaver(:)
      complex(DP), allocatable :: c2(:), c3(:), c2tmp(:), c3tmp(:)
@@ -290,8 +295,6 @@
 !
 !=----------------------------------------------------------------------------=!
 !
-!
-
 !=----------------------------------------------------------------------------=!
 
     SUBROUTINE runcp_uspp_force_pairing_x  &
