@@ -30,7 +30,7 @@ SUBROUTINE wf( clwf, c, bec, eigr, eigrb, taub, irb, &
   USE smallbox_gvec,                    ONLY : npb, nmb, ngb
   USE gvecw,                    ONLY : ngw
   USE gvect,       ONLY : gstart
-  USE control_flags,            ONLY : iverbosity
+  USE control_flags,            ONLY : iverbosity,conv_elec
   USE qgb_mod,                  ONLY : qgb
   USE wannier_base,             ONLY : wfg, nw, weight, indexplus, indexplusz, &
                                        indexminus, indexminusz, tag, tagp,     &
@@ -45,6 +45,9 @@ SUBROUTINE wf( clwf, c, bec, eigr, eigrb, taub, irb, &
   USE fft_base,                 ONLY : dfftp, dfftb
   USE printout_base,            ONLY : printout_base_open, printout_base_unit, &
                                        printout_base_close
+  USE cp_main_variables,        ONLY : nfi, iprint_stdout
+  USE time_step,                ONLY : tps
+  USE input_parameters,         ONLY : tcpbo
   USE parallel_include
   !
   IMPLICIT NONE
@@ -559,7 +562,7 @@ SUBROUTINE wf( clwf, c, bec, eigr, eigrb, taub, irb, &
      IF(nspin.EQ.1) THEN
         IF(.NOT.what1) THEN
            IF(wfsd==1) THEN
-              CALL ddyn_u(nbsp,O,Uall) ! Lingzhu Kong
+              CALL ddyn_u(nbsp,O,Uall, 1)
            ELSE IF(wfsd==2) THEN
               CALL wfsteep(nbsp,O,Uall,b1,b2,b3)
            ELSE IF(wfsd==3) THEN
@@ -579,7 +582,7 @@ SUBROUTINE wf( clwf, c, bec, eigr, eigrb, taub, irb, &
         END DO
         IF(.NOT.what1) THEN
            IF(wfsd==1) THEN
-             CALL ddyn_u(nupdwn(1), Ospin, Uspin) ! Lingzhu Kong
+             CALL ddyn_u(nupdwn(1), Ospin, Uspin, 1)
            ELSE IF (wfsd==2) THEN
              CALL wfsteep(nupdwn(1), Ospin, Uspin,b1,b2,b3)
            ELSE
@@ -602,7 +605,7 @@ SUBROUTINE wf( clwf, c, bec, eigr, eigrb, taub, irb, &
         END DO
         IF(.NOT.what1) THEN
            IF(wfsd==1) THEN
-              CALL ddyn_u(nupdwn(2), Ospin, Uspin) ! Lingzhu Kong
+              CALL ddyn_u(nupdwn(2), Ospin, Uspin, 2)
            ELSE IF (wfsd==2) THEN
               CALL wfsteep(nupdwn(2), Ospin, Uspin,b1,b2,b3)
            ELSE
@@ -695,33 +698,6 @@ COMB:   DO k=3**nw-1,0,-1
      !
   END DO
   !
-  IF ( ionode ) THEN
-     !
-     iunit = printout_base_unit( "wfc" )
-     CALL printout_base_open( "wfc" )
-     IF ( .NOT. what1 ) THEN
-        !
-        ! ... pbc are imposed here in the range [0,1]
-        !
-        DO i = 1, nbsp
-           !
-           temp_vec(:) = MATMUL( ainv(:,:), wfc(:,i) )
-           !
-           temp_vec(:) = temp_vec(:) - floor (temp_vec(:))
-           !
-           temp_vec(:) = MATMUL( h(:,:), temp_vec(:) )
-           !
-           WRITE( iunit, '(3f20.14)' ) temp_vec(:)
-           !
-        END DO
-        !
-     END IF
-     CALL printout_base_close( "wfc" )
-     !
-  END IF
-  !
-  !
-  !
   DEALLOCATE( wr, W, gr, EW, f3, f4, mt0, mt )
   !
 #if defined (__MPI)
@@ -757,16 +733,19 @@ SUBROUTINE ddyn( m, Omat, Umat, b1, b2, b3 )
   ! ... quantities bec
   !
   USE kinds,            ONLY : DP
-  USE io_global,        ONLY : stdout
+  USE io_global,        ONLY : ionode, stdout
   USE wannier_base,     ONLY : wf_friction, nsteps, tolw, adapt, wf_q, &
                                weight, nw, wfdt
   USE cell_base,        ONLY : alat
   USE constants,        ONLY : tpi, bohr_radius_angs
   USE electrons_base,   ONLY : nbsp
-  USE control_flags,    ONLY : iverbosity
+  USE control_flags,    ONLY : iverbosity,conv_elec
   USE mp_global,        ONLY : me_bgrp
   USE printout_base,    ONLY : printout_base_open, printout_base_unit, &
                                printout_base_close
+  USE cp_main_variables,        ONLY : nfi, iprint_stdout
+  USE time_step,                ONLY : tps
+  USE input_parameters,         ONLY : tcpbo
   USE parallel_include
   !
   IMPLICIT NONE
@@ -843,10 +822,10 @@ SUBROUTINE ddyn( m, Omat, Umat, b1, b2, b3 )
 
      IF(ABS(t0-oldt0).LT.tolw) THEN
         IF(me.EQ.1) THEN
-           WRITE(*,*) "MLWF Generated at Step",ini ! Lingzhu Kong
+           WRITE(stdout,*) "MLWF Generated at Step",ini,"  Convergence = ",ABS(t0-oldt0)
         END IF
         IF( iverbosity > 2) THEN
-           WRITE( stdout, * ) "MLWF Generated at Step",ini
+           WRITE(stdout,*) "MLWF Generated at Step",ini,"  Convergence = ",ABS(t0-oldt0)
         END IF
         GO TO 241
      END IF
@@ -950,10 +929,10 @@ SUBROUTINE ddyn( m, Omat, Umat, b1, b2, b3 )
 
      IF(ABS(t0-oldt0).GE.tolw.AND.ini.GE.nsteps) THEN
         IF(me.EQ.1) THEN
-           WRITE(*,*) "MLWF Not generated after",ini,"Steps." ! Lingzhu Kong
+           WRITE(stdout,*) "MLWF Generated at Step",ini,"  Convergence = ",ABS(t0-oldt0)
         END IF
         IF( iverbosity > 2) THEN
-           WRITE( stdout, * ) "MLWF Not generated after",ini,"Steps." 
+           WRITE(stdout,*) "MLWF Generated at Step",ini,"  Convergence = ",ABS(t0-oldt0)
         END IF
         GO TO 241
      END IF
@@ -964,21 +943,37 @@ SUBROUTINE ddyn( m, Omat, Umat, b1, b2, b3 )
 
 241 DEALLOCATE(wr, W)
 
-  spread=0.0d0
+  !BS .. to print WANNIER spreads at every iprint steps in .spr file ..    
+  IF( ( MOD( nfi, iprint_stdout ) == 0 ) )  THEN
+     IF ( ionode ) THEN
+        iunit = printout_base_unit( "spr" )
+        CALL printout_base_open( "spr" )
+        IF (.NOT.tcpbo) THEN
+           WRITE( iunit, '(I8,f16.8)' )nfi,tps
+        ELSE   
+           IF(conv_elec) WRITE( iunit, '(I8,f16.8)' )nfi,tps
+        END IF    
+     END IF    
+  END IF    
 
-!  IF(me.EQ.1) THEN ! Lingzhu Kong
-     iunit = printout_base_unit( "spr" )
-     CALL printout_base_open( "spr" )
-!  END IF ! Lingzhu Kong
+  spread=0.0d0
 
   DO i=1, m
      !
      mt=1.D0-DBLE(Oc(:,i,i)*CONJG(Oc(:,i,i)))
      sp = (alat*bohr_radius_angs/tpi)**2*SUM(mt*weight)
      !
-     IF(me.EQ.1) THEN
-        WRITE(iunit, '(f10.7)') sp
+     !BS .. to print WANNIER spreads at every iprint steps in .spr file ..    
+     IF( ( MOD( nfi, iprint_stdout ) == 0 ) ) THEN
+        IF ( ionode ) THEN
+           IF (.NOT.tcpbo) THEN
+              WRITE( iunit, '(f20.14)' ) sp
+           ELSE
+             IF(conv_elec) WRITE( iunit, '(f20.14)' ) sp
+           END IF   
+        END IF   
      END IF
+     !
      IF ( sp < 0.D0 ) &
         CALL errore( 'cp-wf', 'Something wrong WF Spread negative', 1 )
      !
@@ -986,8 +981,9 @@ SUBROUTINE ddyn( m, Omat, Umat, b1, b2, b3 )
      !
   END DO
 
-  IF(me.EQ.1) THEN
-     CALL printout_base_close( "spr" )
+  !BS .. to print WANNIER spreads at every iprint steps in .spr file ..    
+  IF( ( MOD( nfi, iprint_stdout ) == 0 ) ) THEN
+     IF ( ionode ) CALL printout_base_close( "spr" )
   END IF
 
   spread=spread/m
@@ -1022,7 +1018,7 @@ SUBROUTINE wfunc_init( clwf, b1, b2, b3, ibrav )
                                  indexplusz, indexminusz, tag, tagp, &
                                  wfg, weight, nw
   USE uspp_param,         ONLY : nvb
-  USE mp,                 ONLY : mp_barrier, mp_bcast, mp_gather, mp_set_displs
+  USE mp,                 ONLY : mp_barrier, mp_bcast, mp_gather, mp_set_displs, mp_sum
   USE mp_global,          ONLY : nproc_bgrp, me_bgrp, intra_bgrp_comm, root_bgrp
   USE fft_base,           ONLY : dfftp
   USE parallel_include     
@@ -1043,16 +1039,18 @@ SUBROUTINE wfunc_init( clwf, b1, b2, b3, ibrav )
   INTEGER :: igcount,nw1,jj,nw2, in, kk
   INTEGER, ALLOCATABLE :: i_1(:), j_1(:), k_1(:)
   INTEGER :: ti, tj, tk
-  REAL(DP) ::t1, vt, err1, err2, err3
-  INTEGER :: ti1,tj1,tk1
+  REAL(DP) ::t1, vt
   INTEGER :: me
+  INTEGER :: ng_me          !HK: the number of g vector in proc me
+  INTEGER :: g_start, g_end !HK: ig loop range for proc me
   !
+  CALL start_clock('wf_init')
 
   me = me_bgrp + 1
   !
-  IF ( nbsp < nproc_bgrp ) &
-     CALL errore( 'cp-wf', &
-                & 'Number of Processors is greater than the number of states', 1 )
+! IF ( nbsp < nproc_bgrp ) &
+!    CALL errore( 'cp-wf', &
+!               & 'Number of Processors is greater than the number of states', 1 )
   !
   ALLOCATE(gnx(3,ngw))
   ALLOCATE(gnn(3,ngw))
@@ -1076,15 +1074,8 @@ SUBROUTINE wfunc_init( clwf, b1, b2, b3, ibrav )
 
   CALL mp_set_displs( ngppp, displs, ntot, nproc_bgrp )
 
-  IF(me.EQ.1) THEN
-     ALLOCATE(bigg(3,ntot))
-     ALLOCATE(bign(3,ntot))
-  ELSE
-     ! NOTE: collected array should be allocated anyway 
-     ! with the right first dimensions, otherwise mp_gather will fail
-     ALLOCATE(bigg(3,1))
-     ALLOCATE(bign(3,1))
-  END IF
+  ALLOCATE(bigg(3,ntot))
+  ALLOCATE(bign(3,ntot))
 
 #else
   ntot=ngw
@@ -1139,6 +1130,17 @@ SUBROUTINE wfunc_init( clwf, b1, b2, b3, ibrav )
   !
   CALL mp_gather( gnn, bign, ngppp, displs, root_bgrp, intra_bgrp_comm )
   !
+  CALL mp_bcast( bigg,  root_bgrp, intra_bgrp_comm ) !parallelize wf_init with MPI (serial could be very slow for large system or high cutoff)
+  CALL mp_bcast( bign,  root_bgrp, intra_bgrp_comm ) !parallelization could be done better with MPI_SCATHER and MPI_GATHER (todo)
+  ng_me = ceiling(dble(ntot/nproc_bgrp))
+  g_start = (me-1)*ng_me + 1
+  g_end   = me*ng_me
+  if (g_start.gt.ntot) g_start = ntot+2
+  if (g_end.gt.ntot)   g_end = ntot+1
+  !
+#else
+  g_start = 1
+  g_end   = ntot
 #endif
 
   IF(me.EQ.1) THEN
@@ -1158,145 +1160,109 @@ SUBROUTINE wfunc_init( clwf, b1, b2, b3, ibrav )
 
   DO inw=1,nw1
      IF(i_1(inw).EQ.0.AND.j_1(inw).EQ.0) THEN
+        IF(gstart.EQ.2) THEN
+           indexminusz(1)=-1
+        END IF
+!$omp parallel do private(ti,tj,tk)
         DO ig=1,ngw
-           IF(gstart.EQ.2) THEN
-              indexminusz(1)=-1
-           END IF
            ti=(gnn(1,ig)+i_1(inw))
            tj=(gnn(2,ig)+j_1(inw))
            tk=(gnn(3,ig)+k_1(inw))
            DO ii=1,ngw
-              err1=ABS(gnx(1,ii)-ti)
-              err2=ABS(gnx(2,ii)-tj)
-              err3=ABS(gnx(3,ii)-tk)
               IF(gnn(1,ii).EQ.ti.AND.gnn(2,ii).EQ.tj.AND.gnn(3,ii).EQ.tk) THEN
                  indexplusz(ig)=ii
                  GO TO 224
-              ELSE
               END IF
            END DO
            indexplusz(ig)=-1
 224        ti=(-gnn(1,ig)+i_1(inw))
            tj=(-gnn(2,ig)+j_1(inw))
            tk=(-gnn(3,ig)+k_1(inw))
-           ti1=-gnn(1,ig)+i_1(inw)
-           tj1=-gnn(2,ig)+j_1(inw)
-           tk1=-gnn(3,ig)+k_1(inw)
-           IF(ti1.LT.0.OR.(ti1.EQ.0.AND.(tj1.LT.0.OR.(tj1.EQ.0.AND.tk1.LT.0)))) THEN
+           IF(ti.LT.0.OR.(ti.EQ.0.AND.(tj.LT.0.OR.(tj.EQ.0.AND.tk.LT.0)))) THEN
               DO ii=1,ngw
-                 err1=ABS(gnx(1,ii)+ti)
-                 err2=ABS(gnx(2,ii)+tj)
-                 err3=ABS(gnx(3,ii)+tk)
                  IF(gnn(1,ii).EQ.-ti.AND.gnn(2,ii).EQ.-tj.AND.gnn(3,ii).EQ.-tk) THEN
                     indexminusz(ig)=ii
                     GO TO 223
-                 ELSE
                  END IF
               END DO
               indexminusz(ig)=-1
            ELSE
               DO ii=1,ngw
-                 err1=ABS(gnx(1,ii)-ti)
-                 err2=ABS(gnx(2,ii)-tj)
-                 err3=ABS(gnx(3,ii)-tk)
                  IF(gnn(1,ii).EQ.ti.AND.gnn(2,ii).EQ.tj.AND.gnn(3,ii).EQ.tk) THEN
                     indexminusz(ig)=ii
                     GO TO 223
-                 ELSE
                  END IF
               END DO
               indexminusz(ig)=-1
            END IF
 223        CONTINUE
         END DO
+!$omp end parallel do
         WRITE( stdout, * ) "Translation", inw, "for", ngw, "G vectors"
      ELSE
 #ifdef __MPI
         IF(me.EQ.1) THEN   
 #endif
-           DO ig=1,ntot
-              IF(gstart.EQ.2) THEN
-                 indexminus(1,inw)=-1
-              END IF
-              ti=(bign(1,ig)+i_1(inw))
-              tj=(bign(2,ig)+j_1(inw))
-              tk=(bign(3,ig)+k_1(inw))
-              ti1=bign(1,ig)+i_1(inw)
-              tj1=bign(2,ig)+j_1(inw)
-              tk1=bign(3,ig)+k_1(inw)
-              IF(ti1.LT.0.OR.(ti1.EQ.0.AND.(tj1.LT.0.OR.(tj1.EQ.0.AND.tk1.LT.0)))) THEN
-                 DO ii=1,ntot
-                    err1=ABS(bigg(1,ii)+ti)
-                    err2=ABS(bigg(2,ii)+tj)
-                    err3=ABS(bigg(3,ii)+tk)
-                    !              if(err1.lt.vt.and.err2.lt.vt.and.err3.lt.vt) then
-                    IF(bign(1,ii).EQ.-ti.AND.bign(2,ii).EQ.-tj.AND.bign(3,ii).EQ.-tk) THEN
-                       indexplus(ig,inw)=ii
-                       tagp(ig,inw)=1
-                       !                write (6,*) "Found +", ig,ii,inw 
-                       !               write (6,*) "looking for", -ti,-tj,-tk
-                       GO TO 214
-                    ELSE
-                    END IF
-                 END DO
-                 indexplus(ig,inw)=-1
-                 tagp(ig,inw)=1
-              ELSE
-                 DO ii=1,ntot
-                    err1=ABS(bigg(1,ii)-ti)
-                    err2=ABS(bigg(2,ii)-tj)
-                    err3=ABS(bigg(3,ii)-tk)
-                    IF(bign(1,ii).EQ.ti.AND.bign(2,ii).EQ.tj.AND.bign(3,ii).EQ.tk) THEN
-                       indexplus(ig,inw)=ii
-                       tagp(ig,inw)=-1
-                       GO TO 214
-                    ELSE
-                    END IF
-                 END DO
-                 indexplus(ig,inw)=-1
-                 tagp(ig,inw)=-1
-              END IF
-214           ti=(-bign(1,ig)+i_1(inw))
-              tj=(-bign(2,ig)+j_1(inw))
-              tk=(-bign(3,ig)+k_1(inw))
-              ti1=-bign(1,ig)+i_1(inw)
-              tj1=-bign(2,ig)+j_1(inw)
-              tk1=-bign(3,ig)+k_1(inw)
-              IF(ti1.LT.0.OR.(ti1.EQ.0.AND.(tj1.LT.0.OR.(tj1.EQ.0.AND.tk1.LT.0)))) THEN
-                 DO ii=1,ntot
-                    err1=ABS(bigg(1,ii)+ti)
-                    err2=ABS(bigg(2,ii)+tj)
-                    err3=ABS(bigg(3,ii)+tk)
-                    IF(bign(1,ii).EQ.-ti.AND.bign(2,ii).EQ.-tj.AND.bign(3,ii).EQ.-tk) THEN
-                       indexminus(ig,inw)=ii
-                       tag(ig,inw)=1
-                       GO TO 213
-                    ELSE
-                    END IF
-                 END DO
-                 indexminus(ig,inw)=-1
-                 tag(ig,inw)=1
-              ELSE 
-                 DO ii=1,ntot
-                    err1=ABS(bigg(1,ii)-ti)
-                    err2=ABS(bigg(2,ii)-tj)
-                    err3=ABS(bigg(3,ii)-tk)
-                    IF(bign(1,ii).EQ.ti.AND.bign(2,ii).EQ.tj.AND.bign(3,ii).EQ.tk) THEN
-                       indexminus(ig,inw)=ii
-                       tag(ig,inw)=-1
-                       GO TO 213
-                    ELSE
-                    END IF
-                 END DO
-                 indexminus(ig,inw)=-1
-                 tag(ig,inw)=-1
-              END IF
-213           CONTINUE
-           END DO
-           WRITE( stdout, * ) "Translation", inw, "for", ntot, "G vectors"
+           IF(gstart.EQ.2) THEN
+              indexminus(1,inw)=-1
+           END IF
 #ifdef __MPI
         END IF
 #endif
+!$omp parallel do private(ti,tj,tk)
+        DO ig=g_start,g_end
+           ti=(bign(1,ig)+i_1(inw))
+           tj=(bign(2,ig)+j_1(inw))
+           tk=(bign(3,ig)+k_1(inw))
+           IF(ti.LT.0.OR.(ti.EQ.0.AND.(tj.LT.0.OR.(tj.EQ.0.AND.tk.LT.0)))) THEN
+              DO ii=1,ntot
+                 IF(bign(1,ii).EQ.-ti.AND.bign(2,ii).EQ.-tj.AND.bign(3,ii).EQ.-tk) THEN
+                    indexplus(ig,inw)=ii
+                    tagp(ig,inw)=1
+                    GO TO 214
+                 END IF
+              END DO
+              indexplus(ig,inw)=-1
+              tagp(ig,inw)=1
+           ELSE
+              DO ii=1,ntot
+                 IF(bign(1,ii).EQ.ti.AND.bign(2,ii).EQ.tj.AND.bign(3,ii).EQ.tk) THEN
+                    indexplus(ig,inw)=ii
+                    tagp(ig,inw)=-1
+                    GO TO 214
+                 END IF
+              END DO
+              indexplus(ig,inw)=-1
+              tagp(ig,inw)=-1
+           END IF
+214        ti=-bign(1,ig)+i_1(inw)
+           tj=-bign(2,ig)+j_1(inw)
+           tk=-bign(3,ig)+k_1(inw)
+           IF(ti.LT.0.OR.(ti.EQ.0.AND.(tj.LT.0.OR.(tj.EQ.0.AND.tk.LT.0)))) THEN
+              DO ii=1,ntot
+                 IF(bign(1,ii).EQ.-ti.AND.bign(2,ii).EQ.-tj.AND.bign(3,ii).EQ.-tk) THEN
+                    indexminus(ig,inw)=ii
+                    tag(ig,inw)=1
+                    GO TO 213
+                 END IF
+              END DO
+              indexminus(ig,inw)=-1
+              tag(ig,inw)=1
+           ELSE 
+              DO ii=1,ntot
+                 IF(bign(1,ii).EQ.ti.AND.bign(2,ii).EQ.tj.AND.bign(3,ii).EQ.tk) THEN
+                    indexminus(ig,inw)=ii
+                    tag(ig,inw)=-1
+                    GO TO 213
+                 END IF
+              END DO
+              indexminus(ig,inw)=-1
+              tag(ig,inw)=-1
+           END IF
+213        CONTINUE
+        END DO
+!$omp end parallel do 
+        WRITE( stdout, * ) "Translation", inw, "for", ntot, "G vectors"
      END IF
   END DO
 
@@ -1304,6 +1270,11 @@ SUBROUTINE wfunc_init( clwf, b1, b2, b3, ibrav )
 
   CALL mp_barrier( intra_bgrp_comm )
   !
+  CALL mp_sum( indexplus,  intra_bgrp_comm )
+  CALL mp_sum( indexminus, intra_bgrp_comm )
+  CALL mp_sum( tag,        intra_bgrp_comm )
+  CALL mp_sum( tagp,       intra_bgrp_comm )
+  !
   CALL mp_bcast( indexplus,  root_bgrp, intra_bgrp_comm )
   CALL mp_bcast( indexminus, root_bgrp, intra_bgrp_comm )
   CALL mp_bcast( tag,        root_bgrp, intra_bgrp_comm )
@@ -1312,11 +1283,10 @@ SUBROUTINE wfunc_init( clwf, b1, b2, b3, ibrav )
 #endif
   DEALLOCATE(bigg)
   DEALLOCATE(bign)
-#ifdef __MPI
-
-#endif
   DEALLOCATE(i_1,j_1,k_1)
 
+  CALL stop_clock('wf_init')
+
   RETURN
 END SUBROUTINE wfunc_init
 !
@@ -1896,7 +1866,7 @@ SUBROUTINE tric_wts2( rp1, rp2, rp3, nw, wfg, weight )
      STOP
   END IF
   !
-  weight(1:nw) = S(:)
+  weight(1:nw) = S(1:nw)
   S=matmul(gp,weight(1:nw))
   !
   DO i=1, nw
@@ -1909,6 +1879,10 @@ SUBROUTINE tric_wts2( rp1, rp2, rp3, nw, wfg, weight )
      t = abs(S(i)-R(i))
      IF ( t .gt. 1.D-8 ) THEN
         WRITE( stdout, * ) "G vectors do not satisfy the completeness condition",i,t
+        !HK
+        WRITE( stdout, * ) "NOTE: If you are in a variable cell calculation, "
+        WRITE( stdout, * ) " please take a look if your ibrav is consistent with the restarted cell"
+        WRITE( stdout, * ) " use ibrav=0 and CELL_PARAMETERS card."
         STOP
      END IF
   END DO
@@ -2436,14 +2410,17 @@ SUBROUTINE wfsteep( m, Omat, Umat, b1, b2, b3 )
   !----------------------------------------------------------------------------
   !
   USE kinds,                  ONLY : DP
-  USE io_global,              ONLY : stdout
+  USE io_global,              ONLY : stdout, ionode
   USE wannier_base,           ONLY : nw, weight, nit, tolw, wfdt, maxwfdt, nsd
-  USE control_flags,          ONLY : iverbosity
+  USE control_flags,          ONLY : iverbosity,conv_elec
   USE cell_base,              ONLY : alat
   USE constants,              ONLY : tpi, bohr_radius_angs
   USE mp_global,              ONLY : me_bgrp
   USE printout_base,          ONLY : printout_base_open, printout_base_unit, &
                                      printout_base_close
+  USE cp_main_variables,      ONLY : nfi, iprint_stdout
+  USE time_step,              ONLY : tps
+  USE input_parameters,         ONLY : tcpbo
   USE parallel_include
   !
   IMPLICIT NONE
@@ -2517,7 +2494,7 @@ SUBROUTINE wfsteep( m, Omat, Umat, b1, b2, b3 )
 
      IF(ABS(oldt1-t01).LT.tolw) THEN 
         IF(me.EQ.1) THEN
-           WRITE(*,*) "MLWF Generated at Step",k ! Lingzhu Kong
+           WRITE(stdout,*) "MLWF Generated at Step",k
         END IF
         IF( iverbosity > 2 ) THEN
            WRITE( stdout, * ) "MLWF Generated at Step",k
@@ -2738,7 +2715,7 @@ SUBROUTINE wfsteep( m, Omat, Umat, b1, b2, b3 )
      U3=ZERO
      IF(ABS(t01-oldt1).GE.tolw.AND.k.GE.nit) THEN
         IF(me.EQ.1) THEN
-           WRITE(*,*) "MLWF Not generated after",k,"Steps." ! Lingzhu Kong
+           WRITE(*,*) "MLWF Not generated after",k,"Steps."
         END IF
         IF( iverbosity > 2 ) THEN
            WRITE( stdout, * ) "MLWF Not generated after",k,"Steps."
@@ -2753,33 +2730,51 @@ SUBROUTINE wfsteep( m, Omat, Umat, b1, b2, b3 )
   !
   ! calculate the spread
   !
-  !  write(24, *) "spread: (unit \AA^2)"
+  !BS .. to print WANNIER spreads at every iprint steps in .spr file ..    
+  IF( ( MOD( nfi, iprint_stdout ) == 0 ) )  THEN
+     IF ( ionode ) THEN
+        iunit = printout_base_unit( "spr" )
+        CALL printout_base_open( "spr" )
+        IF (.NOT.tcpbo) THEN
+           WRITE( iunit, '(I8,f16.8)' )nfi,tps
+        ELSE   
+           IF(conv_elec) WRITE( iunit, '(I8,f16.8)' )nfi,tps
+        END IF    
+     END IF    
+  END IF    
 
-  IF(me.EQ.1) THEN
-     iunit = printout_base_unit( "spr" )
-     CALL printout_base_open( "spr" )
-  END IF
+  spread=0.0_DP
 
   DO i=1, m
      !
      mt=1.D0-DBLE(Oc(:,i,i)*CONJG(Oc(:,i,i)))
      sp = (alat*bohr_radius_angs/tpi)**2*SUM(mt*weight)
      !
-     IF(me.EQ.1) THEN
-        WRITE(iunit, '(f10.7)') sp
+     !BS .. to print WANNIER spreads at every iprint steps in .spr file ..    
+     IF( ( MOD( nfi, iprint_stdout ) == 0 ) ) THEN
+        IF ( ionode ) THEN
+           IF (.NOT.tcpbo) THEN
+              WRITE( iunit, '(f20.14)' ) sp
+           ELSE
+             IF(conv_elec) WRITE( iunit, '(f20.14)' ) sp
+           END IF   
+        END IF   
      END IF
+     !
      IF( sp < 0.D0 ) &
         CALL errore( 'cp-wf', 'Something wrong WF Spread negative', 1 )
      !
      spread=spread+sp
      !
   END DO
-  spread=spread/DBLE(m)
 
-  IF(me.EQ.1) THEN
-     CALL printout_base_open( "spr" )
+  !BS .. to print WANNIER spreads at every iprint steps in .spr file ..    
+  IF( ( MOD( nfi, iprint_stdout ) == 0 ) ) THEN
+     IF ( ionode ) CALL printout_base_close( "spr" )
   END IF
 
+  spread=spread/DBLE(m)
+
   IF(me.EQ.1) THEN
      WRITE(24, '(f10.7)') spread
      WRITE(27,*) "Average spread = ", spread
@@ -3076,7 +3071,7 @@ SUBROUTINE jacobi_rotation( m, Omat, Umat, b1, b2, b3 )
 END SUBROUTINE jacobi_rotation
 !==============================================================================
 
-    SUBROUTINE ddyn_u(nbsp, O, U)
+    SUBROUTINE ddyn_u(nbsp, O, U, iss)
 
 ! input: the overlap matrix O
 ! ouput: the unitary transformation matrix U
@@ -3089,13 +3084,19 @@ END SUBROUTINE jacobi_rotation
        USE cp_main_variables, ONLY: descla
        USE cp_interfaces,     ONLY: distribute_lambda, collect_lambda
        USE printout_base,     ONLY : printout_base_open, printout_base_unit, printout_base_close
+       USE cp_main_variables, ONLY : nfi, iprint_stdout
+       USE time_step,         ONLY : tps
+       USE input_parameters,  ONLY : tcpbo
+       USE control_flags,     ONLY : conv_elec 
        USE parallel_include
+       USE io_global,         ONLY : ionode, stdout
 
        IMPLICIT NONE
    
        INTEGER ,      INTENT(in)    :: nbsp
        REAL(DP),      INTENT(out)   :: U(nbsp,nbsp)
        COMPLEX(DP),   INTENT(inout) :: O(nw,nbsp,nbsp)
+       INTEGER ,      INTENT(in)    :: iss
    
        INTEGER                      :: ista(0:nproc_image-1),iend(0:nproc_image-1)
        REAL(DP),    ALLOCATABLE,  DIMENSION(:,:) :: identy,Um,Up,U0,Ul,W,X2,X3,tmpr2,tmpi2, tmpr,tmpi
@@ -3105,40 +3106,37 @@ END SUBROUTINE jacobi_rotation
        REAL(DP) :: t0, myt0, fric, t2(nw), mt(nw), oldt0,fric1,spread,sp, eps, wfdt2, fricp, fricm
 
        nlam = 1
-       IF( SIZE( descla ) < 2 ) THEN
-          IF( descla(1)%active_node > 0 ) &
-             nlam = descla(1)%nrcx
-       ELSE
-          IF( ( descla(1)%active_node > 0 ) .OR. ( descla(2)%active_node > 0 ) ) &
-             nlam = MAX( descla(1)%nrcx, descla(2)%nrcx )
-       END IF
+       !IF( SIZE( descla ) < 2 ) THEN
+       !   IF( descla(1)%active_node > 0 ) &
+       !      nlam = descla(1)%nrcx
+       !ELSE
+       !   IF( ( descla(1)%active_node > 0 ) .OR. ( descla(2)%active_node > 0 ) ) &
+       !      nlam = MAX( descla(1)%nrcx, descla(2)%nrcx )
+       !END IF
+       IF ( descla(iss)%active_node > 0) &
+           nlam = descla(iss)%nrcx
    
        ALLOCATE(Oc(nbsp,nbsp, nw), Ocold(nbsp,nbsp,nw), Ol(nlam,nlam,nw))
        ALLOCATE(Up(nlam,nlam), U0(nlam,nlam), Um(nlam,nlam), Ul(nlam,nlam),  X2(nbsp,nbsp), X3(nbsp,nbsp))
        ALLOCATE(W(nlam,nlam), identy(nlam,nlam))
        ALLOCATE(tmpr(nlam,nlam), tmpi(nlam,nlam), tmpr2(nlam,nlam), tmpi2(nlam,nlam))
 
-       IF(me_image.EQ.0) THEN
-          iunit = printout_base_unit( "spr" )
-          CALL printout_base_open( "spr" )
-       END IF
-
        eps=1.0E-13_DP
        nmax=50
        fric=wf_friction
        oldt0=0.D0
       
-       nr = descla(1)%nr
-       nc = descla(1)%nc
-       ir = descla(1)%ir
-       ic = descla(1)%ic
+       nr = descla(iss)%nr
+       nc = descla(iss)%nc
+       ir = descla(iss)%ir
+       ic = descla(iss)%ic
 
        do inw = 1, nw
           X2(:,:) =  REAL(O(inw, :, :))
           X3(:,:) = AIMAG(O(inw, :, :))
      
-          call distribute_lambda(X2, tmpr, descla(1))
-          call distribute_lambda(X3, tmpi, descla(1))
+          call distribute_lambda(X2, tmpr, descla(iss))
+          call distribute_lambda(X3, tmpi, descla(iss))
       
           Oc(:,:,inw) = DCMPLX(X2,X3)
           Ol(:,:,inw) = DCMPLX(tmpr,tmpi)
@@ -3150,7 +3148,7 @@ END SUBROUTINE jacobi_rotation
        DO i=1,nbsp
           X2(i,i)=1.D0
        END DO
-       call distribute_lambda(X2, identy, descla(1))
+       call distribute_lambda(X2, identy, descla(iss))
     
        Ul = identy
     
@@ -3184,8 +3182,8 @@ END SUBROUTINE jacobi_rotation
              END DO
           END DO
     
-          CALL ortho_u(Up,U0,nlam,identy,eps,nmax,nbsp)  
-          CALL sqr_mm_cannon( 'N', 'N', nbsp, 1.0d0, Ul, nlam, Up, nlam, 0.0d0, tmpr, nlam, descla(1))  
+          CALL ortho_u(Up,U0,nlam,identy,eps,nmax,nbsp, iss)
+          CALL sqr_mm_cannon( 'N', 'N', nbsp, 1.0d0, Ul, nlam, Up, nlam, 0.0d0, tmpr, nlam, descla(iss))  
           Ul = tmpr
 
           Ocold = Oc
@@ -3193,13 +3191,13 @@ END SUBROUTINE jacobi_rotation
              tmpr(:,:)=REAL(Ol(:,:,inw))
              tmpi(:,:)=AIMAG(Ol(:,:,inw))
    
-             CALL sqr_mm_cannon( 'T', 'N', nbsp, 1.0d0, Ul, nlam, tmpr, nlam, 0.0d0, tmpr2, nlam, descla(1)) 
-             CALL sqr_mm_cannon( 'T', 'N', nbsp, 1.0d0, Ul, nlam, tmpi, nlam, 0.0d0, tmpi2, nlam, descla(1)) 
-             CALL sqr_mm_cannon( 'N', 'N', nbsp, 1.0d0, tmpr2, nlam, Ul, nlam, 0.0d0, tmpr, nlam, descla(1)) 
-             CALL sqr_mm_cannon( 'N', 'N', nbsp, 1.0d0, tmpi2, nlam, Ul, nlam, 0.0d0, tmpi, nlam, descla(1)) 
+             CALL sqr_mm_cannon( 'T', 'N', nbsp, 1.0d0, Ul, nlam, tmpr, nlam, 0.0d0, tmpr2, nlam, descla(iss)) 
+             CALL sqr_mm_cannon( 'T', 'N', nbsp, 1.0d0, Ul, nlam, tmpi, nlam, 0.0d0, tmpi2, nlam, descla(iss)) 
+             CALL sqr_mm_cannon( 'N', 'N', nbsp, 1.0d0, tmpr2, nlam, Ul, nlam, 0.0d0, tmpr, nlam, descla(iss)) 
+             CALL sqr_mm_cannon( 'N', 'N', nbsp, 1.0d0, tmpi2, nlam, Ul, nlam, 0.0d0, tmpi, nlam, descla(iss)) 
    
-             call collect_lambda(X2, tmpr, descla(1) )
-             call collect_lambda(X3, tmpi, descla(1))
+             call collect_lambda(X2, tmpr, descla(iss) )
+             call collect_lambda(X3, tmpi, descla(iss))
    
              Oc(:,:,inw)=CMPLX(X2,X3)
           ENDDO
@@ -3219,12 +3217,14 @@ END SUBROUTINE jacobi_rotation
 #else
        t0 = myt0
 #endif
-       if(mod(ini,10) == 0)print *, 'spread at ', ini, ' = ', t0
+!       if(mod(ini,10) == 0)WRITE(stdout,*), 'spread at ', ini, ' = ', t0
        IF(ABS(t0-oldt0).LT.tolw) THEN
-         WRITE(*,*) "MLWF Generated at Step",ini
+         !WRITE(stdout,*) "MLWF Generated at Step",ini,"  Convergence = ",ABS(t0-oldt0)
+         WRITE(stdout,'(3X,"MLWF step",I5,"  Convergence = ",ES10.3,"  Generated")')ini,ABS(t0-oldt0) ! BS
          exit
        ELSEIF(ini.GE.nsteps) THEN
-         WRITE(*,*) "MLWF Not generated after",ini,"Steps."
+         !WRITE(stdout,*) "MLWF Not generated after",ini,"Steps."
+          WRITE(stdout,'(3X,"MLWF step",I5,"  Convergence = ",ES10.3,"  Not Generated")')ini,ABS(t0-oldt0) ! BS
        END IF
 
        IF(oldt0 .GT. t0) fric=fric/2.D0
@@ -3233,23 +3233,49 @@ END SUBROUTINE jacobi_rotation
 
      END DO !cycl for nsteps
 
+     !BS .. to print WANNIER spreads at every iprint steps in .spr file ..    
+     IF( ( MOD( nfi, iprint_stdout ) == 0 ) )  THEN
+        IF ( ionode ) THEN
+           iunit = printout_base_unit( "spr" )
+           CALL printout_base_open( "spr" )
+           IF (.NOT.tcpbo) THEN
+              WRITE( iunit, '(I8,f16.8)' )nfi,tps
+           ELSE   
+              IF(conv_elec) WRITE( iunit, '(I8,f16.8)' )nfi,tps
+           END IF    
+        END IF    
+     END IF    
+
      spread=0.0d0
      DO i=1, nbsp
         mt=1.D0-DBLE(Oc(i,i,:)*CONJG(Oc(i,i,:)))
         sp = (alat*autoaf/tpi)**2*SUM(mt*weight)
-        IF(me_image.EQ.0) WRITE(iunit, '(f10.7)') sp
-        print *, 'sp = ',i, sp
+        !
+        !BS .. to print WANNIER spreads at every iprint steps in .spr file ..    
+        IF( ( MOD( nfi, iprint_stdout ) == 0 ) ) THEN
+           IF ( ionode ) THEN
+              IF (.NOT.tcpbo) THEN
+                 WRITE( iunit, '(f20.14)' ) sp
+              ELSE
+                IF(conv_elec) WRITE( iunit, '(f20.14)' ) sp
+              END IF   
+           END IF   
+        END IF
+        !
         IF ( sp < 0.D0 ) &
            CALL errore( 'cp-wf', 'Something wrong WF Spread negative', 1 )  
         spread=spread+sp
      END DO
 
-     IF(me_image.EQ.0) CALL printout_base_close( "spr" )
+     !BS .. to print WANNIER spreads at every iprint steps in .spr file ..    
+     IF( ( MOD( nfi, iprint_stdout ) == 0 ) ) THEN
+        IF ( ionode ) CALL printout_base_close( "spr" )
+     END IF
 
      spread=spread/nbsp
-     IF(me_image.EQ.0) write(*,*) "Average spread = ", spread
+    !IF(me_image.EQ.0) write(*,'(3X,"Average spread =",ES10.3)') spread ! BS
 
-     call collect_lambda(U,Ul,descla(1))
+     call collect_lambda(U,Ul,descla(iss))
 
      do inw = 1, nw
         O(inw,:,:) = Oc(:,:,inw)
@@ -3262,7 +3288,7 @@ END SUBROUTINE jacobi_rotation
 
 !-------------------------------------------------------------------------
 
-      SUBROUTINE  ortho_u(up,u0,nlam,identy,eps,nmax,nbsp)
+      SUBROUTINE  ortho_u(up,u0,nlam,identy,eps,nmax,nbsp, iss)
 !-----------------------------------------------------------------------
 !     input = up (non-unitary), u0 (must be unitary)
 !     output = up (unitary),
@@ -3277,7 +3303,7 @@ END SUBROUTINE jacobi_rotation
  
       IMPLICIT NONE
 
-      INTEGER, INTENT(IN)    :: nlam, nmax, nbsp
+      INTEGER, INTENT(IN)    :: nlam, nmax, nbsp, iss
       REAL(DP),INTENT(INOUT) :: up(nlam,nlam)
       REAL(DP),INTENT(IN)    :: u0(nlam,nlam), identy(nlam,nlam),eps
       REAL(DP) :: delta
@@ -3288,11 +3314,11 @@ END SUBROUTINE jacobi_rotation
       ALLOCATE( tmp(nlam,nlam),tmp2(nlam,nlam),tmp2t(nlam,nlam) )
       ALLOCATE( amat(nlam,nlam),bmat(nlam,nlam))
 
-      nr = descla(1)%nr
-      nc = descla(1)%nc
+      nr = descla(iss)%nr
+      nc = descla(iss)%nc
 
-      CALL sqr_mm_cannon( 'T', 'N', nbsp, 1.0d0, up, nlam, up, nlam, 0.0d0, amat, nlam, descla(1))
-      CALL sqr_mm_cannon( 'T', 'N', nbsp, 1.0d0, up, nlam, u0, nlam, 0.0d0, bmat, nlam, descla(1))
+      CALL sqr_mm_cannon( 'T', 'N', nbsp, 1.0d0, up, nlam, up, nlam, 0.0d0, amat, nlam, descla(iss))
+      CALL sqr_mm_cannon( 'T', 'N', nbsp, 1.0d0, up, nlam, u0, nlam, 0.0d0, bmat, nlam, descla(iss))
 
       amat = identy-amat
       bmat = identy-bmat
@@ -3301,10 +3327,10 @@ END SUBROUTINE jacobi_rotation
       delta = 1.0E10_DP
       DO iter = 1,nmax
 
-         CALL sqr_mm_cannon( 'N', 'N', nbsp, 1.0d0, bmat, nlam, xloc, nlam, 0.0d0, tmp2, nlam, descla(1))
-         CALL sqr_mm_cannon( 'T', 'N', nbsp, 1.0d0, xloc, nlam, xloc, nlam, 0.0d0, tmp, nlam, descla(1))
+         CALL sqr_mm_cannon( 'N', 'N', nbsp, 1.0d0, bmat, nlam, xloc, nlam, 0.0d0, tmp2, nlam, descla(iss))
+         CALL sqr_mm_cannon( 'T', 'N', nbsp, 1.0d0, xloc, nlam, xloc, nlam, 0.0d0, tmp, nlam, descla(iss))
 
-         CALL sqr_tr_cannon( nbsp, tmp2, nlam, tmp2t, nlam, descla(1) )
+         CALL sqr_tr_cannon( nbsp, tmp2, nlam, tmp2t, nlam, descla(iss) )
 
          do j=1,nc
             do i=1,nr
@@ -3315,10 +3341,10 @@ END SUBROUTINE jacobi_rotation
          IF(iter .GE. 3) THEN
 
             tmp = up         ! upnew = up + u0*xloc
-            CALL sqr_mm_cannon( 'N', 'N', nbsp, 1.0d0, u0, nlam, xloc, nlam, 1.0d0, tmp, nlam, descla(1))
+            CALL sqr_mm_cannon( 'N', 'N', nbsp, 1.0d0, u0, nlam, xloc, nlam, 1.0d0, tmp, nlam, descla(iss))
 
             tmp2 = identy
-            CALL sqr_mm_cannon( 'T', 'N', nbsp, 1.0d0, tmp, nlam, tmp, nlam, -1.0d0, tmp2, nlam, descla(1))
+            CALL sqr_mm_cannon( 'T', 'N', nbsp, 1.0d0, tmp, nlam, tmp, nlam, -1.0d0, tmp2, nlam, descla(iss))
             delta = 0.d0
             do j=1,nc
             do i=1,nr
