@@ -1,8 +1,8 @@
-# LaTeX2HTML 2002-2-1 (1.71)
+# LaTeX2HTML 2008 (1.71)
 # Associate internals original text with physical files.
 
 
-$key = q/SubSec:Inst/;
+$key = q/SubSec:propedit/;
 $ref_files{$key} = "$dir".q|developer_man.html|; 
 $noresave{$key} = "$nosave";
 
@@ -10,14 +10,26 @@ $key = q/SubSec:Bugs/;
 $ref_files{$key} = "$dir".q|developer_man.html|; 
 $noresave{$key} = "$nosave";
 
+$key = q/SubSec:Inst/;
+$ref_files{$key} = "$dir".q|developer_man.html|; 
+$noresave{$key} = "$nosave";
+
 $key = q/SubSec:CPP/;
 $ref_files{$key} = "$dir".q|developer_man.html|; 
 $noresave{$key} = "$nosave";
 
+$key = q/SubSec:Merge/;
+$ref_files{$key} = "$dir".q|developer_man.html|; 
+$noresave{$key} = "$nosave";
+
 $key = q/Sec:SVN/;
 $ref_files{$key} = "$dir".q|developer_man.html|; 
 $noresave{$key} = "$nosave";
 
+$key = q/SubSec:conf/;
+$ref_files{$key} = "$dir".q|developer_man.html|; 
+$noresave{$key} = "$nosave";
+
 $key = q/SubSec:Conflicts/;
 $ref_files{$key} = "$dir".q|developer_man.html|; 
 $noresave{$key} = "$nosave";
