@@ -1,20 +1,8 @@
-# LaTeX2HTML 2002-2-1 (1.71)
+# LaTeX2HTML 2008 (1.71)
 # Associate labels original text with physical files.
 
 
-$key = q/SubSec:Inst/;
-$external_labels{$key} = "$URL/" . q|developer_man.html|; 
-$noresave{$key} = "$nosave";
-
-$key = q/SubSec:Bugs/;
-$external_labels{$key} = "$URL/" . q|developer_man.html|; 
-$noresave{$key} = "$nosave";
-
-$key = q/SubSec:CPP/;
-$external_labels{$key} = "$URL/" . q|developer_man.html|; 
-$noresave{$key} = "$nosave";
-
-$key = q/Sec:SVN/;
+$key = q/SubSec:Merge/;
 $external_labels{$key} = "$URL/" . q|developer_man.html|; 
 $noresave{$key} = "$nosave";
 
@@ -22,31 +10,71 @@ $key = q/SubSec:Conflicts/;
 $external_labels{$key} = "$URL/" . q|developer_man.html|; 
 $noresave{$key} = "$nosave";
 
+$key = q/SubSec:conf/;
+$external_labels{$key} = "$URL/" . q|developer_man.html|; 
+$noresave{$key} = "$nosave";
+
+$key = q/Sec:SVN/;
+$external_labels{$key} = "$URL/" . q|developer_man.html|; 
+$noresave{$key} = "$nosave";
+
+$key = q/SubSec:propedit/;
+$external_labels{$key} = "$URL/" . q|developer_man.html|; 
+$noresave{$key} = "$nosave";
+
+$key = q/SubSec:CPP/;
+$external_labels{$key} = "$URL/" . q|developer_man.html|; 
+$noresave{$key} = "$nosave";
+
+$key = q/SubSec:Bugs/;
+$external_labels{$key} = "$URL/" . q|developer_man.html|; 
+$noresave{$key} = "$nosave";
+
+$key = q/SubSec:Inst/;
+$external_labels{$key} = "$URL/" . q|developer_man.html|; 
+$noresave{$key} = "$nosave";
+
 1;
 
 
-# LaTeX2HTML 2002-2-1 (1.71)
+# LaTeX2HTML 2008 (1.71)
 # labels from external_latex_labels array.
 
 
-$key = q/SubSec:Inst/;
-$external_latex_labels{$key} = q|3.1|; 
+$key = q/SubSec:Conflicts/;
+$external_latex_labels{$key} = q|11.2|; 
 $noresave{$key} = "$nosave";
 
-$key = q/SubSec:Bugs/;
-$external_latex_labels{$key} = q|2.4|; 
-$noresave{$key} = "$nosave";
-
-$key = q/SubSec:CPP/;
-$external_latex_labels{$key} = q|5.1|; 
+$key = q/SubSec:conf/;
+$external_latex_labels{$key} = q|5.1.2|; 
 $noresave{$key} = "$nosave";
 
 $key = q/Sec:SVN/;
-$external_latex_labels{$key} = q|10|; 
+$external_latex_labels{$key} = q|11|; 
 $noresave{$key} = "$nosave";
 
-$key = q/SubSec:Conflicts/;
-$external_latex_labels{$key} = q|10.2|; 
+$key = q/SubSec:Merge/;
+$external_latex_labels{$key} = q|11.3|; 
+$noresave{$key} = "$nosave";
+
+$key = q/_/;
+$external_latex_labels{$key} = q|<|; 
+$noresave{$key} = "$nosave";
+
+$key = q/SubSec:CPP/;
+$external_latex_labels{$key} = q|5.1.1|; 
+$noresave{$key} = "$nosave";
+
+$key = q/SubSec:Inst/;
+$external_latex_labels{$key} = q|5.1|; 
+$noresave{$key} = "$nosave";
+
+$key = q/SubSec:Bugs/;
+$external_latex_labels{$key} = q|3.5|; 
+$noresave{$key} = "$nosave";
+
+$key = q/SubSec:propedit/;
+$external_latex_labels{$key} = q|11.4|; 
 $noresave{$key} = "$nosave";
 
 1;
