@@ -1,46 +1,54 @@
-# LaTeX2HTML 2002-2-1 (1.71)
+# LaTeX2HTML 2008 (1.71)
 # Associate internals original text with physical files.
 
 
-$key = q/SubSec:para/;
-$ref_files{$key} = "$dir".q|node17.html|; 
+$key = q/SubSec:badpara/;
+$ref_files{$key} = "$dir".q|node19.html|; 
 $noresave{$key} = "$nosave";
 
 $key = q/Sec:Installation/;
 $ref_files{$key} = "$dir".q|node9.html|; 
 $noresave{$key} = "$nosave";
 
-$key = q/SubSec:Contacts/;
-$ref_files{$key} = "$dir".q|node4.html|; 
-$noresave{$key} = "$nosave";
-
-$key = q/SubSec:Terms/;
-$ref_files{$key} = "$dir".q|node6.html|; 
-$noresave{$key} = "$nosave";
-
-$key = q/SubSec:Examples/;
-$ref_files{$key} = "$dir".q|node13.html|; 
-$noresave{$key} = "$nosave";
-
-$key = q/SubSec:Guidelines/;
-$ref_files{$key} = "$dir".q|node5.html|; 
-$noresave{$key} = "$nosave";
-
 $key = q/SubSec:manconf/;
 $ref_files{$key} = "$dir".q|node10.html|; 
 $noresave{$key} = "$nosave";
 
-$key = q/SubSec:LinuxPCMPI/;
-$ref_files{$key} = "$dir".q|node14.html|; 
-$noresave{$key} = "$nosave";
-
-$key = q/Sec:para/;
-$ref_files{$key} = "$dir".q|node15.html|; 
-$noresave{$key} = "$nosave";
-
 $key = q/Sec:Libraries/;
 $ref_files{$key} = "$dir".q|node11.html|; 
 $noresave{$key} = "$nosave";
 
+$key = q/SubSec:Guidelines/;
+$ref_files{$key} = "$dir".q|node5.html|; 
+$noresave{$key} = "$nosave";
+
+$key = q/SubSec:Examples/;
+$ref_files{$key} = "$dir".q|node13.html|; 
+$noresave{$key} = "$nosave";
+
+$key = q/SubSec:Terms/;
+$ref_files{$key} = "$dir".q|node6.html|; 
+$noresave{$key} = "$nosave";
+
+$key = q/SubSec:LinuxPCMPI/;
+$ref_files{$key} = "$dir".q|node14.html|; 
+$noresave{$key} = "$nosave";
+
+$key = q/SubSec:Download/;
+$ref_files{$key} = "$dir".q|node8.html|; 
+$noresave{$key} = "$nosave";
+
+$key = q/Sec:para/;
+$ref_files{$key} = "$dir".q|node15.html|; 
+$noresave{$key} = "$nosave";
+
+$key = q/SubSec:Compilation/;
+$ref_files{$key} = "$dir".q|node12.html|; 
+$noresave{$key} = "$nosave";
+
+$key = q/SubSec:Contacts/;
+$ref_files{$key} = "$dir".q|node4.html|; 
+$noresave{$key} = "$nosave";
+
 1;
 
