@@ -21,19 +21,6 @@ MODULE becmod
   !
   SAVE
   !
-#ifdef __STD_F95
-  TYPE bec_type
-     REAL(DP),   POINTER :: r(:,:)    ! appropriate for gammaonly
-     COMPLEX(DP),POINTER :: k(:,:)    ! appropriate for generic k
-     COMPLEX(DP),POINTER :: nc(:,:,:)   ! appropriate for noncolin
-     INTEGER :: comm
-     INTEGER :: nbnd
-     INTEGER :: nproc
-     INTEGER :: mype
-     INTEGER :: nbnd_loc
-     INTEGER :: ibnd_begin
-  END TYPE bec_type
-#else
   TYPE bec_type
      REAL(DP),   ALLOCATABLE :: r(:,:)    ! appropriate for gammaonly
      COMPLEX(DP),ALLOCATABLE :: k(:,:)    ! appropriate for generic k
@@ -45,7 +32,6 @@ MODULE becmod
      INTEGER :: nbnd_loc
      INTEGER :: ibnd_begin
   END TYPE bec_type
-#endif
   !
   TYPE (bec_type) :: becp  ! <beta|psi>
 
@@ -333,11 +319,7 @@ CONTAINS
     IMPLICIT NONE
     TYPE (bec_type) :: bec
     LOGICAL :: isalloc
-#ifdef __STD_F95
-    isalloc = (associated(bec%r) .or. associated(bec%nc) .or. associated(bec%k))
-#else
     isalloc = (allocated(bec%r) .or. allocated(bec%nc) .or. allocated(bec%k))
-#endif
     RETURN
     !
     !-----------------------------------------------------------------------
@@ -355,12 +337,6 @@ CONTAINS
     INTEGER :: ierr, nbnd_siz
     INTEGER, EXTERNAL :: ldim_block, lind_block, gind_block
     !
-#ifdef __STD_F95
-    NULLIFY(bec%r)
-    NULLIFY(bec%nc)
-    NULLIFY(bec%k)
-#endif
-    !
     nbnd_siz = nbnd
     bec%comm = mp_get_comm_null()
     bec%nbnd = nbnd
@@ -422,15 +398,9 @@ CONTAINS
     bec%comm = mp_get_comm_null()
     bec%nbnd = 0
     !
-#ifdef __STD_F95
-    IF (associated(bec%r))  DEALLOCATE(bec%r)
-    IF (associated(bec%nc)) DEALLOCATE(bec%nc)
-    IF (associated(bec%k))  DEALLOCATE(bec%k)
-#else
     IF (allocated(bec%r))  DEALLOCATE(bec%r)
     IF (allocated(bec%nc)) DEALLOCATE(bec%nc)
     IF (allocated(bec%k))  DEALLOCATE(bec%k)
-#endif
     !
     RETURN
     !
