@@ -188,9 +188,7 @@ CONTAINS
     !
 #if defined(__OPENMP)
     INTEGER, EXTERNAL :: omp_get_max_threads
-#endif
     !
-#if defined(__OPENMP)
     WRITE( stdout, '(/5X,"Parallel version (MPI & OpenMP), running on ",&
          &I7," processor cores")' ) nproc * omp_get_max_threads()
     !
@@ -198,6 +196,11 @@ CONTAINS
     !
     WRITE( stdout, '(5X,"Threads/MPI process:               ",I7)' ) &
          omp_get_max_threads()
+#if defined(__FFTW) || defined(__ESSL)
+#else
+    IF ( omp_get_max_threads() > 1 ) &
+       WRITE( stdout, '(5X,"BEWARE: you are not using multi-threaded FFTs!")')
+#endif
 #else
     WRITE( stdout, '(/5X,"Parallel version (MPI), running on ",&
          &I5," processors")' ) nproc 
