@@ -1,15 +1,13 @@
 !
-! Copyright (C) 2006-2010 Quantum ESPRESSO group
+! Copyright (C) 2006-2015 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
 ! or http://www.gnu.org/copyleft/gpl.txt .
 !
-!
-!
 !----------------------------------------------------------------------
 ! FFT base Module.
-! Written by Carlo Cavazzoni
+! Written by Carlo Cavazzoni, modified by Paolo Giannozzi
 !----------------------------------------------------------------------
 !
 !=----------------------------------------------------------------------=!
@@ -23,6 +21,13 @@
 
         IMPLICIT NONE
 
+        INTERFACE gather_grid
+           MODULE PROCEDURE gather_real_grid, gather_complex_grid
+        END INTERFACE
+        INTERFACE scatter_grid
+           MODULE PROCEDURE scatter_real_grid, scatter_complex_grid
+        END INTERFACE
+
         ! ... data structure containing all information
         ! ... about fft data distribution for a given
         ! ... potential grid, and its wave functions sub-grid.
@@ -44,15 +49,11 @@
 
         PRIVATE
 
-        PUBLIC :: fft_scatter, grid_gather, grid_scatter
         PUBLIC :: dfftp, dffts, dfftb, fft_dlay_descriptor
-        PUBLIC :: cgather_sym, cgather_smooth, cgather_custom
-        PUBLIC :: cscatter_sym, cscatter_smooth, cscatter_custom
-        PUBLIC :: gather_smooth, scatter_smooth
+        PUBLIC :: fft_scatter, gather_grid, scatter_grid
+        PUBLIC :: cgather_sym, cgather_sym_many, cscatter_sym_many
         PUBLIC :: tg_gather, tg_cgather
 
-
-
 !=----------------------------------------------------------------------=!
       CONTAINS
 !=----------------------------------------------------------------------=!
@@ -83,12 +84,15 @@ SUBROUTINE fft_scatter ( dfft, f_in, nr3x, nxx_, f_aux, ncp_, npp_, isgn, use_tg
   !        must be received, all the others must be sent to "proc")
   !    Finally one gets the "planes" representation:
   !    processor "me" has npp_(me) complete xy planes
+  !    f_in  contains input columns, is destroyed on output
+  !    f_aux contains output planes
   !
   !  b) From planes to columns (isgn < 0)
   !
-  !  Quite the same in the opposite direction
+  !    Quite the same in the opposite direction
+  !    f_aux contains input planes, is destroyed on output
+  !    f_in  contains output columns
   !
-  !  The output is overwritten on f_in ; f_aux is used as work space
   !
   !  If optional argument "use_tg" is true the subroutines performs
   !  the trasposition using the Task Groups distribution
@@ -108,7 +112,7 @@ SUBROUTINE fft_scatter ( dfft, f_in, nr3x, nxx_, f_aux, ncp_, npp_, isgn, use_tg
 #ifdef __MPI
 
   INTEGER :: dest, from, k, offset, proc, ierr, me, nprocp, gproc, gcomm, i, kdest, kfrom
-  INTEGER :: me_p, nppx, mc, j, npp, nnp, ii, it, ip, ioff, sendsiz, ncpx
+  INTEGER :: me_p, nppx, mc, j, npp, nnp, ii, it, ip, ioff, sendsiz, ncpx, ipp
   !
   LOGICAL :: use_tg_
 
@@ -166,20 +170,24 @@ SUBROUTINE fft_scatter ( dfft, f_in, nr3x, nxx_, f_aux, ncp_, npp_, isgn, use_tg
      offset = 1
 
      DO proc = 1, nprocp
-        from = offset
         IF( use_tg_ ) THEN
            gproc = dfft%nplist(proc)+1
         ELSE
            gproc = proc
         ENDIF
+        !
+        from = offset
         dest = 1 + ( proc - 1 ) * sendsiz
         !
+        kdest = dest - 1
+        kfrom = from - 1
+        !
         DO k = 1, ncp_ (me)
-           kdest = dest + (k - 1) * nppx - 1
-           kfrom = from + (k - 1) * nr3x - 1
            DO i = 1, npp_ ( gproc )
               f_aux ( kdest + i ) =  f_in ( kfrom + i )
            ENDDO
+           kdest = kdest + nppx
+           kfrom = kfrom + nr3x
         ENDDO
         offset = offset + npp_ ( gproc )
      ENDDO
@@ -197,8 +205,6 @@ SUBROUTINE fft_scatter ( dfft, f_in, nr3x, nxx_, f_aux, ncp_, npp_, isgn, use_tg
         gcomm = dfft%comm
      ENDIF
 
-     ! CALL mpi_barrier (gcomm, ierr)  ! why barrier? for buggy openmpi over ib
-
      CALL mpi_alltoall (f_aux(1), sendsiz, MPI_DOUBLE_COMPLEX, f_in(1), sendsiz, MPI_DOUBLE_COMPLEX, gcomm, ierr)
 
      IF( abs(ierr) /= 0 ) CALL errore ('fft_scatter', 'info<>0', abs(ierr) )
@@ -209,23 +215,18 @@ SUBROUTINE fft_scatter ( dfft, f_in, nr3x, nxx_, f_aux, ncp_, npp_, isgn, use_tg
      !
      IF( isgn == 1 ) THEN
 
-!!$omp parallel default(none) private(ip,ioff,i,mc,it,j) shared(dfft,nppx,sendsiz,me,f_in,f_aux)
-!!$omp do
-
         DO ip = 1, dfft%nproc
            ioff = dfft%iss( ip )
+           it = ( ip - 1 ) * sendsiz
            DO i = 1, dfft%nsp( ip )
               mc = dfft%ismap( i + ioff )
-              it = ( i - 1 ) * nppx + ( ip - 1 ) * sendsiz
               DO j = 1, dfft%npp( me )
                  f_aux( mc + ( j - 1 ) * dfft%nnp ) = f_in( j + it )
               ENDDO
+              it = it + nppx
            ENDDO
         ENDDO
 
-!!$omp end do
-!!$omp end parallel
-
      ELSE
 
         IF( use_tg_ ) THEN
@@ -236,37 +237,34 @@ SUBROUTINE fft_scatter ( dfft, f_in, nr3x, nxx_, f_aux, ncp_, npp_, isgn, use_tg
            nnp  = dfft%nnp
         ENDIF
         !
-!!$omp parallel default(none) private(ip,ioff,i,mc,it,j,gproc,ii) shared(dfft,nppx,npp,nnp,sendsiz,use_tg_,f_in,f_aux)
-!!$omp do
-        DO ip = 1, dfft%nproc
-
-           IF( use_tg_ ) THEN
-              gproc = ( ip - 1 ) / dfft%nogrp + 1
-              IF( MOD( ip - 1, dfft%nogrp ) == 0 ) ii = 0
-           ELSE
-              gproc = ip
-              ii = 0
-           ENDIF
+        ip = 1
+        !
+        DO gproc = 1, dfft%nproc / dfft%nogrp
            !
-           ioff = dfft%iss( ip )
+           it = ( gproc - 1 ) * sendsiz
            !
-           DO i = 1, dfft%nsw( ip )
+           DO ipp = 1, dfft%nogrp 
               !
-              mc = dfft%ismap( i + ioff )
+              ioff = dfft%iss( ip ) + 1
               !
-              it = ii * nppx + ( gproc - 1 ) * sendsiz
-              !
-              DO j = 1, npp
-                 f_aux( mc + ( j - 1 ) * nnp ) = f_in( j + it )
+              DO i = 1, dfft%nsw( ip )
+                 !
+                 mc = dfft%ismap( ioff )
+                 !
+                 DO j = 1, npp
+                    f_aux( mc + ( j - 1 ) * nnp ) = f_in( j + it )
+                 ENDDO
+                 !
+                 it = it  + nppx
+                 ioff = ioff + 1
+                 !
               ENDDO
               !
-              ii = ii + 1
+              ip = ip + 1
               !
            ENDDO
            !
         ENDDO
-!!$omp end do
-!!$omp end parallel
 
      END IF
 
@@ -278,12 +276,13 @@ SUBROUTINE fft_scatter ( dfft, f_in, nr3x, nxx_, f_aux, ncp_, npp_, isgn, use_tg
 
         DO ip = 1, dfft%nproc
            ioff = dfft%iss( ip )
+           it = ( ip - 1 ) * sendsiz
            DO i = 1, dfft%nsp( ip )
               mc = dfft%ismap( i + ioff )
-              it = ( i - 1 ) * nppx + ( ip - 1 ) * sendsiz
               DO j = 1, dfft%npp( me )
                  f_in( j + it ) = f_aux( mc + ( j - 1 ) * dfft%nnp )
               ENDDO
+              it = it + nppx
            ENDDO
         ENDDO
 
@@ -296,30 +295,31 @@ SUBROUTINE fft_scatter ( dfft, f_in, nr3x, nxx_, f_aux, ncp_, npp_, isgn, use_tg
            npp  = dfft%npp( me )
            nnp  = dfft%nnp
         ENDIF
-
-        DO ip = 1, dfft%nproc
-
-           IF( use_tg_ ) THEN
-              gproc = ( ip - 1 ) / dfft%nogrp + 1
-              IF( MOD( ip - 1, dfft%nogrp ) == 0 ) ii = 0
-           ELSE
-              gproc = ip
-              ii = 0
-           ENDIF
+        !
+        ip = 1
+        !
+        DO gproc = 1, dfft%nproc / dfft%nogrp
            !
-           ioff = dfft%iss( ip )
+           it = ( gproc - 1 ) * sendsiz
            !
-           DO i = 1, dfft%nsw( ip )
+           DO ipp = 1, dfft%nogrp 
               !
-              mc = dfft%ismap( i + ioff )
+              ioff = dfft%iss( ip ) + 1
               !
-              it = ii * nppx + ( gproc - 1 ) * sendsiz
-              !
-              DO j = 1, npp
-                 f_in( j + it ) = f_aux( mc + ( j - 1 ) * nnp )
+              DO i = 1, dfft%nsw( ip )
+                 !
+                 mc = dfft%ismap( ioff )
+                 !
+                 DO j = 1, npp
+                    f_in( j + it ) = f_aux( mc + ( j - 1 ) * nnp )
+                 ENDDO
+                 !
+                 it = it + nppx
+                 ioff = ioff + 1
+                 !
               ENDDO
               !
-              ii = ii + 1
+              ip = ip + 1
               !
            ENDDO
 
@@ -358,12 +358,15 @@ SUBROUTINE fft_scatter ( dfft, f_in, nr3x, nxx_, f_aux, ncp_, npp_, isgn, use_tg
         ENDIF
         dest = 1 + ( proc - 1 ) * sendsiz
         !
+        kdest = dest - 1
+        kfrom = from - 1
+        !
         DO k = 1, ncp_ (me)
-           kdest = dest + (k - 1) * nppx - 1
-           kfrom = from + (k - 1) * nr3x - 1
            DO i = 1, npp_ ( gproc )  
               f_in ( kfrom + i ) = f_aux ( kdest + i )
            ENDDO
+           kdest = kdest + nppx
+           kfrom = kfrom + nr3x
         ENDDO
         offset = offset + npp_ ( gproc )
      ENDDO
@@ -383,140 +386,275 @@ SUBROUTINE fft_scatter ( dfft, f_in, nr3x, nxx_, f_aux, ncp_, npp_, isgn, use_tg
   RETURN
 
 END SUBROUTINE fft_scatter
-
-
+!
 !----------------------------------------------------------------------------
-SUBROUTINE grid_gather( f_in, f_out )
+SUBROUTINE gather_real_grid ( dfft, f_in, f_out )
   !----------------------------------------------------------------------------
   !
-  ! ... gathers nproc distributed data on the first processor of every pool
+  ! ... gathers a distributed real-space FFT grid to dfft%root, that is,
+  ! ... the first processor of input descriptor dfft - version for real arrays
   !
-  ! ... REAL*8  f_in  = distributed variable (nxx)
-  ! ... REAL*8  f_out = gathered variable (nr1x*nr2x*nr3x)
+  ! ... REAL*8  f_in  = distributed variable (dfft%nnr)
+  ! ... REAL*8  f_out = gathered variable (dfft%nr1x*dfft%nr2x*dfft%nr3x)
   !
   USE kinds,     ONLY : DP
   USE parallel_include
   !
   IMPLICIT NONE
   !
-  REAL(DP) :: f_in( : ), f_out( : )
+  REAL(DP), INTENT(in) :: f_in (:)
+  REAL(DP), INTENT(inout):: f_out(:)
+  TYPE ( fft_dlay_descriptor ), INTENT(IN) :: dfft
   !
 #if defined (__MPI)
   !
   INTEGER :: proc, info
-  INTEGER :: displs(0:dfftp%nproc-1), recvcount(0:dfftp%nproc-1)
+  ! ... the following are automatic arrays
+  INTEGER :: displs(0:dfft%nproc-1), recvcount(0:dfft%nproc-1)
   !
-  IF( size( f_in ) < dfftp%nnr ) &
-     CALL errore( ' grid_gather ', ' f_in too small ', dfftp%nnr - size( f_in ) )
+  IF( size( f_in ) < dfft%nnr ) &
+     CALL errore( ' gather_grid ', ' f_in too small ', dfftp%nnr-size( f_in ) )
   !
-  CALL start_clock( 'gather' )
+  CALL start_clock( 'gather_grid' )
   !
-  DO proc = 0, ( dfftp%nproc - 1 )
-     !
-     recvcount(proc) = dfftp%nnp * dfftp%npp(proc+1)
+  DO proc = 0, ( dfft%nproc - 1 )
      !
+     recvcount(proc) = dfft%nnp * dfft%npp(proc+1)
      IF ( proc == 0 ) THEN
-        !
         displs(proc) = 0
-        !
      ELSE
-        !
         displs(proc) = displs(proc-1) + recvcount(proc-1)
-        !
      ENDIF
      !
   ENDDO
   !
-  info = size( f_out ) - displs( dfftp%nproc - 1 ) - recvcount( dfftp%nproc - 1 )
+  ! ... the following check should be performed only on processor dfft%root
+  ! ... otherwise f_out must be allocated on all processors even if not used
   !
+  info = size( f_out ) - displs( dfft%nproc-1 ) - recvcount( dfft%nproc-1 )
   IF( info < 0 ) &
-     CALL errore( ' grid_gather ', ' f_out too small ', -info )
+     CALL errore( ' gather_grid ', ' f_out too small ', -info )
   !
   info = 0
   !
-  CALL MPI_GATHERV( f_in, recvcount(dfftp%mype), MPI_DOUBLE_PRECISION, f_out, &
-                    recvcount, displs, MPI_DOUBLE_PRECISION, dfftp%root,    &
-                    dfftp%comm, info )
+  CALL MPI_GATHERV( f_in, recvcount(dfft%mype), MPI_DOUBLE_PRECISION, f_out, &
+                    recvcount, displs, MPI_DOUBLE_PRECISION, dfft%root,      &
+                    dfft%comm, info )
   !
-  CALL errore( 'grid_gather', 'info<>0', info )
+  CALL errore( 'gather_grid', 'info<>0', info )
   !
-  CALL stop_clock( 'gather' )
+  CALL stop_clock( 'gather_grid' )
   !
 #else
-  CALL errore('grid_gather', 'do not use in serial execution', 1)
+  CALL errore('gather_grid', 'do not use in serial execution', 1)
 #endif
   !
   RETURN
   !
-END SUBROUTINE grid_gather
-
+END SUBROUTINE gather_real_grid
 
 !----------------------------------------------------------------------------
-SUBROUTINE grid_scatter( f_in, f_out )
+SUBROUTINE gather_complex_grid ( dfft, f_in, f_out )
   !----------------------------------------------------------------------------
   !
-  ! ... scatters data from the first processor of every pool
+  ! ... gathers a distributed real-space FFT grid to dfft%root, that is,
+  ! ... the first processor of input descriptor dfft - complex arrays
   !
-  ! ... REAL*8  f_in  = gathered variable (nr1x*nr2x*nr3x)
-  ! ... REAL*8  f_out = distributed variable (nxx)
+  ! ... COMPLEX*16  f_in  = distributed variable (dfft%nnr)
+  ! ... COMPLEX*16  f_out = gathered variable (dfft%nr1x*dfft%nr2x*dfft%nr3x)
   !
   USE kinds,     ONLY : DP
   USE parallel_include
   !
   IMPLICIT NONE
   !
-  REAL(DP) :: f_in( : ), f_out( : )
+  COMPLEX(DP), INTENT(in) :: f_in (:)
+  COMPLEX(DP), INTENT(inout):: f_out(:)
+  TYPE ( fft_dlay_descriptor ), INTENT(IN) :: dfft
   !
 #if defined (__MPI)
   !
   INTEGER :: proc, info
-  INTEGER :: displs(0:dfftp%nproc-1), sendcount(0:dfftp%nproc-1)
+  ! ... the following are automatic arrays
+  INTEGER :: displs(0:dfft%nproc-1), recvcount(0:dfft%nproc-1)
   !
-  IF( size( f_out ) < dfftp%nnr ) &
-     CALL errore( ' grid_scatter ', ' f_out too small ', dfftp%nnr - size( f_in ) )
+  IF( 2*size( f_in ) < dfft%nnr ) &
+     CALL errore( ' gather_grid ', ' f_in too small ', dfftp%nnr-size( f_in ) )
   !
-  CALL start_clock( 'scatter' )
+  CALL start_clock( 'gather_grid' )
   !
-  DO proc = 0, ( dfftp%nproc - 1 )
-     !
-     sendcount(proc) = dfftp%nnp * dfftp%npp(proc+1)
+  DO proc = 0, ( dfft%nproc - 1 )
      !
+     recvcount(proc) = 2*dfft%nnp * dfft%npp(proc+1)
      IF ( proc == 0 ) THEN
-        !
         displs(proc) = 0
-        !
      ELSE
-        !
-        displs(proc) = displs(proc-1) + sendcount(proc-1)
-        !
+        displs(proc) = displs(proc-1) + recvcount(proc-1)
      ENDIF
      !
   ENDDO
   !
-  info = size( f_in ) - displs( dfftp%nproc - 1 ) - sendcount( dfftp%nproc - 1 )
+  ! ... the following check should be performed only on processor dfft%root
+  ! ... otherwise f_out must be allocated on all processors even if not used
   !
+  info = 2*size( f_out ) - displs( dfft%nproc - 1 ) - recvcount( dfft%nproc-1 )
   IF( info < 0 ) &
-     CALL errore( ' grid_scatter ', ' f_in too small ', -info )
+     CALL errore( ' gather_grid ', ' f_out too small ', -info )
+  !
+  info = 0
+  !
+  CALL MPI_GATHERV( f_in, recvcount(dfft%mype), MPI_DOUBLE_PRECISION, f_out, &
+                    recvcount, displs, MPI_DOUBLE_PRECISION, dfft%root,      &
+                    dfft%comm, info )
+  !
+  CALL errore( 'gather_grid', 'info<>0', info )
+  !
+  CALL stop_clock( 'gather_grid' )
+  !
+#else
+  CALL errore('gather_grid', 'do not use in serial execution', 1)
+#endif
+  !
+  RETURN
+  !
+END SUBROUTINE gather_complex_grid
+
+!----------------------------------------------------------------------------
+SUBROUTINE scatter_real_grid ( dfft, f_in, f_out )
+  !----------------------------------------------------------------------------
+  !
+  ! ... scatters a real-space FFT grid from dfft%root, first processor of
+  ! ... input descriptor dfft, to all others - opposite of "gather_grid"
+  !
+  ! ... REAL*8  f_in  = gathered variable (dfft%nr1x*dfft%nr2x*dfft%nr3x)
+  ! ... REAL*8  f_out = distributed variable (dfft%nnr)
+  !
+  USE kinds,     ONLY : DP
+  USE parallel_include
+  !
+  IMPLICIT NONE
+  !
+  REAL(DP), INTENT(in) :: f_in (:)
+  REAL(DP), INTENT(inout):: f_out(:)
+  TYPE ( fft_dlay_descriptor ), INTENT(IN) :: dfft
+  !
+#if defined (__MPI)
+  !
+  INTEGER :: proc, info
+  ! ... the following are automatic arrays
+  INTEGER :: displs(0:dfft%nproc-1), sendcount(0:dfft%nproc-1)
+  !
+  IF( size( f_out ) < dfft%nnr ) &
+     CALL errore( ' scatter_grid ', ' f_out too small ', dfft%nnr-size( f_in ) )
+  !
+  CALL start_clock( 'scatter_grid' )
+  !
+  DO proc = 0, ( dfft%nproc - 1 )
+     !
+     sendcount(proc) = dfft%nnp * dfft%npp(proc+1)
+     IF ( proc == 0 ) THEN
+        displs(proc) = 0
+     ELSE
+        displs(proc) = displs(proc-1) + sendcount(proc-1)
+     ENDIF
+     !
+  ENDDO
+  !
+  ! ... the following check should be performed only on processor dfft%root
+  ! ... otherwise f_in must be allocated on all processors even if not used
+  !
+  info = size( f_in ) - displs( dfft%nproc - 1 ) - sendcount( dfft%nproc - 1 )
+  IF( info < 0 ) &
+     CALL errore( ' scatter_grid ', ' f_in too small ', -info )
   !
   info = 0
   !
   CALL MPI_SCATTERV( f_in, sendcount, displs, MPI_DOUBLE_PRECISION,   &
-                     f_out, sendcount(dfftp%mype), MPI_DOUBLE_PRECISION, &
-                     dfftp%root, dfftp%comm, info )
+                     f_out, sendcount(dfft%mype), MPI_DOUBLE_PRECISION, &
+                     dfft%root, dfft%comm, info )
   !
-  CALL errore( 'grid_scatter', 'info<>0', info )
+  CALL errore( 'scatter_grid', 'info<>0', info )
   !
-  IF ( sendcount(dfftp%mype) /= dfftp%nnr ) f_out(sendcount(dfftp%mype)+1:dfftp%nnr) = 0.D0
+  IF ( sendcount(dfft%mype) /= dfft%nnr ) &
+     f_out(sendcount(dfft%mype)+1:dfft%nnr) = 0.D0
   !
-  CALL stop_clock( 'scatter' )
+  CALL stop_clock( 'scatter_grid' )
   !
 #else
-  CALL errore('grid_scatter', 'do not use in serial execution', 1)
+  CALL errore('scatter_grid', 'do not use in serial execution', 1)
 #endif
   !
   RETURN
   !
-END SUBROUTINE grid_scatter
+END SUBROUTINE scatter_real_grid
+!----------------------------------------------------------------------------
+SUBROUTINE scatter_complex_grid ( dfft, f_in, f_out )
+  !----------------------------------------------------------------------------
+  !
+  ! ... scatters a real-space FFT grid from dfft%root, first processor of
+  ! ... input descriptor dfft, to all others - opposite of "gather_grid"
+  !
+  ! ... COMPLEX*16  f_in  = gathered variable (dfft%nr1x*dfft%nr2x*dfft%nr3x)
+  ! ... COMPLEX*16  f_out = distributed variable (dfft%nnr)
+  !
+  USE kinds,     ONLY : DP
+  USE parallel_include
+  !
+  IMPLICIT NONE
+  !
+  COMPLEX(DP), INTENT(in) :: f_in (:)
+  COMPLEX(DP), INTENT(inout):: f_out(:)
+  TYPE ( fft_dlay_descriptor ), INTENT(IN) :: dfft
+  !
+#if defined (__MPI)
+  !
+  INTEGER :: proc, info
+  ! ... the following are automatic arrays
+  INTEGER :: displs(0:dfft%nproc-1), sendcount(0:dfft%nproc-1)
+  !
+  IF( 2*size( f_out ) < dfft%nnr ) &
+     CALL errore( ' scatter_grid ', ' f_out too small ', dfft%nnr-size( f_in ) )
+  !
+  CALL start_clock( 'scatter_grid' )
+  !
+  DO proc = 0, ( dfft%nproc - 1 )
+     !
+     sendcount(proc) = 2*dfft%nnp * dfft%npp(proc+1)
+     IF ( proc == 0 ) THEN
+        displs(proc) = 0
+     ELSE
+        displs(proc) = displs(proc-1) + sendcount(proc-1)
+     ENDIF
+     !
+  ENDDO
+  !
+  ! ... the following check should be performed only on processor dfft%root
+  ! ... otherwise f_in must be allocated on all processors even if not used
+  !
+  info = 2*size( f_in ) - displs( dfft%nproc - 1 ) - sendcount( dfft%nproc - 1 )
+  IF( info < 0 ) &
+     CALL errore( ' scatter_grid ', ' f_in too small ', -info )
+  !
+  info = 0
+  !
+  CALL MPI_SCATTERV( f_in, sendcount, displs, MPI_DOUBLE_PRECISION,   &
+                     f_out, sendcount(dfft%mype), MPI_DOUBLE_PRECISION, &
+                     dfft%root, dfft%comm, info )
+  !
+  CALL errore( 'scatter_grid', 'info<>0', info )
+  !
+  IF ( sendcount(dfft%mype) /= dfft%nnr ) &
+     f_out(sendcount(dfft%mype)+1:dfft%nnr) = 0.D0
+  !
+  CALL stop_clock( 'scatter_grid' )
+  !
+#else
+  CALL errore('scatter_grid', 'do not use in serial execution', 1)
+#endif
+  !
+  RETURN
+  !
+END SUBROUTINE scatter_complex_grid
 !
 ! ... "gather"-like subroutines
 !
@@ -524,7 +662,9 @@ END SUBROUTINE grid_scatter
 SUBROUTINE cgather_sym( f_in, f_out )
   !-----------------------------------------------------------------------
   !
-  ! ... gather complex data for symmetrization (in phonon code)
+  ! ... gather complex data for symmetrization (used in phonon code)
+  ! ... Differs from gather_grid because mpi_allgatherv is used instead
+  ! ... of mpi_gatherv - all data is gathered on ALL processors
   ! ... COMPLEX*16  f_in  = distributed variable (nrxx)
   ! ... COMPLEX*16  f_out = gathered variable (nr1x*nr2x*nr3x)
   !
@@ -546,15 +686,10 @@ SUBROUTINE cgather_sym( f_in, f_out )
   DO proc = 0, ( dfftp%nproc - 1 )
      !
      recvcount(proc) = 2 * dfftp%nnp * dfftp%npp(proc+1)
-     !
      IF ( proc == 0 ) THEN
-        !
         displs(proc) = 0
-        !
      ELSE
-        !
         displs(proc) = displs(proc-1) + recvcount(proc-1)
-        !
      ENDIF
      !
   ENDDO
@@ -567,8 +702,6 @@ SUBROUTINE cgather_sym( f_in, f_out )
   !
   CALL errore( 'cgather_sym', 'info<>0', info )
   !
-!  CALL mp_barrier( dfftp%comm )
-  !
   CALL stop_clock( 'cgather' )
   !
 #else
@@ -578,442 +711,6 @@ SUBROUTINE cgather_sym( f_in, f_out )
   RETURN
   !
 END SUBROUTINE cgather_sym
-!
-!----------------------------------------------------------------------------
-SUBROUTINE cgather_smooth ( f_in, f_out )
-  !----------------------------------------------------------------------------
-  !
-  ! ... gathers data on the smooth AND complex fft grid
-  !
-  ! ... gathers nproc distributed data on the first processor of every pool
-  !
-  ! ... COMPLEX*16  f_in  = distributed variable ( dffts%nnr )
-  ! ... COMPLEX*16  f_out = gathered variable (nr1sx*nr2sx*nr3sx)
-  !
-  USE mp,        ONLY : mp_barrier
-  USE kinds,     ONLY : DP
-  USE parallel_include
-  !
-  IMPLICIT NONE
-  !
-  COMPLEX(DP) :: f_in(:), f_out(:)
-  !
-#if defined (__MPI)
-  !
-  INTEGER :: proc, info
-  INTEGER :: displs(0:dfftp%nproc-1), recvcount(0:dfftp%nproc-1)
-  !
-  !
-  CALL start_clock( 'gather' )
-  !
-  DO proc = 0, ( dfftp%nproc - 1 )
-     !
-     recvcount(proc) = 2 * dffts%nnp * dffts%npp(proc+1)
-     !
-     IF ( proc == 0 ) THEN
-        !
-        displs(proc) = 0
-        !
-     ELSE
-        !
-        displs(proc) = displs(proc-1) + recvcount(proc-1)
-        !
-     ENDIF
-     !
-  ENDDO
-  !
-  CALL mp_barrier( dfftp%comm )
-  !
-  CALL MPI_GATHERV( f_in, recvcount(dfftp%mype), MPI_DOUBLE_PRECISION, f_out, &
-                    recvcount, displs, MPI_DOUBLE_PRECISION, dfftp%root,    &
-                    dfftp%comm, info )
-  !
-  CALL errore( 'cgather_smooth', 'info<>0', info )
-  !
-  CALL stop_clock( 'gather' )
-  !
-#else
-  CALL errore('cgather_smooth', 'do not use in serial execution', 1)
-#endif
-  !
-  RETURN
-  !
-END SUBROUTINE cgather_smooth
-!
-!----------------------------------------------------------------------------
-SUBROUTINE cgather_custom ( f_in, f_out, dfftt )
-  !----------------------------------------------------------------------------
-  !
-  ! ... gathers data on the custom AND complex fft grid
-  !
-  ! ... gathers nproc distributed data on the first processor of every pool
-  !
-  ! ... COMPLEX*16  f_in  = distributed variable ( dfftt%nnr )
-  ! ... COMPLEX*16  f_out = gathered variable (nr1sx*nr2sx*nr3sx)
-  !
-  USE mp,        ONLY : mp_barrier
-  USE kinds,     ONLY : DP
-  USE parallel_include
-  !
-  IMPLICIT NONE
-  !
-  COMPLEX(DP) :: f_in(:), f_out(:)
-  TYPE ( fft_dlay_descriptor ), INTENT(IN) :: dfftt 
-  !
-#if defined (__MPI)
-  !
-  INTEGER :: proc, info
-  INTEGER :: displs(0:dfftp%nproc-1), recvcount(0:dfftp%nproc-1)
-  !
-  !
-  CALL start_clock( 'gather' )
-  !
-  DO proc = 0, ( dfftp%nproc - 1 )
-     !
-     recvcount(proc) = 2 * dfftt%nnp * dfftt%npp(proc+1)
-     !
-     IF ( proc == 0 ) THEN
-        !
-        displs(proc) = 0
-        !
-     ELSE
-        !
-        displs(proc) = displs(proc-1) + recvcount(proc-1)
-        !
-     ENDIF
-     !
-  ENDDO
-  !
-  CALL mp_barrier( dfftp%comm )
-  !
-  CALL MPI_GATHERV( f_in, recvcount(dfftp%mype), MPI_DOUBLE_PRECISION, f_out, &
-                    recvcount, displs, MPI_DOUBLE_PRECISION, dfftp%root,    &
-                    dfftp%comm, info )
-  !
-  CALL errore( 'cgather_custom', 'info<>0', info )
-  !
-  CALL stop_clock( 'gather' )
-  !
-#else
-  CALL errore('cgather_custom', 'do not use in serial execution', 1)
-#endif
-  !
-  RETURN
-  !
-END SUBROUTINE cgather_custom
-!
-! ... "scatter"-like subroutines
-!
-!----------------------------------------------------------------------------
-SUBROUTINE cscatter_sym( f_in, f_out )
-  !----------------------------------------------------------------------------
-  !
-  ! ... scatters data from the first processor of every pool
-  !
-  ! ... COMPLEX*16  f_in  = gathered variable (nr1x*nr2x*nr3x)
-  ! ... COMPLEX*16  f_out = distributed variable (nxx)
-  !
-  USE mp,        ONLY : mp_barrier
-  USE kinds,     ONLY : DP
-  USE parallel_include
-  !
-  IMPLICIT NONE
-  !
-  COMPLEX(DP) :: f_in(:), f_out(:)
-  !
-#if defined (__MPI)
-  !
-  INTEGER :: proc, info
-  INTEGER :: displs(0:dfftp%nproc-1), sendcount(0:dfftp%nproc-1)
-  !
-  !
-  CALL start_clock( 'cscatter_sym' )
-  !
-  DO proc = 0, ( dfftp%nproc - 1 )
-     !
-     sendcount(proc) = 2 * dfftp%nnp * dfftp%npp(proc+1)
-     !
-     IF ( proc == 0 ) THEN
-        !
-        displs(proc) = 0
-        !
-     ELSE
-        !
-        displs(proc) = displs(proc-1) + sendcount(proc-1)
-        !
-     ENDIF
-     !
-  ENDDO
-  !
-  CALL mp_barrier( dfftp%comm )
-  !
-  CALL MPI_SCATTERV( f_in, sendcount, displs, MPI_DOUBLE_PRECISION,   &
-                     f_out, sendcount(dfftp%mype), MPI_DOUBLE_PRECISION, &
-                     dfftp%root, dfftp%comm, info )
-  !
-  CALL errore( 'cscatter_sym', 'info<>0', info )
-  !
-  IF ( sendcount(dfftp%mype) /=  dfftp%nnr  ) f_out(sendcount(dfftp%mype)+1: dfftp%nnr ) = 0.D0
-  !
-  CALL stop_clock( 'cscatter_sym' )
-  !
-#else
-  CALL errore('cscatter_sym', 'do not use in serial execution', 1)
-#endif
-  !
-  RETURN
-  !
-END SUBROUTINE cscatter_sym
-!
-!----------------------------------------------------------------------------
-SUBROUTINE cscatter_smooth( f_in, f_out )
-  !----------------------------------------------------------------------------
-  !
-  ! ... scatters data on the smooth AND complex fft grid
-  ! ... scatters data from the first processor of every pool
-  !
-  ! ... COMPLEX*16  f_in  = gathered variable (nr1sx*nr2sx*nr3sx)
-  ! ... COMPLEX*16  f_out = distributed variable ( dffts%nnr)
-  !
-  USE mp,        ONLY : mp_barrier
-  USE kinds,     ONLY : DP
-  USE parallel_include
-  !
-  IMPLICIT NONE
-  !
-  COMPLEX(DP) :: f_in(:), f_out(:)
-  !
-#if defined (__MPI)
-  !
-  INTEGER :: proc, info
-  INTEGER :: displs(0:dfftp%nproc-1), sendcount(0:dfftp%nproc-1)
-  !
-  !
-  CALL start_clock( 'scatter' )
-  !
-  DO proc = 0, ( dfftp%nproc - 1 )
-     !
-     sendcount(proc) = 2 * dffts%nnp * dffts%npp(proc+1)
-     !
-     IF ( proc == 0 ) THEN
-        !
-        displs(proc) = 0
-        !
-     ELSE
-        !
-        displs(proc) = displs(proc-1) + sendcount(proc-1)
-        !
-     ENDIF
-     !
-  ENDDO
-  !
-  CALL mp_barrier( dfftp%comm )
-  !
-  CALL MPI_SCATTERV( f_in, sendcount, displs, MPI_DOUBLE_PRECISION,   &
-                     f_out, sendcount(dfftp%mype), MPI_DOUBLE_PRECISION, &
-                     dfftp%root, dfftp%comm, info )
-  !
-  CALL errore( 'scatter', 'info<>0', info )
-  !
-  IF ( sendcount(dfftp%mype) /=  dffts%nnr  ) f_out(sendcount(dfftp%mype)+1: dffts%nnr ) = 0.D0
-  !
-  CALL stop_clock( 'scatter' )
-  !
-#else
-  CALL errore('cscatter_smooth', 'do not use in serial execution', 1)
-#endif
-  !
-  RETURN
-  !
-END SUBROUTINE cscatter_smooth
-!
-!----------------------------------------------------------------------------
-SUBROUTINE cscatter_custom( f_in, f_out, dfftt )
-  !----------------------------------------------------------------------------
-  !
-  ! ... scatters data on the custom AND complex fft grid
-  ! ... scatters data from the first processor of every pool
-  !
-  ! ... COMPLEX*16  f_in  = gathered variable (nr1sx*nr2sx*nr3sx)
-  ! ... COMPLEX*16  f_out = distributed variable ( dfftt%nnr)
-  !
-  USE mp,        ONLY : mp_barrier
-  USE kinds,     ONLY : DP
-  USE parallel_include
-  !
-  IMPLICIT NONE
-  !
-  COMPLEX(DP) :: f_in(:), f_out(:)
-  TYPE ( fft_dlay_descriptor ), INTENT(IN) :: dfftt 
-  !
-#if defined (__MPI)
-  !
-  INTEGER :: proc, info
-  INTEGER :: displs(0:dfftp%nproc-1), sendcount(0:dfftp%nproc-1)
-  !
-  !
-  CALL start_clock( 'scatter' )
-  !
-  DO proc = 0, ( dfftp%nproc - 1 )
-     !
-     sendcount(proc) = 2 * dfftt%nnp * dfftt%npp(proc+1)
-     !
-     IF ( proc == 0 ) THEN
-        !
-        displs(proc) = 0
-        !
-     ELSE
-        !
-        displs(proc) = displs(proc-1) + sendcount(proc-1)
-        !
-     ENDIF
-     !
-  ENDDO
-  !
-  CALL mp_barrier( dfftp%comm )
-  !
-  CALL MPI_SCATTERV( f_in, sendcount, displs, MPI_DOUBLE_PRECISION,   &
-                     f_out, sendcount(dfftp%mype), MPI_DOUBLE_PRECISION, &
-                     dfftp%root, dfftp%comm, info )
-  !
-  CALL errore( 'scatter', 'info<>0', info )
-  !
-  IF ( sendcount(dfftp%mype) /=  dfftt%nnr  ) f_out(sendcount(dfftp%mype)+1: dfftt%nnr ) = 0.D0
-  !
-  CALL stop_clock( 'scatter' )
-  !
-#else
-  CALL errore('cscatter_custom', 'do not use in serial execution', 1)
-#endif
-  !
-  RETURN
-  !
-END SUBROUTINE cscatter_custom
-!
-!----------------------------------------------------------------------------
-SUBROUTINE gather_smooth ( f_in, f_out )
-  !----------------------------------------------------------------------------
-  !
-  ! ... gathers data on the smooth AND real fft grid
-  !
-  ! ... gathers nproc distributed data on the first processor of every pool
-  !
-  ! ... REAL*8      f_in  = distributed variable ( dffts%nnr )
-  ! ... REAL*8      f_out = gathered variable (nr1sx*nr2sx*nr3sx)
-  !
-  USE mp,        ONLY : mp_barrier
-  USE kinds,     ONLY : DP
-  USE parallel_include
-  !
-  IMPLICIT NONE
-  !
-  REAL(DP) :: f_in(:), f_out(:)
-  !
-#if defined (__MPI)
-  !
-  INTEGER :: proc, info
-  INTEGER :: displs(0:dffts%nproc-1), recvcount(0:dffts%nproc-1)
-  !
-  !
-  CALL start_clock( 'gather' )
-  !
-  DO proc = 0, ( dffts%nproc - 1 )
-     !
-     recvcount(proc) = dffts%nnp * dffts%npp(proc+1)
-     !
-     IF ( proc == 0 ) THEN
-        !
-        displs(proc) = 0
-        !
-     ELSE
-        !
-        displs(proc) = displs(proc-1) + recvcount(proc-1)
-        !
-     ENDIF
-     !
-  ENDDO
-  !
-  CALL mp_barrier( dffts%comm )
-  !
-  CALL MPI_GATHERV( f_in, recvcount(dffts%mype), MPI_DOUBLE_PRECISION, f_out, &
-                    recvcount, displs, MPI_DOUBLE_PRECISION, dffts%root,    &
-                    dffts%comm, info )
-  !
-  CALL errore( 'gather', 'info<>0', info )
-  !
-  CALL stop_clock( 'gather' )
-  !
-#else
-  CALL errore('gather_smooth', 'do not use in serial execution', 1)
-#endif
-  !
-  RETURN
-  !
-END SUBROUTINE gather_smooth
-!
-!----------------------------------------------------------------------------
-SUBROUTINE scatter_smooth( f_in, f_out )
-  !----------------------------------------------------------------------------
-  !
-  ! ... scatters data on the smooth AND real fft grid
-  ! ... scatters data from the first processor of every pool
-  !
-  ! ... REAL*8      f_in  = gathered variable (nr1sx*nr2sx*nr3sx)
-  ! ... REAL*8      f_out = distributed variable ( dffts%nnr)
-  !
-  USE mp,        ONLY : mp_barrier
-  USE kinds,     ONLY : DP
-  USE parallel_include
-  !
-  IMPLICIT NONE
-  !
-  REAL(DP) :: f_in(:), f_out(:)
-  !
-#if defined (__MPI)
-  !
-  INTEGER :: proc, info
-  INTEGER :: displs(0:dffts%nproc-1), sendcount(0:dffts%nproc-1)
-  !
-  !
-  CALL start_clock( 'scatter' )
-  !
-  DO proc = 0, ( dffts%nproc - 1 )
-     !
-     sendcount(proc) = dffts%nnp * dffts%npp(proc+1)
-     !
-     IF ( proc == 0 ) THEN
-        !
-        displs(proc) = 0
-        !
-     ELSE
-        !
-        displs(proc) = displs(proc-1) + sendcount(proc-1)
-        !
-     ENDIF
-     !
-  ENDDO
-  !
-  CALL mp_barrier( dffts%comm )
-  !
-  CALL MPI_SCATTERV( f_in, sendcount, displs, MPI_DOUBLE_PRECISION,   &
-                     f_out, sendcount(dffts%mype), MPI_DOUBLE_PRECISION, &
-                     dffts%root, dffts%comm, info )
-  !
-  CALL errore( 'scatter', 'info<>0', info )
-  !
-  IF ( sendcount(dffts%mype) /=  dffts%nnr  ) f_out(sendcount(dffts%mype)+1: dffts%nnr ) = 0.D0
-  !
-  CALL stop_clock( 'scatter' )
-  !
-#else
-  CALL errore('scatter_smooth', 'do not use in serial execution', 1)
-#endif
-  !
-  RETURN
-  !
-END SUBROUTINE scatter_smooth
-
-
 !
 SUBROUTINE tg_gather( dffts, v, tg_v )
    !
@@ -1141,6 +838,162 @@ SUBROUTINE tg_cgather( dffts, v, tg_v )
 
 END SUBROUTINE tg_cgather
 
+!
+!-----------------------------------------------------------------------
+SUBROUTINE cgather_sym_many( f_in, f_out, nbnd, nbnd_proc, start_nbnd_proc )
+  !-----------------------------------------------------------------------
+  !
+  ! ... Written by A. Dal Corso
+  !
+  ! ... This routine generalizes cgather_sym, receiving nbnd complex 
+  ! ... distributed functions and collecting nbnd_proc(dfftp%mype+1) 
+  ! ... functions in each processor.
+  ! ... start_nbnd_proc(dfftp%mype+1), says where the data for each processor
+  ! ... start in the distributed variable
+  ! ... COMPLEX*16  f_in  = distributed variable (nrxx,nbnd)
+  ! ... COMPLEX*16  f_out = gathered variable (nr1x*nr2x*nr3x, 
+  !                                             nbnd_proc(dfftp%mype+1))
+  !
+  USE mp,        ONLY : mp_barrier
+  USE parallel_include
+  !
+  IMPLICIT NONE
+  !
+  INTEGER :: nbnd, nbnd_proc(dfftp%nproc), start_nbnd_proc(dfftp%nproc)
+  COMPLEX(DP) :: f_in(dfftp%nnr,nbnd)
+  COMPLEX(DP) :: f_out(dfftp%nnp*dfftp%nr3x,nbnd_proc(dfftp%mype+1))
+  !
+#if defined (__MPI)
+  !
+  INTEGER :: proc, info
+  INTEGER :: ibnd, jbnd
+  INTEGER :: displs(0:dfftp%nproc-1), recvcount(0:dfftp%nproc-1)
+  !
+  !
+  CALL start_clock( 'cgather' )
+  !
+  DO proc = 0, ( dfftp%nproc - 1 )
+     !
+     recvcount(proc) = 2 * dfftp%nnp * dfftp%npp(proc+1)
+     !
+     IF ( proc == 0 ) THEN
+        !
+        displs(proc) = 0
+        !
+     ELSE
+        !
+        displs(proc) = displs(proc-1) + recvcount(proc-1)
+        !
+     ENDIF
+     !
+  ENDDO
+  !
+  CALL mp_barrier( dfftp%comm )
+  !
+  DO proc = 0, dfftp%nproc - 1
+     DO ibnd = 1, nbnd_proc(proc+1)
+        jbnd = start_nbnd_proc(proc+1) + ibnd - 1
+        CALL MPI_GATHERV( f_in(1,jbnd), recvcount(dfftp%mype), &
+                        MPI_DOUBLE_PRECISION, f_out(1,ibnd), recvcount, &
+                        displs, MPI_DOUBLE_PRECISION, proc, dfftp%comm, info )
+     END DO
+  END DO
+  !
+  CALL errore( 'cgather_sym_many', 'info<>0', info )
+  !
+!  CALL mp_barrier( dfftp%comm )
+  !
+  CALL stop_clock( 'cgather' )
+  !
+#else
+  CALL errore('cgather_sym_many', 'do not use in serial execution', 1)
+#endif
+  !
+  RETURN
+  !
+END SUBROUTINE cgather_sym_many
+!
+!----------------------------------------------------------------------------
+SUBROUTINE cscatter_sym_many( f_in, f_out, target_ibnd, nbnd, nbnd_proc, &
+                              start_nbnd_proc   )
+  !----------------------------------------------------------------------------
+  !
+  ! ... Written by A. Dal Corso
+  !
+  ! ... generalizes cscatter_sym. It assumes that each processor has
+  ! ... a certain number of bands (nbnd_proc(dfftp%mype+1)). The processor 
+  ! ... that has target_ibnd scatters it to all the other processors 
+  ! ... that receive a distributed part of the target function. 
+  ! ... start_nbnd_proc(dfftp%mype+1) is used to identify the processor
+  ! ... that has the required band
+  !
+  ! ... COMPLEX*16  f_in  = gathered variable (nr1x*nr2x*nr3x,
+  !                                                nbnd_proc(dfftp%mype+1) )
+  ! ... COMPLEX*16  f_out = distributed variable (nrxx)
+  !
+  USE mp,        ONLY : mp_barrier
+  USE kinds,     ONLY : DP
+  USE parallel_include
+  !
+  IMPLICIT NONE
+  !
+  INTEGER :: nbnd, nbnd_proc(dfftp%nproc), start_nbnd_proc(dfftp%nproc)
+  COMPLEX(DP) :: f_in(dfftp%nnp*dfftp%nr3x,nbnd_proc(dfftp%mype+1))
+  COMPLEX(DP) :: f_out(dfftp%nnr)
+  INTEGER :: target_ibnd
+  !
+#if defined (__MPI)
+  !
+  INTEGER :: proc, info
+  INTEGER :: displs(0:dfftp%nproc-1), sendcount(0:dfftp%nproc-1)
+  INTEGER :: ibnd, jbnd
+  !
+  !
+  CALL start_clock( 'cscatter_sym' )
+  !
+  DO proc = 0, ( dfftp%nproc - 1 )
+     !
+     sendcount(proc) = 2 * dfftp%nnp * dfftp%npp(proc+1)
+     !
+     IF ( proc == 0 ) THEN
+        !
+        displs(proc) = 0
+        !
+     ELSE
+        !
+        displs(proc) = displs(proc-1) + sendcount(proc-1)
+        !
+     ENDIF
+     !
+  ENDDO
+  !
+  f_out = (0.0_DP, 0.0_DP)
+  !
+  CALL mp_barrier( dfftp%comm )
+  !
+  DO proc = 0, dfftp%nproc - 1
+     DO ibnd = 1, nbnd_proc(proc+1)
+        jbnd = start_nbnd_proc(proc+1) + ibnd - 1
+        IF (jbnd==target_ibnd) &
+        CALL MPI_SCATTERV( f_in(1,ibnd), sendcount, displs, &
+               MPI_DOUBLE_PRECISION, f_out, sendcount(dfftp%mype), &
+               MPI_DOUBLE_PRECISION, proc, dfftp%comm, info )
+     ENDDO
+  ENDDO
+  !
+  CALL errore( 'cscatter_sym_many', 'info<>0', info )
+  !
+  CALL stop_clock( 'cscatter_sym' )
+  !
+#else
+  CALL errore('cscatter_sym_many', 'do not use in serial execution', 1)
+#endif
+  !
+  RETURN
+  !
+END SUBROUTINE cscatter_sym_many
+
+
 !=----------------------------------------------------------------------=!
    END MODULE fft_base
 !=----------------------------------------------------------------------=!
