@@ -73,6 +73,13 @@ SUBROUTINE tg_cft3s( f, dfft, isgn, use_task_groups )
   LOGICAL, OPTIONAL, INTENT(in) :: use_task_groups
                                            ! specify if you want to use task groups parallelization
   !
+  ! the following ifdef prevents usage of directive in older ifort versions
+  !
+#if defined(__INTEL_COMPILER)
+#if __INTEL_COMPILER  >= 1300
+!dir$ attributes align: 4096 :: yf, aux
+#endif
+#endif
   INTEGER                    :: me_p
   INTEGER                    :: n1, n2, n3, nx1, nx2, nx3
   COMPLEX(DP), ALLOCATABLE   :: yf(:), aux (:)
@@ -117,10 +124,9 @@ SUBROUTINE tg_cft3s( f, dfft, isgn, use_task_groups )
         planes = dfft%iplp
         !
      ELSE
-        !
-        CALL pack_group_sticks()
         !
         IF( use_tg ) THEN
+           CALL pack_group_sticks( f, yf, dfft )
            CALL cft_1z( yf, dfft%tg_nsw( me_p ), n3, nx3, isgn, aux )
         ELSE
            CALL cft_1z( f, dfft%nsw( me_p ), n3, nx3, isgn, aux )
@@ -130,7 +136,7 @@ SUBROUTINE tg_cft3s( f, dfft, isgn, use_task_groups )
         !
      ENDIF
      !
-     CALL fw_scatter( isgn ) ! forwart scatter from stick to planes
+     CALL fw_scatter( isgn ) ! forward scatter from stick to planes
      !
      IF( use_tg ) THEN
         CALL cft_2xy( f, dfft%tg_npp( me_p ), n1, n2, nx1, nx2, isgn, planes )
@@ -169,97 +175,25 @@ SUBROUTINE tg_cft3s( f, dfft, isgn, use_task_groups )
         !
         IF( use_tg ) THEN
            CALL cft_1z( aux, dfft%tg_nsw( me_p ), n3, nx3, isgn, yf )
+           CALL unpack_group_sticks( yf, f, dfft )
         ELSE
            CALL cft_1z( aux, dfft%nsw( me_p ), n3, nx3, isgn, f )
         ENDIF
         !
-        CALL unpack_group_sticks()
-        !
      ENDIF
      !
   ENDIF
   !
-  DEALLOCATE( aux )
-  !
   IF( use_tg ) THEN
      DEALLOCATE( yf )
   ENDIF
+
+  DEALLOCATE( aux )
   !
   RETURN
   !
 CONTAINS
   !
-
-  SUBROUTINE pack_group_sticks()
-
-     INTEGER                     :: ierr
-     !
-     IF( .not. use_tg ) RETURN
-     !
-     IF( dfft%tg_rdsp(dfft%nogrp) + dfft%tg_rcv(dfft%nogrp) > size( yf ) ) THEN
-        CALL errore( 'pack_group_sticks' , ' inconsistent size ', 1 )
-     ENDIF
-     IF( dfft%tg_psdsp(dfft%nogrp) + dfft%tg_snd(dfft%nogrp) > size( f ) ) THEN
-        CALL errore( 'pack_group_sticks', ' inconsistent size ', 2 )
-     ENDIF
-
-     CALL start_clock( 'ALLTOALL' )
-     !
-     !  Collect all the sticks of the different states,
-     !  in "yf" processors will have all the sticks of the OGRP
-
-#if defined __MPI
-
-     CALL MPI_ALLTOALLV( f(1), dfft%tg_snd, dfft%tg_psdsp, MPI_DOUBLE_COMPLEX, yf(1), dfft%tg_rcv, &
-      &                     dfft%tg_rdsp, MPI_DOUBLE_COMPLEX, dfft%ogrp_comm, IERR)
-     IF( ierr /= 0 ) THEN
-        CALL errore( 'pack_group_sticks', ' alltoall error 1 ', abs(ierr) )
-     ENDIF
-
-#endif
-
-     CALL stop_clock( 'ALLTOALL' )
-     !
-     !YF Contains all ( ~ NOGRP*dfft%nsw(me) ) Z-sticks
-     !
-     RETURN
-  END SUBROUTINE pack_group_sticks
-
-  !
-
-  SUBROUTINE unpack_group_sticks()
-     !
-     !  Bring pencils back to their original distribution
-     !
-     INTEGER                     :: ierr
-     !
-     IF( .not. use_tg ) RETURN
-     !
-     IF( dfft%tg_usdsp(dfft%nogrp) + dfft%tg_snd(dfft%nogrp) > size( f ) ) THEN
-        CALL errore( 'unpack_group_sticks', ' inconsistent size ', 3 )
-     ENDIF
-     IF( dfft%tg_rdsp(dfft%nogrp) + dfft%tg_rcv(dfft%nogrp) > size( yf ) ) THEN
-        CALL errore( 'unpack_group_sticks', ' inconsistent size ', 4 )
-     ENDIF
-
-     CALL start_clock( 'ALLTOALL' )
-
-#if defined __MPI
-     CALL MPI_Alltoallv( yf(1), &
-          dfft%tg_rcv, dfft%tg_rdsp, MPI_DOUBLE_COMPLEX, f(1), &
-          dfft%tg_snd, dfft%tg_usdsp, MPI_DOUBLE_COMPLEX, dfft%ogrp_comm, IERR)
-     IF( ierr /= 0 ) THEN
-        CALL errore( 'unpack_group_sticks', ' alltoall error 2 ', abs(ierr) )
-     ENDIF
-#endif
-
-     CALL stop_clock( 'ALLTOALL' )
-
-     RETURN
-  END SUBROUTINE unpack_group_sticks
-
-  !
-
   SUBROUTINE fw_scatter( iopt )
 
         !Transpose data for the 2-D FFT on the x-y plane
@@ -329,4 +263,209 @@ CONTAINS
   !
 END SUBROUTINE tg_cft3s
 !
+!
+!
+!----------------------------------------------------------------------------
+SUBROUTINE fw_tg_cft3_z( f_in, dfft, f_out )
+  !----------------------------------------------------------------------------
+  !
+  USE fft_scalar, ONLY : cft_1z
+  USE kinds,      ONLY : DP
+  USE fft_types,  ONLY : fft_dlay_descriptor
+  USE parallel_include
+  !
+  IMPLICIT NONE
+  !
+  COMPLEX(DP), INTENT(inout)    :: f_in( : )  ! INPUT array containing data to be transformed
+  COMPLEX(DP), INTENT(inout)   :: f_out (:)  ! OUTPUT
+  TYPE (fft_dlay_descriptor), INTENT(in) :: dfft ! descriptor of fft data layout
+  !
+  CALL cft_1z( f_in, dfft%tg_nsw( dfft%mype + 1 ), dfft%nr3, dfft%nr3x, 2, f_out )
+  !
+END SUBROUTINE fw_tg_cft3_z
+!
+!----------------------------------------------------------------------------
+SUBROUTINE bw_tg_cft3_z( f_out, dfft, f_in )
+  !----------------------------------------------------------------------------
+  !
+  USE fft_scalar, ONLY : cft_1z
+  USE kinds,      ONLY : DP
+  USE fft_types,  ONLY : fft_dlay_descriptor
+  USE parallel_include
+  !
+  IMPLICIT NONE
+  !
+  COMPLEX(DP), INTENT(inout)    :: f_out( : ) ! OUTPUT
+  COMPLEX(DP), INTENT(inout)   :: f_in (:) ! INPUT array containing data to be transformed
+  TYPE (fft_dlay_descriptor), INTENT(in) :: dfft ! descriptor of fft data layout
+  !
+  CALL cft_1z( f_in, dfft%tg_nsw( dfft%mype + 1 ), dfft%nr3, dfft%nr3x, -2, f_out )
+  !
+END SUBROUTINE bw_tg_cft3_z
+!
+!----------------------------------------------------------------------------
+SUBROUTINE fw_tg_cft3_scatter( f, dfft, aux )
+  !----------------------------------------------------------------------------
+  !
+  USE fft_base,   ONLY : fft_scatter
+  USE kinds,      ONLY : DP
+  USE fft_types,  ONLY : fft_dlay_descriptor
+  !
+  IMPLICIT NONE
+  !
+  COMPLEX(DP), INTENT(inout)    :: f( : ), aux( : )  ! array containing data to be transformed
+  TYPE (fft_dlay_descriptor), INTENT(in) :: dfft     ! descriptor of fft data layout
+  !
+  CALL fft_scatter( dfft, aux, dfft%nr3x, dfft%nogrp*dfft%tg_nnr, f, dfft%tg_nsw, dfft%tg_npp, 2, .true. )
+  !
+END SUBROUTINE fw_tg_cft3_scatter
+!
+!----------------------------------------------------------------------------
+SUBROUTINE bw_tg_cft3_scatter( f, dfft, aux )
+  !----------------------------------------------------------------------------
+  !
+  USE fft_base,   ONLY : fft_scatter
+  USE kinds,      ONLY : DP
+  USE fft_types,  ONLY : fft_dlay_descriptor
+  !
+  IMPLICIT NONE
+  !
+  COMPLEX(DP), INTENT(inout)    :: f( : ), aux( : )  ! array containing data to be transformed
+  TYPE (fft_dlay_descriptor), INTENT(in) :: dfft     ! descriptor of fft data layout
+  !
+  CALL fft_scatter( dfft, aux, dfft%nr3x, dfft%nogrp*dfft%tg_nnr, f, dfft%tg_nsw, dfft%tg_npp, -2, .true. )
+  !
+END SUBROUTINE bw_tg_cft3_scatter
+!
+!----------------------------------------------------------------------------
+SUBROUTINE fw_tg_cft3_xy( f, dfft )
+  !----------------------------------------------------------------------------
+  !
+  USE fft_scalar, ONLY : cft_2xy
+  USE kinds,      ONLY : DP
+  USE fft_types,  ONLY : fft_dlay_descriptor
+  !
+  IMPLICIT NONE
+  !
+  COMPLEX(DP), INTENT(inout)    :: f( : ) ! INPUT/OUTPUT array containing data to be transformed
+  TYPE (fft_dlay_descriptor), INTENT(in) :: dfft ! descriptor of fft data layout
+  INTEGER                    :: planes( dfft%nr1x )
+  !
+  planes = dfft%iplw
+  CALL cft_2xy( f, dfft%tg_npp( dfft%mype + 1 ), dfft%nr1, dfft%nr2, dfft%nr1x, dfft%nr2x, 2, planes )
+  !
+END SUBROUTINE fw_tg_cft3_xy
+!
+!----------------------------------------------------------------------------
+SUBROUTINE bw_tg_cft3_xy( f, dfft )
+  !----------------------------------------------------------------------------
+  !
+  USE fft_scalar, ONLY : cft_2xy
+  USE kinds,      ONLY : DP
+  USE fft_types,  ONLY : fft_dlay_descriptor
+  !
+  IMPLICIT NONE
+  !
+  COMPLEX(DP), INTENT(inout)    :: f( : ) ! INPUT/OUTPUT  array containing data to be transformed
+  TYPE (fft_dlay_descriptor), INTENT(in) :: dfft ! descriptor of fft data layout
+  INTEGER                    :: planes( dfft%nr1x )
+  !
+  planes = dfft%iplw
+  CALL cft_2xy( f, dfft%tg_npp( dfft%mype + 1 ), dfft%nr1, dfft%nr2, dfft%nr1x, dfft%nr2x, -2, planes )
+  !
+END SUBROUTINE bw_tg_cft3_xy
+
+
+!----------------------------------------------------------------------------
+  SUBROUTINE pack_group_sticks( f, yf, dfft )
+
+     USE kinds,      ONLY : DP
+     USE fft_types,  ONLY : fft_dlay_descriptor
+     USE parallel_include
+
+     IMPLICIT NONE
+
+     COMPLEX(DP), INTENT(in)    :: f( : )  ! array containing all bands, and gvecs distributed across processors
+     COMPLEX(DP), INTENT(out)    :: yf( : )  ! array containing bands collected into task groups
+     TYPE (fft_dlay_descriptor), INTENT(in) :: dfft
+     INTEGER                     :: ierr
+     !
+     IF( dfft%tg_rdsp(dfft%nogrp) + dfft%tg_rcv(dfft%nogrp) > size( yf ) ) THEN
+        CALL errore( 'pack_group_sticks' , ' inconsistent size ', 1 )
+     ENDIF
+     IF( dfft%tg_psdsp(dfft%nogrp) + dfft%tg_snd(dfft%nogrp) > size( f ) ) THEN
+        CALL errore( 'pack_group_sticks', ' inconsistent size ', 2 )
+     ENDIF
+
+     CALL start_clock( 'ALLTOALL' )
+     !
+     !  Collect all the sticks of the different states,
+     !  in "yf" processors will have all the sticks of the OGRP
+
+#if defined __MPI
+
+     CALL MPI_ALLTOALLV( f(1), dfft%tg_snd, dfft%tg_psdsp, MPI_DOUBLE_COMPLEX, yf(1), dfft%tg_rcv, &
+      &                     dfft%tg_rdsp, MPI_DOUBLE_COMPLEX, dfft%ogrp_comm, IERR)
+     IF( ierr /= 0 ) THEN
+        CALL errore( 'pack_group_sticks', ' alltoall error 1 ', abs(ierr) )
+     ENDIF
+
+#else
+
+     IF( dfft%tg_rcv(dfft%nogrp) /= dfft%tg_snd(dfft%nogrp) ) THEN
+        CALL errore( 'pack_group_sticks', ' inconsistent size ', 3 )
+     ENDIF
+
+     yf( 1 : dfft%tg_rcv(dfft%nogrp) ) =  f( 1 : dfft%tg_snd(dfft%nogrp) )
+
+#endif
+
+     CALL stop_clock( 'ALLTOALL' )
+     !
+     !YF Contains all ( ~ NOGRP*dfft%nsw(me) ) Z-sticks
+     !
+     RETURN
+  END SUBROUTINE pack_group_sticks
+
+  !
+
+  SUBROUTINE unpack_group_sticks( yf, f, dfft )
+
+     USE kinds,      ONLY : DP
+     USE fft_types,  ONLY : fft_dlay_descriptor
+     USE parallel_include
+
+     IMPLICIT NONE
+
+     COMPLEX(DP), INTENT(out)    :: f( : )  ! array containing all bands, and gvecs distributed across processors
+     COMPLEX(DP), INTENT(in)    :: yf( : )  ! array containing bands collected into task groups
+     TYPE (fft_dlay_descriptor), INTENT(in) :: dfft
+     !
+     !  Bring pencils back to their original distribution
+     !
+     INTEGER                     :: ierr
+     !
+     IF( dfft%tg_usdsp(dfft%nogrp) + dfft%tg_snd(dfft%nogrp) > size( f ) ) THEN
+        CALL errore( 'unpack_group_sticks', ' inconsistent size ', 3 )
+     ENDIF
+     IF( dfft%tg_rdsp(dfft%nogrp) + dfft%tg_rcv(dfft%nogrp) > size( yf ) ) THEN
+        CALL errore( 'unpack_group_sticks', ' inconsistent size ', 4 )
+     ENDIF
+
+     CALL start_clock( 'ALLTOALL' )
+
+#if defined __MPI
+     CALL MPI_Alltoallv( yf(1), &
+          dfft%tg_rcv, dfft%tg_rdsp, MPI_DOUBLE_COMPLEX, f(1), &
+          dfft%tg_snd, dfft%tg_usdsp, MPI_DOUBLE_COMPLEX, dfft%ogrp_comm, IERR)
+     IF( ierr /= 0 ) THEN
+        CALL errore( 'unpack_group_sticks', ' alltoall error 2 ', abs(ierr) )
+     ENDIF
+#endif
+
+     CALL stop_clock( 'ALLTOALL' )
+
+     RETURN
+  END SUBROUTINE unpack_group_sticks
+
 END MODULE fft_parallel
