@@ -1,27 +1,36 @@
 !
-! Copyright (C) 2001-2012 Quantum ESPRESSO group
+! Copyright (C) 2001-2015 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
 ! or http://www.gnu.org/copyleft/gpl.txt .
 !
 !--------------------------------------------------------------------------!
-! FFT scalar drivers Module - contains machine-dependent routines for:     !
-! FFTW, FFTW3, ESSL, LINUX_ESSL, SCSL, SUNPERF, NEC ASL libraries          !
-! (both 3d for serial execution and 1d+2d FFTs for parallel execution,     !
-! excepted NEC ASL, 3d only, no parallel execution)                        !
+! FFT scalar drivers Module - contains machine-dependent routines for      !
+! FFTW, FFTW3, ESSL (both 3d for serial execution and 1d+2d FFTs for       !
+! parallel execution; NEC ASL libraries (3d only, no parallel execution)   !
 ! Written by Carlo Cavazzoni, modified by P. Giannozzi, contributions      !
 ! by Martin Hilgemans, Guido Roma, Pascal Thibaudeau, Stephane Lefranc,    !
-! Nicolas Lacorne, Filippo Spiga - Last update Aug 2012                    !
+! Nicolas Lacorne, Filippo Spiga, Nicola Varini - Last update Jul 2015     !
 !--------------------------------------------------------------------------!
 
 #include "fft_defs.h"
+
+#if defined __DFTI
+#include "mkl_dfti.f90"
+#endif
+
 !=----------------------------------------------------------------------=!
    MODULE fft_scalar
 !=----------------------------------------------------------------------=!
        USE kinds
 
-        IMPLICIT NONE
+       USE, intrinsic ::  iso_c_binding
+       
+#if defined __DFTI
+       USE MKL_DFTI ! -- this can be found in the MKL include directory
+#endif
+       IMPLICIT NONE
         SAVE
 
         PRIVATE
@@ -41,36 +50,37 @@
         !   in order to avoid multiple copies of the same workspace
         !   lwork:   Dimension of the work space array (if any)
 
-#if ( defined __ESSL || defined __LINUX_ESSL ) && ! ( defined __FFTW || defined __FFTW3 )
+#if defined __FFTW 
+
+        INTEGER   :: cft_b_dims( 4 )
+!$omp threadprivate (cft_b_dims)
+        C_POINTER :: cft_b_bw_planz = 0
+!$omp threadprivate (cft_b_bw_planz)
+        C_POINTER :: cft_b_bw_planx = 0
+!$omp threadprivate (cft_b_bw_planx)
+        C_POINTER :: cft_b_bw_plany = 0
+!$omp threadprivate (cft_b_bw_plany)
+
+#elif defined __FFTW3
+
+#if defined __OPENMP
+#include "fftw3.f03"
+#else
+#include "fftw3.f"
+#endif
+
+#elif defined __DFTI
+        TYPE dfti_descriptor_array
+           TYPE(DFTI_DESCRIPTOR), POINTER :: desc
+        END TYPE
+
+#elif ( defined __ESSL || defined __LINUX_ESSL )
 
         !   ESSL IBM library: see the ESSL manual for DCFT
 
         INTEGER, PARAMETER :: lwork = 20000 + ( 2*nfftx + 256 ) * 64 + 3*nfftx
         REAL (DP) :: work( lwork )
 
-#elif defined __SCSL || defined __SUNPERF
-
-        !   SGI scientific library scsl and SUN sunperf
-
-        INTEGER, PARAMETER :: lwork = 2 * nfftx
-        COMPLEX (DP) :: work(lwork)
-
-#elif defined __FFTW3
-
-        !  Only FFTW_ESTIMATE is actually used
-
-#define  FFTW_MEASURE  0
-#define  FFTW_ESTIMATE 64
-
-#endif
-
-#if defined __FFTW 
-
-        INTEGER   :: cft_b_dims( 4 )
-        C_POINTER :: cft_b_bw_planz = 0
-        C_POINTER :: cft_b_bw_planx = 0
-        C_POINTER :: cft_b_bw_plany = 0
-
 #endif
 
 !=----------------------------------------------------------------------=!
@@ -106,7 +116,7 @@
      COMPLEX (DP) :: c(:), cout(:)
 
      REAL (DP)  :: tscale
-     INTEGER    :: i, err, idir, ip
+     INTEGER    :: i, err, idir, ip, void
      INTEGER, SAVE :: zdims( 3, ndims ) = -1
      INTEGER, SAVE :: icurrent = 1
      LOGICAL :: done
@@ -138,6 +148,21 @@
      C_POINTER, SAVE :: fw_planz( ndims ) = 0
      C_POINTER, SAVE :: bw_planz( ndims ) = 0
 
+#elif defined __DFTI
+
+     !   Intel MKL native FFT driver
+
+     TYPE(DFTI_DESCRIPTOR_ARRAY), SAVE :: hand( ndims )
+     LOGICAL, SAVE :: dfti_first = .TRUE.
+     INTEGER :: dfti_status = 0
+     !
+     IF( dfti_first .EQ. .TRUE. ) THEN
+        DO ip = 1, ndims
+           hand(ip)%desc => NULL()
+        END DO
+        dfti_first = .FALSE.
+     END IF
+
 #elif defined __ESSL || defined __LINUX_ESSL
 
      !   ESSL IBM library: see the ESSL manual for DCFT
@@ -146,15 +171,6 @@
      REAL (DP), SAVE :: fw_tablez( ltabl, ndims )
      REAL (DP), SAVE :: bw_tablez( ltabl, ndims )
 
-#elif defined __SCSL
-
-     !   SGI scientific library scsl
-
-     INTEGER, PARAMETER :: ltabl = 2 * nfftx + 256
-     REAL (DP), SAVE :: tablez (ltabl, ndims)
-     REAL (DP)       :: DUMMY
-     INTEGER, SAVE :: isys(0:1) = (/ 1, 1 /)
-
 #elif defined __SX6
 
      !   NEC MathKeisan
@@ -165,13 +181,6 @@
      COMPLEX (DP)    :: DUMMY
      INTEGER, SAVE :: isys = 1
 
-#elif defined __SUNPERF
-
-     !   SUN sunperf library
-
-     INTEGER, PARAMETER :: ltabl = 4 * nfftx + 15
-     REAL (DP), SAVE :: tablez (ltabl, ndims)
-
 #endif
 
      IF( nsl < 0 ) THEN
@@ -203,7 +212,6 @@
        !   initialize a new one
 
        ! WRITE( stdout, fmt="('DEBUG cft_1z, reinitializing tables ', I3)" ) icurrent
-
 #if defined __FFTW
 
        IF( fw_planz( icurrent) /= 0 ) CALL DESTROY_PLAN_1D( fw_planz( icurrent) )
@@ -213,6 +221,11 @@
 
 #elif defined __FFTW3
 
+#if defined __OPENMP
+       CALL dfftw_cleanup_threads() 
+       void = fftw_init_threads()
+       CALL dfftw_plan_with_nthreads(omp_get_max_threads())      
+#endif
 
        IF( fw_planz( icurrent) /= 0 ) CALL dfftw_destroy_plan( fw_planz( icurrent) )
        IF( bw_planz( icurrent) /= 0 ) CALL dfftw_destroy_plan( bw_planz( icurrent) )
@@ -223,6 +236,53 @@
        CALL dfftw_plan_many_dft( bw_planz( icurrent), 1, nz, nsl, c, &
             (/SIZE(c)/), 1, ldz, cout, (/SIZE(cout)/), 1, ldz, idir, FFTW_ESTIMATE)
 
+#elif defined __DFTI
+
+       if( ASSOCIATED( hand( icurrent )%desc ) ) THEN
+          dfti_status = DftiFreeDescriptor( hand( icurrent )%desc )
+          IF( dfti_status /= 0) THEN
+             WRITE(*,*) "stopped in DftiFreeDescriptor", dfti_status
+             STOP
+          ENDIF
+       END IF
+
+     dfti_status = DftiCreateDescriptor(hand( icurrent )%desc, DFTI_DOUBLE, DFTI_COMPLEX, 1,nz)
+     IF(dfti_status /= 0) THEN
+        WRITE(*,*) "stopped in DftiCreateDescriptor", dfti_status
+        STOP
+     ENDIF
+     dfti_status = DftiSetValue(hand( icurrent )%desc, DFTI_NUMBER_OF_TRANSFORMS,nsl)
+     IF(dfti_status /= 0)THEN
+        WRITE(*,*) "stopped in DFTI_NUMBER_OF_TRANSFORMS", dfti_status
+        STOP
+     ENDIF
+     dfti_status = DftiSetValue(hand( icurrent )%desc,DFTI_INPUT_DISTANCE, ldz )
+     IF(dfti_status /= 0)THEN
+        WRITE(*,*) "stopped in DFTI_INPUT_DISTANCE", dfti_status
+        STOP
+     ENDIF
+     dfti_status = DftiSetValue(hand( icurrent )%desc, DFTI_PLACEMENT, DFTI_INPLACE)
+     IF(dfti_status /= 0)THEN
+        WRITE(*,*) "stopped in DFTI_PLACEMENT", dfti_status
+        STOP
+     ENDIF
+     tscale = 1.0_DP/nz
+     dfti_status = DftiSetValue( hand( icurrent )%desc, DFTI_FORWARD_SCALE, tscale);
+     IF(dfti_status /= 0)THEN
+        WRITE(*,*) "stopped in DFTI_FORWARD_SCALE", dfti_status
+        STOP
+     ENDIF
+     dfti_status = DftiSetValue( hand( icurrent )%desc, DFTI_BACKWARD_SCALE, DBLE(1) );
+     IF(dfti_status /= 0)THEN
+        WRITE(*,*) "stopped in DFTI_BACKWARD_SCALE", dfti_status
+        STOP
+     ENDIF
+     dfti_status = DftiCommitDescriptor(hand( icurrent )%desc)
+     IF(dfti_status /= 0)THEN
+        WRITE(*,*) "stopped in DftiCommitDescriptor", dfti_status
+        STOP
+     ENDIF
+
 #elif defined __ESSL || defined __LINUX_ESSL
 
        tscale = 1.0_DP / nz
@@ -233,20 +293,10 @@
           1.0_DP, bw_tablez(1, icurrent), ltabl, work(1), lwork)
 
 
-#elif defined __SCSL
-
-       CALL ZZFFTM (0, nz, 0, 0.0_DP, DUMMY, 1, DUMMY, 1, &
-                    tablez (1, icurrent), DUMMY, isys)
-
 #elif defined __SX6
 
        CALL ZZFFTM (0, nz, 1, 1.0_DP, DUMMY, ldz, DUMMY, ldz, &
                     tablez (1, icurrent), work, isys)
-
-#elif defined __SUNPERF
-
-       CALL zffti (nz, tablez (1, icurrent) )
-
 #else
 
        CALL errore(' cft_1z ',' no scalar fft driver specified ', 1)
@@ -282,11 +332,9 @@
           CALL FFT_Z_STICK_SINGLE(fw_planz( ip), c(offset), ldz_t)
        END DO
 !$omp end do
-       tscale = 1.0_DP / nz
-!$omp workshare
-       cout( 1 : ldz * nsl ) = c( 1 : ldz * nsl ) * tscale
-!$omp end workshare
 !$omp end parallel
+       tscale = 1.0_DP / nz
+       cout( 1 : ldz * nsl ) = c( 1 : ldz * nsl ) * tscale
      ELSE IF (isign > 0) THEN
 !$omp parallel default(none) private(tid,offset,i) shared(c,isign,nsl,bw_planz,ip,cout,ldz) &
 !$omp &        firstprivate(ldz_t)
@@ -326,17 +374,23 @@
         CALL dfftw_execute_dft( bw_planz( ip), c, cout)
      END IF
 
-#elif defined __SCSL
+#elif defined __DFTI
 
-     IF ( isign < 0 ) THEN
-        idir   = -1
-        tscale = 1.0_DP / nz
-     ELSE IF ( isign > 0 ) THEN
-        idir   = 1
-        tscale = 1.0_DP
+     IF (isign < 0) THEN
+        dfti_status = DftiComputeForward(hand(ip)%desc, c )
+        cout( 1 : ldz * nsl ) = c( 1 : ldz * nsl )
+        IF(dfti_status /= 0) THEN
+           WRITE(*,*) "stopped in DftiComputeForward", dfti_status
+           STOP
+        ENDIF
+     ELSE IF (isign > 0) THEN
+        dfti_status = DftiComputeBackward(hand(ip)%desc, c )
+        cout( 1 : ldz * nsl ) = c( 1 : ldz * nsl )
+        IF(dfti_status /= 0) THEN
+           WRITE(*,*) "stopped in DftiComputeBackward", dfti_status
+           STOP
+        ENDIF
      END IF
-     IF (isign /= 0) CALL ZZFFTM (idir, nz, nsl, tscale, c(1), ldz, &
-          cout(1), ldz, tablez (1, ip), work, isys)
 
 #elif defined __SX6
 
@@ -367,21 +421,6 @@
              tscale, bw_tablez(1, ip), ltabl, work, lwork)
      END IF
 
-
-#elif defined __SUNPERF
-
-     IF ( isign < 0) THEN
-        DO i = 1, nsl
-           CALL zfftf ( nz, c(1+(i-1)*ldz), tablez ( 1, ip) )
-        END DO
-        cout( 1 : ldz * nsl ) = c( 1 : ldz * nsl ) / nz
-     ELSE IF( isign > 0 ) THEN
-        DO i = 1, nsl
-           CALL zfftb ( nz, c(1+(i-1)*ldz), tablez ( 1, ip) )
-        enddo
-        cout( 1 : ldz * nsl ) = c( 1 : ldz * nsl )
-     END IF
-
 #else
 
     CALL errore(' cft_1z ',' no scalar fft driver specified ', 1)
@@ -426,7 +465,7 @@
      INTEGER, INTENT(IN) :: isign, ldx, ldy, nx, ny, nzl
      INTEGER, OPTIONAL, INTENT(IN) :: pl2ix(:)
      COMPLEX (DP) :: r( : )
-     INTEGER :: i, k, j, err, idir, ip, kk
+     INTEGER :: i, k, j, err, idir, ip, kk, void
      REAL(DP) :: tscale
      INTEGER, SAVE :: icurrent = 1
      INTEGER, SAVE :: dims( 4, ndims) = -1
@@ -440,14 +479,26 @@
      INTEGER :: offset
      INTEGER :: nx_t, ny_t, nzl_t, ldx_t, ldy_t
      INTEGER  :: itid, mytid, ntids
-     INTEGER  :: omp_get_thread_num, omp_get_num_threads
-     EXTERNAL :: omp_get_thread_num, omp_get_num_threads
+     INTEGER  :: omp_get_thread_num, omp_get_num_threads,omp_get_max_threads
+     EXTERNAL :: omp_get_thread_num, omp_get_num_threads, omp_get_max_threads
 #endif
 
-#if defined __FFTW || defined __FFTW3
+#if defined __DFTI
 
+     TYPE(DFTI_DESCRIPTOR_ARRAY), SAVE :: hand( ndims )
+     LOGICAL, SAVE :: dfti_first = .TRUE.
+     INTEGER :: dfti_status = 0
+
+#elif defined __FFTW || defined __FFTW3
+
+#if defined __FFTW && __FFTW_ALL_XY_PLANES
+     C_POINTER, SAVE :: fw_plan_2d( ndims ) = 0
+     C_POINTER, SAVE :: bw_plan_2d( ndims ) = 0
+#else
      C_POINTER, SAVE :: fw_plan( 2, ndims ) = 0
      C_POINTER, SAVE :: bw_plan( 2, ndims ) = 0
+#endif
+
 
 #elif defined __ESSL || defined __LINUX_ESSL
 
@@ -455,14 +506,6 @@
      REAL (DP), SAVE :: fw_tablex( ltabl, ndims ), fw_tabley( ltabl, ndims )
      REAL (DP), SAVE :: bw_tablex( ltabl, ndims ), bw_tabley( ltabl, ndims )
 
-#elif defined __SCSL
-
-     INTEGER, PARAMETER :: ltabl = 2 * nfftx + 256
-     REAL (DP), SAVE :: tablex (ltabl, ndims), tabley(ltabl, ndims)
-     COMPLEX (DP) :: XY(nx+nx*ny)
-     REAL (DP)    :: DUMMY
-     INTEGER, SAVE :: isys(0:1) = (/ 1, 1 /)
-
 #elif defined __SX6
 
      INTEGER, PARAMETER :: ltabl = 2*nfftx + 64
@@ -472,19 +515,9 @@
      COMPLEX (DP) :: DUMMY
      INTEGER, SAVE :: isys = 1
 
-#elif defined __SUNPERF
-
-     INTEGER, PARAMETER :: ltabl = 4 * nfftx + 15
-     REAL (DP), SAVE :: tablex (ltabl, ndims)
-     REAL (DP), SAVE :: tabley (ltabl, ndims)
-
 #endif
 
 
-#if defined __SCSL
-     isys(0) = 1
-#endif
-
      dofft( 1 : nx ) = .TRUE.
      IF( PRESENT( pl2ix ) ) THEN
        IF( SIZE( pl2ix ) < nx ) &
@@ -499,6 +532,14 @@
      !
      !   Here initialize table only if necessary
      !
+#if defined __DFTI
+     IF( dfti_first .EQ. .TRUE. ) THEN
+        DO ip = 1, ndims
+           hand(ip)%desc => NULL()
+        END DO
+        dfti_first = .FALSE.
+     END IF
+#endif
 
      DO ip = 1, ndims
 
@@ -506,11 +547,9 @@
        !   for this combination of parameters
 
        done = ( ny == dims(1,ip) ) .AND. ( nx == dims(3,ip) )
-#if defined __ESSL || defined __LINUX_ESSL || defined __FFTW3
-        !   The initialization in ESSL and FFTW v.3 depends on all four parameters
        done = done .AND. ( ldx == dims(2,ip) ) .AND.  ( nzl == dims(4,ip) )
-#endif
        IF (done) EXIT
+
      END DO
 
      IF( .NOT. done ) THEN
@@ -520,8 +559,62 @@
 
        ! WRITE( stdout, fmt="('DEBUG cft_2xy, reinitializing tables ', I3)" ) icurrent
 
-#if defined __FFTW
+#if defined __DFTI
 
+       if( ASSOCIATED( hand( icurrent )%desc ) ) THEN
+          dfti_status = DftiFreeDescriptor( hand( icurrent )%desc )
+          IF( dfti_status /= 0) THEN
+             WRITE(*,*) "stopped in DftiFreeDescriptor", dfti_status
+             STOP
+          ENDIF
+       END IF
+
+       dfti_status = DftiCreateDescriptor(hand( icurrent )%desc, DFTI_DOUBLE, DFTI_COMPLEX, 2,(/nx,ny/))
+       IF(dfti_status /= 0) THEN
+          WRITE(*,*) "stopped in DftiCreateDescriptor", dfti_status
+          STOP
+       ENDIF
+       dfti_status = DftiSetValue(hand( icurrent )%desc, DFTI_NUMBER_OF_TRANSFORMS,nzl)
+       IF(dfti_status /= 0)THEN
+          WRITE(*,*) "stopped in DFTI_NUMBER_OF_TRANSFORMS", dfti_status
+          STOP
+       ENDIF
+       dfti_status = DftiSetValue(hand( icurrent )%desc,DFTI_INPUT_DISTANCE, ldx*ldy )
+       IF(dfti_status /= 0)THEN
+          WRITE(*,*) "stopped in DFTI_INPUT_DISTANCE", dfti_status
+          STOP
+       ENDIF
+       dfti_status = DftiSetValue(hand( icurrent )%desc, DFTI_PLACEMENT, DFTI_INPLACE)
+       IF(dfti_status /= 0)THEN
+          WRITE(*,*) "stopped in DFTI_PLACEMENT", dfti_status
+          STOP
+       ENDIF
+       tscale = 1.0_DP/ (nx * ny )
+       dfti_status = DftiSetValue( hand( icurrent )%desc, DFTI_FORWARD_SCALE, tscale);
+       IF(dfti_status /= 0)THEN
+          WRITE(*,*) "stopped in DFTI_FORWARD_SCALE", dfti_status
+          STOP
+       ENDIF
+       dfti_status = DftiSetValue( hand( icurrent )%desc, DFTI_BACKWARD_SCALE, DBLE(1) );
+       IF(dfti_status /= 0)THEN
+          WRITE(*,*) "stopped in DFTI_BACKWARD_SCALE", dfti_status
+          STOP
+       ENDIF
+       dfti_status = DftiCommitDescriptor(hand( icurrent )%desc)
+       IF(dfti_status /= 0)THEN
+          WRITE(*,*) "stopped in DftiCommitDescriptor", dfti_status
+          STOP
+       ENDIF
+
+
+#elif defined __FFTW
+
+#if defined __FFTW_ALL_XY_PLANES
+       IF( fw_plan_2d( icurrent) /= 0 )  CALL DESTROY_PLAN_2D(fw_plan_2d(icurrent) )
+       IF( bw_plan_2d( icurrent) /= 0 )  CALL DESTROY_PLAN_2D(bw_plan_2d(icurrent) )
+       idir = -1; CALL CREATE_PLAN_2D( fw_plan_2d(icurrent), nx, ny, idir)
+       idir =  1; CALL CREATE_PLAN_2D( bw_plan_2d(icurrent), nx, ny, idir)
+#else
        IF( fw_plan( 2,icurrent) /= 0 )   CALL DESTROY_PLAN_1D( fw_plan( 2,icurrent) )
        IF( bw_plan( 2,icurrent) /= 0 )   CALL DESTROY_PLAN_1D( bw_plan( 2,icurrent) )
        idir = -1; CALL CREATE_PLAN_1D( fw_plan( 2,icurrent), ny, idir)
@@ -531,9 +624,16 @@
        IF( bw_plan( 1,icurrent) /= 0 ) CALL DESTROY_PLAN_1D( bw_plan( 1,icurrent) )
        idir = -1; CALL CREATE_PLAN_1D( fw_plan( 1,icurrent), nx, idir)
        idir =  1; CALL CREATE_PLAN_1D( bw_plan( 1,icurrent), nx, idir)
+#endif
 
 #elif defined __FFTW3
 
+#if defined __OPENMP
+       CALL dfftw_cleanup_threads() 
+       void = fftw_init_threads()
+       CALL dfftw_plan_with_nthreads(omp_get_max_threads())      
+#endif
+
        IF ( ldx /= nx .OR. ldy /= ny ) THEN
           IF( fw_plan(2,icurrent) /= 0 )  CALL dfftw_destroy_plan( fw_plan(2,icurrent) )
           IF( bw_plan(2,icurrent) /= 0 )  CALL dfftw_destroy_plan( bw_plan(2,icurrent) )
@@ -597,15 +697,6 @@
 
 #endif
 
-
-
-#elif defined __SCSL
-
-       CALL ZZFFTMR (0, ny, 0, 0.0_DP, DUMMY, 1, DUMMY, 1,               &
-                     tabley (1, icurrent), DUMMY, isys)
-       CALL ZZFFTM  (0, nx, 0, 0.0_DP, DUMMY, 1, DUMMY, 1,               &
-                     tablex (1, icurrent), DUMMY, isys)
-
 #elif defined __SX6
 
 
@@ -613,12 +704,6 @@
                   tabley (1, icurrent), work, isys)
        CALL ZZFFTM  (0, nx, 1, 1.0_DP, DUMMY, ldx, DUMMY, ldx,           &
                      tablex(1, icurrent), work, isys)
-
-#elif defined __SUNPERF
-
-       CALL zffti (ny, tabley (1, icurrent) )
-       CALL zffti (nx, tablex (1, icurrent) )
-
 #else
 
        CALL errore(' cft_2xy ',' no scalar fft driver specified ', 1)
@@ -641,9 +726,45 @@
 #endif
 
 
-#if defined __FFTW
+#if defined __DFTI
 
-#if defined __OPENMP
+     IF( isign < 0 ) THEN
+        !
+        dfti_status = DftiComputeForward(hand(ip)%desc, r(:))
+        IF(dfti_status /= 0)THEN
+           WRITE(*,*) "stopped in DftiComputeForward", dfti_status
+           STOP
+        ENDIF
+        !
+     ELSE IF( isign > 0 ) THEN
+        !
+        dfti_status = DftiComputeBackward(hand(ip)%desc, r(:))
+        IF(dfti_status /= 0)THEN
+           WRITE(*,*) "stopped in DftiComputeBackward", dfti_status
+           STOP
+        ENDIF
+        !
+     END IF
+
+
+#elif defined __FFTW
+
+#if defined __FFTW_ALL_XY_PLANES
+
+     IF( isign < 0 ) THEN
+        !
+        tscale = 1.0_DP / ( nx * ny )
+        !
+        CALL fftw_inplace_drv_2d( fw_plan_2d(ip), nzl, r(1), 1, ldx*ldy )
+        CALL ZDSCAL( ldx * ldy * nzl, tscale, r(1), 1)
+        !
+     ELSE IF( isign > 0 ) THEN
+        !
+        CALL fftw_inplace_drv_2d( bw_plan_2d(ip), nzl, r(1), 1, ldx*ldy )
+        !
+     END IF
+
+#elif defined __OPENMP
 
      nx_t  = nx
      ny_t  = ny
@@ -909,107 +1030,6 @@
 
      END IF
 
-#elif defined __SCSL
-
-      IF( isign < 0 ) THEN
-
-       idir = -1
-       tscale = 1.0_DP / (nx * ny)
-       DO k = 0, nzl-1
-          kk = k * ldx * ldy
-! FORWARD: ny FFTs in the X direction
-          CALL ZZFFTM ( idir, nx, ny, tscale, r(kk+1), ldx, r(kk+1), ldx,   &
-                        tablex (1, ip), work(1), isys )
-! FORWARD: nx FFTs in the Y direction
-          DO i = 1, nx
-             IF ( dofft(i) ) THEN
-!DIR$IVDEP
-!DIR$LOOP COUNT (50)
-                DO j = 0, ny-1
-                   XY(j+1) = r(i + (j) * ldx + kk)
-                END DO
-                CALL ZZFFT(idir, ny, 1.0_DP, XY, XY, tabley (1, ip),      &
-                           work(1), isys)
-!DIR$IVDEP
-!DIR$LOOP COUNT (50)
-                DO j = 0, ny-1
-                   r(i + (j) * ldx + kk) = XY(j+1)
-                END DO
-             END IF
-          END DO
-       END DO
-
-     ELSE IF ( isign > 0 ) THEN
-
-       idir = 1
-       tscale = 1.0_DP
-       DO k = 0, nzl-1
-! BACKWARD: nx FFTs in the Y direction
-          kk = (k) * ldx * ldy
-          DO i = 1, nx
-             IF ( dofft(i) ) THEN
-!DIR$IVDEP
-!DIR$LOOP COUNT (50)
-                DO j = 0, ny-1
-                   XY(j+1) = r(i + (j) * ldx + kk)
-                END DO
-                CALL ZZFFT(idir, ny, 1.0_DP, XY, XY, tabley (1, ip),      &
-                           work(1), isys)
-!DIR$IVDEP
-!DIR$LOOP COUNT (50)
-                DO j = 0, ny-1
-                   r(i + (j) * ldx + kk) = XY(j+1)
-                END DO
-             END IF
-          END DO
-! BACKWARD: ny FFTs in the X direction
-          CALL ZZFFTM ( idir, nx, ny, tscale, r(kk+1), ldx, r(kk+1), ldx,   &
-                        tablex (1, ip), work(1), isys )
-       END DO
-
-     END IF
-
-#elif defined __SUNPERF
-
-     IF ( isign < 0 ) THEN
-
-        DO k = 1, ny * nzl
-           kk = 1 + ( k - 1 ) * ldx
-           CALL zfftf ( nx, r (kk), tablex (1, ip) )
-        END DO
-
-        DO i = 1, nx
-           IF ( dofft(i) ) THEN
-              DO j = 1, nzl
-                 kk = (j - 1) * ldx * ny + i
-                 CALL ZCOPY (ny, r (kk), ldx, work, 1)
-                 CALL zfftf (ny, work, tabley (1, ip) )
-                 CALL ZCOPY (ny, work, 1, r (kk), ldx)
-              END DO
-           END IF
-        END DO
-        CALL ZDSCAL ( ldx * ny * nzl, 1.0_DP/(nx * ny), r, 1)
-
-     ELSE IF (isign > 0) THEN
-
-        DO i = 1, nx
-           IF ( dofft(i) ) THEN
-              DO j = 1, nzl
-                 kk = (j - 1) * ldx * ny + i
-                 CALL ZCOPY (ny, r (kk), ldx, work, 1)
-                 CALL zfftb (ny, work, tabley (1, ip) )
-                 CALL ZCOPY (ny, work, 1, r (kk), ldx)
-              END DO
-           END IF
-        END DO
-
-        DO k = 1, ny * nzl
-           kk = 1 + ( k - 1 ) * ldx
-           CALL zfftb ( nx, r (kk), tablex (1, ip) )
-        END DO
-
-     END IF
-
 #else
 
      CALL errore(' cft_2xy ',' no scalar fft driver specified ', 1)
@@ -1064,17 +1084,21 @@
      C_POINTER, save :: fw_plan(ndims) = 0
      C_POINTER, save :: bw_plan(ndims) = 0
 
-#elif defined __SCSL
+#elif defined __DFTI
 
-     INTEGER, PARAMETER :: ltabl = (2 * nfftx + 256)*3
-     REAL (DP), SAVE :: table (ltabl, ndims)
-     REAL (DP)       :: DUMMY
-     INTEGER, SAVE :: isys(0:1) = (/ 1, 1 /)
+     !   Intel MKL native FFT driver
 
-#elif defined __SUNPERF
-
-     INTEGER, PARAMETER :: ltabl = (4 * nfftx + 15)*3
-     REAL (DP), SAVE :: table (ltabl, ndims)
+     TYPE(DFTI_DESCRIPTOR_ARRAY), SAVE :: hand(ndims)
+     LOGICAL, SAVE :: dfti_first = .TRUE.
+     INTEGER :: dfti_status = 0
+     !
+     IF( dfti_first .EQ. .TRUE. ) THEN
+        DO ip = 1, ndims
+           hand(ip)%desc => NULL()
+        END DO
+ 
+        dfti_first = .FALSE.
+     END IF
 
 #elif defined __SX6
 
@@ -1154,19 +1178,57 @@
        CALL dfftw_plan_dft_3d ( bw_plan(icurrent), nx, ny, nz, f(1:), &
             f(1:), idir, FFTW_ESTIMATE)
 
+#elif defined __DFTI
+
+      if( ASSOCIATED( hand(icurrent)%desc ) ) THEN
+          dfti_status = DftiFreeDescriptor( hand(icurrent)%desc )
+          IF( dfti_status /= 0) THEN
+             WRITE(*,*) "stopped in DftiFreeDescriptor", dfti_status
+             STOP
+          ENDIF
+       END IF
+
+       dfti_status = DftiCreateDescriptor(hand(icurrent)%desc, DFTI_DOUBLE, DFTI_COMPLEX, 3,(/nx,ny,nz/))
+       IF(dfti_status /= 0) THEN
+          WRITE(*,*) "stopped in DftiCreateDescriptor", dfti_status
+          STOP
+       ENDIF
+       dfti_status = DftiSetValue(hand(icurrent)%desc, DFTI_NUMBER_OF_TRANSFORMS,1)
+       IF(dfti_status /= 0)THEN
+          WRITE(*,*) "stopped in DFTI_NUMBER_OF_TRANSFORMS", dfti_status
+          STOP
+       ENDIF
+       dfti_status = DftiSetValue(hand(icurrent)%desc, DFTI_PLACEMENT, DFTI_INPLACE)
+       IF(dfti_status /= 0)THEN
+         WRITE(*,*) "stopped in DFTI_PLACEMENT", dfti_status
+         STOP
+      ENDIF
+       tscale = 1.0_DP/ (nx * ny * nz)
+       dfti_status = DftiSetValue( hand(icurrent)%desc, DFTI_FORWARD_SCALE, tscale);
+       IF(dfti_status /= 0)THEN
+          WRITE(*,*) "stopped in DFTI_FORWARD_SCALE", dfti_status
+          STOP
+       ENDIF
+       tscale = 1.0_DP
+       dfti_status = DftiSetValue( hand(icurrent)%desc, DFTI_BACKWARD_SCALE, tscale );
+       IF(dfti_status /= 0)THEN
+          WRITE(*,*) "stopped in DFTI_BACKWARD_SCALE", dfti_status
+          STOP
+       ENDIF
+
+       dfti_status = DftiCommitDescriptor(hand(icurrent)%desc)
+       IF(dfti_status /= 0) THEN
+          WRITE(*,*) "stopped in DftiCreateDescriptor", dfti_status
+          STOP
+       ENDIF
+
+
+
+
 #elif defined __ESSL || defined __LINUX_ESSL
 
        ! no initialization for 3d FFT's from ESSL
 
-#elif defined __SCSL
-
-       CALL zzfft3d (0, nx, ny, nz, 0.0_DP, DUMMY, 1, 1, DUMMY, 1, 1, &
-                     table(1,icurrent), work(1), isys)
-
-#elif defined __SUNPERF
-
-       CALL zfft3i ( nx, ny, nz, table (1,icurrent) )
-
 #elif defined __SX6
 
 #  if defined ASL
@@ -1225,6 +1287,29 @@
 
    END IF
 
+#elif defined __DFTI
+
+     IF( isign < 0 ) THEN
+        !
+        dfti_status = DftiComputeForward(hand(ip)%desc, f(1:))
+        IF(dfti_status /= 0)THEN
+           WRITE(*,*) "stopped in DftiComputeForward", dfti_status
+           STOP
+        ENDIF
+        !
+     ELSE IF( isign > 0 ) THEN
+        !
+        dfti_status = DftiComputeBackward(hand(ip)%desc, f(1:))
+        IF(dfti_status /= 0)THEN
+           WRITE(*,*) "stopped in DftiComputeBackward", dfti_status
+           STOP
+        ENDIF
+        !
+     END IF
+
+
+
+
 #elif defined __ESSL || defined __LINUX_ESSL
 
      IF ( isign < 0 ) THEN
@@ -1238,30 +1323,6 @@
      IF( isign /= 0 ) CALL dcft3( f(1), ldx,ldx*ldy, f(1), ldx,ldx*ldy, &
           nx,ny,nz, idir, tscale, work(1), lwork)
 
-#elif defined __SCSL
-
-     IF ( isign /= 0 ) THEN
-        IF ( isign < 0 ) THEN
-           idir = -1
-           tscale = 1.0_DP / DBLE( nx * ny * nz )
-        ELSE IF ( isign > 0 ) THEN
-           idir = 1
-           tscale = 1.0_DP
-        END IF
-        CALL ZZFFT3D ( idir, nx, ny, nz, tscale, f(1), ldx, ldy,   &
-                       f(1), ldx, ldy, table(1,ip), work(1), isys )
-     END IF
-
-#elif defined __SUNPERF
-
-     IF( isign < 0 ) THEN
-        CALL zfft3f ( nx, ny, nz, f(1), ldx, ldy, table(1,ip), ltabl )
-        tscale = 1.0_DP / DBLE( nx * ny * nz )
-        CALL ZDSCAL ( ldx*ldy*ldz, tscale, f(1), 1 )
-     ELSE IF( isign > 0 ) THEN
-        CALL zfft3b ( nx, ny, nz, f(1), ldx, ldy, table(1,ip), ltabl )
-     ENDIF
-
 #elif defined __SX6
 
 #  if defined ASL
@@ -1650,7 +1711,7 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, isign, &
       integer, save :: icurrent = 1
       integer, save :: dims( 4, ndims ) = -1
 
-#if defined __FFTW || __FFTW3
+#if defined __FFTW || __FFTW3 || __DFTI
 
       C_POINTER, save :: bw_planz(  ndims ) = 0
       C_POINTER, save :: bw_planx(  ndims ) = 0
@@ -1664,15 +1725,6 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, isign, &
       real(dp), save :: aux2( ltabl, ndims )
       real(dp), save :: aux1( ltabl, ndims )
 
-#elif defined __SCSL
-
-     INTEGER, PARAMETER :: ltabl = 2 * nfftx + 256
-     real(dp), save :: bw_coeffz( ltabl,  ndims )
-     real(dp), save :: bw_coeffy( ltabl,  ndims )
-     real(dp), save :: bw_coeffx( ltabl,  ndims )
-     REAL(DP)    :: DUMMY
-     INTEGER, SAVE :: isys(0:1) = (/ 1, 1 /)
-
 #endif
 
       isign = -sgn
@@ -1756,15 +1808,6 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, isign, &
               tscale, aux2(1,icurrent), ltabl, work(1), lwork)
          end if
 
-#elif defined __SCSL
-
-         CALL ZZFFT (0, nz, 0.0_DP, DUMMY, 1, bw_coeffz(1, icurrent),    &
-                     work(1), isys)
-         CALL ZZFFT (0, ny, 0.0_DP, DUMMY, 1, bw_coeffy(1, icurrent),    &
-                     work(1), isys)
-         CALL ZZFFT (0, nx, 0.0_DP, DUMMY, 1, bw_coeffx(1, icurrent),    &
-                     work(1), isys)
-
 #else
 
         CALL errore(' cft_b ',' no scalar fft driver specified ', 1)
@@ -1794,7 +1837,7 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, isign, &
         call FFTW_INPLACE_DRV_1D( bw_planx(ip), ny, f((k-1)*ldx*ldy + 1), 1, ldx )
       end do   
 
-#elif defined __FFTW3
+#elif defined __FFTW3 || defined __DFTI
 
       call dfftw_execute_dft(bw_planz(ip), f(1:), f(1:))
       call dfftw_execute_dft(bw_planxy(ip), f(nstart:), f(nstart:))
@@ -1819,21 +1862,9 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, isign, &
              tscale, aux2(1,ip), ltabl, work(1), lwork)
       END DO
 
-#elif defined __SCSL
-
-      CALL ZZFFTMR (1, nz, ldx*ldy, tscale, f(1), ldx*ldy, f(1),        &
-                     ldx*ldy, bw_coeffz(1, ip), work(1), isys)
-      CALL ZZFFTM (1, nx, ldy*nplanes, tscale, f(nstart), ldx,          &
-                    f(nstart), ldx, bw_coeffx(1, ip), work(1), isys)
-      DO k = imin3, imax3
-        nstart = ( k - 1 ) * ldx * ldy + 1
-        CALL ZZFFTMR (1, ny, ldx, tscale, f(nstart), ldx, f(nstart),    &
-                      ldx, bw_coeffy(1, ip), work(1), isys)
-
-      END DO
-
 #endif
-     RETURN
+
+      RETURN
    END SUBROUTINE cft_b
 
 !
@@ -1859,7 +1890,7 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, isign, &
       !
 #if defined __FFTW
 
-!$omp single
+!$omp parallel
 
       IF( cft_b_bw_planz  == 0 ) THEN
          CALL CREATE_PLAN_1D( cft_b_bw_planz, nz, 1 )
@@ -1874,7 +1905,7 @@ SUBROUTINE cfft3ds (f, nx, ny, nz, ldx, ldy, ldz, isign, &
          cft_b_dims(2) = ny
       END IF
 
-!$omp end single
+!$omp end parallel
 
 #else
 
@@ -1976,6 +2007,7 @@ integer function good_fft_dimension (n)
   !
   if (mod (n, 2) ==0) nx = n + 1
   ! for nec vector machines: if n is even increase dimension by 1
+  !
 #endif
   !
   good_fft_dimension = nx
@@ -2060,7 +2092,7 @@ end function allowed
 !    nr  (input) tentative order n of a fft
 !
 !    np  (optional input) if present restrict the search of the order
-!        in the ensamble of multiples of np
+!        in the ensemble of multiples of np
 !
 !    Output: the same if n is a good number
 !         the closest higher number that is good
