@@ -15,6 +15,7 @@ MODULE london_module
   ! [ S. Grimme, J. Comp. Chem., 27, 1787 (2006) ].
   !
   USE kinds ,           ONLY : DP
+  USE parameters ,      ONLY : nsx
   !
   IMPLICIT NONE
   !
@@ -35,10 +36,11 @@ MODULE london_module
   ! r     ( 3 , mxr )     : ordered distance vectors
   ! dist2 ( mxr )         : ordered distances
   !
-  REAL ( DP ) , PUBLIC :: scal6 , lon_rcut
+  REAL ( DP ) , PUBLIC :: scal6=0._dp , lon_rcut=0._dp , in_C6 ( nsx )
   !
   ! scal6    : global scaling factor
   ! lon_rcut : public cut-off radius
+  ! in_C6 ( ntyp ) : input (user) specified atomic C6 coefficients
   !
   INTEGER , PRIVATE :: mxr
   !
@@ -204,7 +206,11 @@ MODULE london_module
            !
            i = atomic_number ( atom_label ( ilab ) )
            IF ( i > 0 .AND. i < 87 ) THEN
-              C6_i  ( ilab )  = vdw_coeffs(1,i)
+              IF ( in_C6 (ilab) > 0.0_DP ) THEN
+                 C6_i  ( ilab )  = in_C6 (ilab)
+              ELSE
+                 C6_i  ( ilab )  = vdw_coeffs(1,i)
+              END IF
               R_vdw ( ilab )  = vdw_coeffs(2,i)
            ELSE
              CALL errore ( ' init_london ' ,&
@@ -436,7 +442,7 @@ MODULE london_module
     ! ityp : type of each atom
     !
     REAL ( DP ) :: dist , f_damp , dtau ( 3 ) , force_london ( 3 , nat ) , &
-                   dist6 , dist7 , exparg , expval , par , fac , add
+                   dist6 , dist7 , exparg , expval , par , fac , add, aux(3)
     ! locals :
     ! dist         : distance R_ij between the current pair of atoms
     ! f_damp       :  damping function
@@ -501,7 +507,8 @@ MODULE london_module
            !
            par = beta / ( R_sum ( ityp ( atb ) , ityp ( ata ) ) )
            !
-!$omp parallel do private(nr,dist,dist6,dist7,exparg,expval,fac,add,ipol) default(shared), reduction(+:force_london)
+           aux(:) = 0.d0
+!$omp parallel do private(nr,dist,dist6,dist7,exparg,expval,fac,add,ipol) default(shared), reduction(+:aux)
            DO nr = 1 , nrm
             !
             dist  = alat * sqrt ( dist2 ( nr ) )
@@ -515,8 +522,8 @@ MODULE london_module
             add = 6.d0 / dist
             !
             DO ipol = 1 , 3
-              !
-              force_london ( ipol , ata ) = force_london ( ipol , ata ) + &
+              ! workaround for OpenMP on Intel compilers
+              aux(ipol) = aux(ipol) + &
                            ( scal6 / ( 1 + expval ) * fac * &
                            ( - par * expval / ( 1.d0 + expval ) + add ) * &
                            r ( ipol , nr ) * alat / dist )
@@ -525,6 +532,9 @@ MODULE london_module
             !
            END DO
 !$omp end parallel do 
+           DO ipol = 1 , 3
+              force_london ( ipol , ata ) = force_london ( ipol , ata ) + aux(ipol)
+           ENDDO
            !
          END IF
          !
