@@ -212,176 +212,6 @@
       RETURN
       END SUBROUTINE splitwf
 
-!=----------------------------------------------------------------------------=!
-
-      SUBROUTINE mergerho(rho, rhot, ngl, ig_l2g, mpime, nproc, root)
-
-! ... This subroutine merges the pieces of a charge density (rho) splitted across 
-! ... processors into a total charge (rhot) containing al the components
-! ... in a pre-defined order (the same as if only one processor is used)
-
-      USE kinds
-      USE parallel_include
-
-      IMPLICIT NONE
-
-      REAL(DP), INTENT(IN) :: rho(:)
-      REAL(DP), INTENT(OUT) :: rhot(:)
-      INTEGER, INTENT(IN) :: mpime, nproc, root
-      INTEGER, INTENT(IN) :: ig_l2g(:)
-      INTEGER, INTENT(IN) :: ngl
-
-      INTEGER, ALLOCATABLE :: ig_ip(:)
-      REAL(DP), ALLOCATABLE :: rho_ip(:)
-
-      INTEGER :: ierr, i, ip, ng_ip, ng_lmax, ng_g
-
-#if defined __MPI
-      INTEGER :: istatus(MPI_STATUS_SIZE)
-#endif
-
-#if defined __MPI
-
-! ... Get local and global wavefunction dimensions
-      CALL MPI_ALLREDUCE(ngl, ng_lmax, 1, MPI_INTEGER, MPI_MAX, MPI_COMM_WORLD, ierr)
-      CALL MPI_ALLREDUCE(ngl, ng_g   , 1, MPI_INTEGER, MPI_SUM, MPI_COMM_WORLD, ierr)
-      IF( ng_g > SIZE( rhot ) ) THEN
-        CALL errore(' mergerho ',' wrong size for rho ',1 )
-      END IF
-
-      DO ip = 1, nproc
-
-        IF( (ip-1) /= root ) THEN
-
-! ...     In turn each processors send to root the rho components and their indexes in the 
-! ...     global array
-          IF ( mpime == (ip-1) ) THEN
-            CALL MPI_SEND( ig_l2g, ngl, MPI_INTEGER, root, ip, MPI_COMM_WORLD, ierr)
-            CALL MPI_SEND( rho(1), ngl, MPI_DOUBLE_PRECISION, root, ip+nproc, MPI_COMM_WORLD,ierr)
-          END IF
-          IF ( mpime == root ) THEN
-            ALLOCATE( ig_ip(ng_lmax) )
-            ALLOCATE( rho_ip(ng_lmax) )
-            CALL MPI_RECV( ig_ip, ng_lmax, MPI_INTEGER, (ip-1), ip, MPI_COMM_WORLD, istatus, ierr )
-            CALL MPI_RECV( rho_ip, ng_lmax, MPI_DOUBLE_PRECISION, (ip-1), ip+nproc, MPI_COMM_WORLD, istatus, ierr )
-            CALL MPI_GET_COUNT(istatus, MPI_DOUBLE_PRECISION, ng_ip, ierr)
-            DO I = 1, ng_ip
-              rhot(ig_ip(i)) = rho_ip(i)
-            END DO
-            DEALLOCATE(ig_ip)
-            DEALLOCATE(rho_ip)
-          END IF
-
-        ELSE
-
-          IF(mpime == root) THEN
-            DO I = 1, ngl
-              rhot(ig_l2g(i)) = rho(i)
-            END DO
-          END IF
-
-        END IF
-
-        CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)
-
-      END DO
-
-#elif ! defined __MPI
-
-      DO I = 1, ngl
-        rhot( ig_l2g(i) ) = rho(i)
-      END DO
-
-#else
-
-      CALL errore(' mergerho ',' no communication protocol ',0)
-
-#endif
-
-      RETURN
-      END SUBROUTINE mergerho
-
-
-      SUBROUTINE splitrho(rho, rhot, ngl, ig_l2g, mpime, nproc, root)
-
-! ... This subroutine splits rho containing al the G-vecs components
-! ... in a pre-defined order (the same as if only one processor is used), across 
-! ... processors (rho).
-
-      USE kinds
-      USE parallel_include
-      IMPLICIT NONE
-
-      REAL(DP), INTENT(OUT) :: rho(:)
-      REAL(DP), INTENT(IN) :: rhot(:)
-      INTEGER, INTENT(IN) :: mpime, nproc, root
-      INTEGER, INTENT(IN) :: ig_l2g(:)
-      INTEGER, INTENT(IN) :: ngl
-
-      INTEGER :: ierr, i, ng_ip, ip, ng_lmax, ng_g
-
-#if defined __MPI
-      INTEGER :: istatus(MPI_STATUS_SIZE)
-#endif
-
-      INTEGER, ALLOCATABLE :: ig_ip(:)
-      COMPLEX(DP), ALLOCATABLE :: rho_ip(:)
-
-#if defined __MPI
-
-! ... Get local and global rho dimensions
-      CALL MPI_ALLREDUCE(ngl, ng_lmax, 1, MPI_INTEGER, MPI_MAX, MPI_COMM_WORLD, ierr)
-      CALL MPI_ALLREDUCE(ngl, ng_g   , 1, MPI_INTEGER, MPI_SUM, MPI_COMM_WORLD, ierr)
-      IF( ng_g > SIZE( rhot ) ) THEN
-        CALL errore(' splitrho ',' wrong size for rhot ', 1 )
-      END IF
-
-      DO ip = 1, nproc
-! ...   In turn each processor send to root the the indexes of its rho conponents
-! ...   Root receive the indexes and send the componens of the rho read from the disk (rhot)
-        IF ( (ip-1) /= root ) THEN
-          IF ( mpime == (ip-1) ) THEN
-            CALL MPI_SEND( ig_l2g, ngl, MPI_INTEGER, root, ip, MPI_COMM_WORLD, ierr)
-            CALL MPI_RECV( rho(1), ngl, MPI_DOUBLE_PRECISION, root, ip+nproc, MPI_COMM_WORLD, istatus, ierr )
-          END IF
-          IF ( mpime == root ) THEN
-            ALLOCATE(ig_ip(ng_lmax))
-            ALLOCATE(rho_ip(ng_lmax))
-            CALL MPI_RECV( ig_ip, ng_lmax, MPI_INTEGER, (ip-1), IP, MPI_COMM_WORLD, istatus, ierr )
-            CALL MPI_GET_COUNT(istatus, MPI_INTEGER, ng_ip, ierr)
-            DO i = 1, ng_ip
-              rho_ip(i) = rhot(ig_ip(i))
-            END DO
-            CALL MPI_SEND( rho_ip, ng_ip, MPI_DOUBLE_PRECISION, (ip-1), ip+nproc, MPI_COMM_WORLD, ierr)
-            DEALLOCATE(ig_ip)
-            DEALLOCATE(rho_ip)
-          END IF
-        ELSE
-          IF ( mpime == root ) THEN
-            DO i = 1, ngl
-              rho(i) = rhot(ig_l2g(i)) 
-            END DO
-          END IF
-        END IF
-        CALL MPI_BARRIER(MPI_COMM_WORLD, ierr)
-      END DO
-
-#elif ! defined __MPI
-
-      DO i = 1, ngl
-         rho(i) = rhot( ig_l2g(i) ) 
-      END DO
-
-#else
-
-      CALL errore(' splitrho ',' no communication protocol ',0)
-
-#endif
-
-      RETURN
-      END SUBROUTINE splitrho
-
-!=----------------------------------------------------------------------------=!
 
 
       SUBROUTINE mergeig(igl, igtot, ngl, mpime, nproc, root, comm)
@@ -775,6 +605,7 @@ SUBROUTINE redistwf( c_dist_pw, c_dist_st, npw_p, nst_p, comm, idir )
    RETURN
 END SUBROUTINE redistwf
 
+!=----------------------------------------------------------------------------=!
 
 SUBROUTINE redistwfr( c_dist_pw, c_dist_st, npw_p, nst_p, comm, idir )
    !
