@@ -126,6 +126,9 @@ MODULE read_namelists_module
        saverho = .TRUE.
        memory = 'default'
        !
+       lfcpopt = .FALSE.
+       lfcpdyn = .FALSE.
+       !
        RETURN
        !
      END SUBROUTINE
@@ -245,6 +248,7 @@ MODULE read_namelists_module
        london      = .false.
        london_s6   = 0.75_DP
        london_rcut = 200.00_DP
+       london_c6   = -1.0_DP
        ts_vdw          = .FALSE.
        ts_vdw_isolated = .FALSE.
        ts_vdw_econv_thr = 1.E-6_DP
@@ -257,10 +261,25 @@ MODULE read_namelists_module
        esm_bc='pbc'
        esm_efield=0.0_DP
        esm_w=0.0_DP
+       esm_a=0.0_DP
+       esm_zb=-2.0_DP
        esm_nfit=4
        esm_debug=.FALSE.
        esm_debug_gpmax=0
        !
+       ! ... FCP
+       !
+       fcp_mu          = 0.0_DP
+       fcp_mass        = 10000.0_DP
+       fcp_tempw       = 0.0_DP
+       fcp_relax_step  = 0.5_DP
+       fcp_relax_crit  = 0.001_DP
+       !
+       space_group=0
+       uniqueb = .FALSE.
+       origin_choice = 1
+       rhombohedral = .TRUE.
+       !
        RETURN
        !
      END SUBROUTINE
@@ -283,12 +302,12 @@ MODULE read_namelists_module
        emass = 400.0_DP
        emass_cutoff = 2.5_DP
        orthogonalization = 'ortho'
-       ortho_eps = 1.E-8_DP
-       ortho_max = 20
+       ortho_eps = 1.E-9_DP
+       ortho_max = 300
        electron_maxstep = 100
        scf_must_converge = .true.
        !
-       ! ... ( 'sd' | 'cg' | 'damp' | 'verlet' | 'none' | 'diis' )
+       ! ... ( 'sd' | 'cg' | 'damp' | 'verlet' | 'none' | 'diis' | 'cp-bo' )
        !
        electron_dynamics = 'none'
        electron_damping = 0.1_DP
@@ -361,6 +380,7 @@ MODULE read_namelists_module
        efield_cart(1)=0.d0
        efield_cart(2)=0.d0
        efield_cart(3)=0.d0
+       efield_phase='none'
        !
        occupation_constraints = .false.
        !
@@ -368,6 +388,13 @@ MODULE read_namelists_module
        conv_thr_init  =  0.1E-2_DP
        conv_thr_multi =  0.1_DP
        !
+       ! ... CP-BO ...
+       tcpbo = .false.
+       emass_emin = 200.0_DP
+       emass_cutoff_emin = 6.0_DP
+       electron_damping_emin = 0.35_DP
+       dt_emin = 4.0_DP
+       !
        RETURN
        !
      END SUBROUTINE
@@ -411,11 +438,6 @@ MODULE read_namelists_module
        !
        CHARACTER(LEN=2) :: prog   ! ... specify the calling program
        !
-       !
-       ! ... ( 'full' | 'coarse-grained' )
-       !
-       phase_space = 'full'
-       !
        ! ... ( 'sd' | 'cg' | 'damp' | 'verlet' | 'none' | 'bfgs' | 'beeman' )
        !
        ion_dynamics = 'none'
@@ -466,16 +488,6 @@ MODULE read_namelists_module
        w_1              = 0.01_DP
        w_2              = 0.50_DP
        !
-       sic_rloc = 0.0_DP
-       !
-       ! ... meta-dynamics defaults
-       !
-       fe_step     = 0.4_DP
-       fe_nstep    = 100
-       sw_nstep    = 10
-       eq_nstep    = 0
-       g_amplitude = 0.005_DP
-       !
        RETURN
        !
      END SUBROUTINE
@@ -520,7 +532,7 @@ MODULE read_namelists_module
        cell_dofree = 'all'
        cell_factor = 0.0_DP
        cell_nstepe = 1
-       cell_damping = 0.0_DP
+       cell_damping = 0.1_DP
        press_conv_thr = 0.5_DP
        !
        RETURN
@@ -601,13 +613,15 @@ MODULE read_namelists_module
        wf_q        = 1500.0_DP
        wf_friction = 0.3_DP
 !=======================================================================
-!Lingzhu Kong
-       neigh       = 48
-       vnbsp       = 0
-       poisson_eps = 1.D-6
-       dis_cutoff  = 7.0_DP
-       exx_ps_rcut = 5.0
-       exx_me_rcut = 10.0
+!exx_wf related
+       exx_neigh        =  60 
+       vnbsp            =  0
+       exx_poisson_eps  =  1.E-6_DP
+       exx_dis_cutoff   =  8.0_DP
+       exx_ps_rcut_self =  6.0_DP
+       exx_ps_rcut_pair =  5.0_DP
+       exx_me_rcut_self = 10.0_DP
+       exx_me_rcut_pair =  7.0_DP
 !=======================================================================
        !
        nit    = 10
@@ -688,6 +702,8 @@ MODULE read_namelists_module
        CALL mp_bcast( tqmmm,         ionode_id, intra_image_comm )
        CALL mp_bcast( vdw_table_name,ionode_id, intra_image_comm )
        CALL mp_bcast( memory,        ionode_id, intra_image_comm )
+       CALL mp_bcast( lfcpopt,       ionode_id, intra_image_comm )
+       CALL mp_bcast( lfcpdyn,       ionode_id, intra_image_comm )
        !
        RETURN
        !
@@ -803,6 +819,7 @@ MODULE read_namelists_module
        CALL mp_bcast( london,                    ionode_id, intra_image_comm )
        CALL mp_bcast( london_s6,                 ionode_id, intra_image_comm )
        CALL mp_bcast( london_rcut,               ionode_id, intra_image_comm )
+       CALL mp_bcast( london_c6,                 ionode_id, intra_image_comm )
        CALL mp_bcast( xdm,                       ionode_id, intra_image_comm )
        CALL mp_bcast( xdm_a1,                    ionode_id, intra_image_comm )
        CALL mp_bcast( xdm_a2,                    ionode_id, intra_image_comm )
@@ -814,9 +831,27 @@ MODULE read_namelists_module
        CALL mp_bcast( esm_bc,             ionode_id, intra_image_comm )
        CALL mp_bcast( esm_efield,         ionode_id, intra_image_comm )
        CALL mp_bcast( esm_w,              ionode_id, intra_image_comm )
+       CALL mp_bcast( esm_a,              ionode_id, intra_image_comm )
+       CALL mp_bcast( esm_zb,             ionode_id, intra_image_comm )
        CALL mp_bcast( esm_nfit,           ionode_id, intra_image_comm )
        CALL mp_bcast( esm_debug,          ionode_id, intra_image_comm )
        CALL mp_bcast( esm_debug_gpmax,    ionode_id, intra_image_comm )
+       !
+       ! ... FCP
+       !
+       CALL mp_bcast( fcp_mu,          ionode_id, intra_image_comm )
+       CALL mp_bcast( fcp_mass,        ionode_id, intra_image_comm )
+       CALL mp_bcast( fcp_tempw,       ionode_id, intra_image_comm )
+       CALL mp_bcast( fcp_relax_step,  ionode_id, intra_image_comm )
+       CALL mp_bcast( fcp_relax_crit,  ionode_id, intra_image_comm )
+       !
+       !
+       ! ... space group information
+       !
+       CALL mp_bcast( space_group,        ionode_id, intra_image_comm )
+       CALL mp_bcast( uniqueb,            ionode_id, intra_image_comm )
+       CALL mp_bcast( origin_choice,      ionode_id, intra_image_comm )
+       CALL mp_bcast( rhombohedral,       ionode_id, intra_image_comm )
 
        RETURN
        !
@@ -917,6 +952,7 @@ MODULE read_namelists_module
        CALL mp_bcast( epol2,   ionode_id, intra_image_comm )
        CALL mp_bcast( efield2, ionode_id, intra_image_comm )
        CALL mp_bcast( efield_cart,   ionode_id, intra_image_comm )
+       CALL mp_bcast( efield_phase,   ionode_id, intra_image_comm )
        !
        ! ... occupation constraints ...
        !
@@ -927,6 +963,14 @@ MODULE read_namelists_module
        CALL mp_bcast( adaptive_thr,       ionode_id, intra_image_comm )
        CALL mp_bcast( conv_thr_init,      ionode_id, intra_image_comm )
        CALL mp_bcast( conv_thr_multi,     ionode_id, intra_image_comm )
+       !
+       ! ... CP-BO ...
+       CALL mp_bcast( tcpbo,                 ionode_id, intra_image_comm )
+       CALL mp_bcast( emass_emin,            ionode_id, intra_image_comm )
+       CALL mp_bcast( emass_cutoff_emin,     ionode_id, intra_image_comm )
+       CALL mp_bcast( electron_damping_emin, ionode_id, intra_image_comm )
+       CALL mp_bcast( dt_emin,               ionode_id, intra_image_comm )
+       !
        RETURN
        !
      END SUBROUTINE
@@ -948,7 +992,6 @@ MODULE read_namelists_module
        !
        IMPLICIT NONE
        !
-       CALL mp_bcast( phase_space,       ionode_id, intra_image_comm )
        CALL mp_bcast( ion_dynamics,      ionode_id, intra_image_comm )
        CALL mp_bcast( ion_radius,        ionode_id, intra_image_comm )
        CALL mp_bcast( ion_damping,       ionode_id, intra_image_comm )
@@ -985,14 +1028,6 @@ MODULE read_namelists_module
        CALL mp_bcast( w_1,              ionode_id, intra_image_comm )
        CALL mp_bcast( w_2,              ionode_id, intra_image_comm )
        !
-       CALL mp_bcast( sic_rloc, ionode_id, intra_image_comm )
-       !
-       CALL mp_bcast( fe_step,     ionode_id, intra_image_comm )
-       CALL mp_bcast( fe_nstep,    ionode_id, intra_image_comm )
-       CALL mp_bcast( sw_nstep,    ionode_id, intra_image_comm )
-       CALL mp_bcast( eq_nstep,    ionode_id, intra_image_comm )
-       CALL mp_bcast( g_amplitude, ionode_id, intra_image_comm )
-       !
        RETURN
        !
      END SUBROUTINE
@@ -1114,6 +1149,16 @@ MODULE read_namelists_module
        CALL mp_bcast( nwf,         ionode_id, intra_image_comm )
        CALL mp_bcast( wffort,      ionode_id, intra_image_comm )
        CALL mp_bcast( writev,      ionode_id, intra_image_comm )
+!=================================================================
+!exx_wf related
+       CALL mp_bcast( exx_neigh,       ionode_id, intra_image_comm )
+       CALL mp_bcast( exx_poisson_eps, ionode_id, intra_image_comm )
+       CALL mp_bcast( exx_dis_cutoff,  ionode_id, intra_image_comm )
+       CALL mp_bcast( exx_ps_rcut_self, ionode_id, intra_image_comm )
+       CALL mp_bcast( exx_ps_rcut_pair, ionode_id, intra_image_comm )
+       CALL mp_bcast( exx_me_rcut_self, ionode_id, intra_image_comm )
+       CALL mp_bcast( exx_me_rcut_pair, ionode_id, intra_image_comm )
+       CALL mp_bcast( vnbsp,       ionode_id, intra_image_comm )
        !
        RETURN
        !
@@ -1422,13 +1467,6 @@ MODULE read_namelists_module
        LOGICAL           :: allowed = .FALSE.
        !
        !
-       DO i = 1, SIZE( phase_space_allowed )
-          IF( TRIM( phase_space ) == phase_space_allowed(i) ) allowed = .TRUE.
-       END DO
-       IF ( .NOT. allowed ) &
-          CALL errore( sub_name, ' phase_space '''// &
-                       & TRIM( phase_space )// ''' not allowed ', 1 )
-       !
        allowed = .FALSE.
        DO i = 1, SIZE(ion_dynamics_allowed)
           IF( TRIM(ion_dynamics) == ion_dynamics_allowed(i) ) allowed = .TRUE.
@@ -1449,9 +1487,6 @@ MODULE read_namelists_module
        IF( ion_maxstep < 0 ) &
           CALL errore( sub_name,' ion_maxstep out of range ',1)
        !
-       IF (sic /= 'none' .and. sic_rloc == 0.0_DP) &
-          CALL errore( sub_name, ' invalid sic_rloc with sic activated ', 1 )
-       !
        RETURN
        !
      END SUBROUTINE
@@ -1586,9 +1621,19 @@ MODULE read_namelists_module
              IF ( prog == 'PW' ) &
                 CALL errore( sub_name, ' calculation ' // &
                            & TRIM( calculation ) // ' not implemented ', 1 )
+          CASE ( 'vc-cp-wf' )
+             IF( prog == 'CP' ) THEN
+                electron_dynamics = 'verlet'
+                ion_dynamics      = 'verlet'
+                cell_dynamics     = 'pr'
+             ELSE IF( prog == 'PW' ) THEN
+                CALL errore( sub_name, ' calculation ' // &
+                           & TRIM( calculation ) // ' not implemented ', 1 )
+             END IF
+             !
 !=========================================================================
 !Lingzhu Kong
-          CASE ( 'cp-wf-nscf','cp-wf-pbe0','pbe0-nscf' )
+          CASE ( 'cp-wf-nscf' )
              IF( prog == 'CP' ) THEN
                 occupations       = 'fixed'
                 electron_dynamics = 'damp'
@@ -1795,9 +1840,8 @@ MODULE read_namelists_module
                TRIM( calculation ) == 'cp'       .OR. &
                TRIM( calculation ) == 'vc-cp'    .OR. &
                TRIM( calculation ) == 'smd'      .OR. &
-               TRIM( calculation ) == 'cp-wf-nscf' .OR. &   !Lingzhu Kong
-               TRIM( calculation ) == 'cp-wf-pbe0' .OR. &   !Lingzhu Kong
-               TRIM( calculation ) == 'pbe0-nscf'  .OR. &   !Lingzhu Kong
+               TRIM( calculation ) == 'cp-wf-nscf' .OR. &
+               TRIM( calculation ) == 'vc-cp-wf'   .OR. &
                TRIM( calculation ) == 'cp-wf' ) READ( unit_loc, ions, iostat = ios )
   
        END IF
@@ -1817,7 +1861,8 @@ MODULE read_namelists_module
           IF( TRIM( calculation ) == 'vc-relax' .OR. &
               TRIM( calculation ) == 'vc-cp'    .OR. &
               TRIM( calculation ) == 'vc-md'    .OR. &
-              TRIM( calculation ) == 'vc-md' ) THEN
+              TRIM( calculation ) == 'vc-md'    .OR. & 
+              TRIM( calculation ) == 'vc-cp-wf') THEN
              READ( unit_loc, cell, iostat = ios )
           END IF
        END IF
@@ -1849,10 +1894,9 @@ MODULE read_namelists_module
        CALL wannier_defaults( prog )
        ios = 0
        IF( ionode ) THEN
-          IF( TRIM( calculation ) == 'cp-wf'       .OR. & ! Lingzhu Kong
-              TRIM( calculation ) == 'cp-wf-nscf'  .OR. & ! Lingzhu Kong
-              TRIM( calculation ) == 'cp-wf-pbe0'  .OR. & ! Lingzhu Kong
-              TRIM( calculation ) == 'pbe0-nscf' ) THEN   ! Lingzhu Kong
+          IF( TRIM( calculation ) == 'cp-wf'       .OR. &
+              TRIM( calculation ) == 'vc-cp-wf'    .OR. &
+              TRIM( calculation ) == 'cp-wf-nscf') THEN
              READ( unit_loc, wannier, iostat = ios )
           END IF
        END IF
