@@ -221,14 +221,12 @@
 ! ...     Maximum number of sticks (wave func.)
           nstpwx = maxval( nstpw  )
 
-          IF( dffts%have_task_groups ) THEN
-            !
-            !  Initialize task groups.
-            !  Note that this call modify dffts adding task group data.
-            !
-            CALL task_groups_init( dffts )
-            !
-          END IF
+          !
+          !  Initialize task groups.
+          !  Note that this call modify dffts adding task group data.
+          !
+          CALL task_groups_init( dffts )
+          !
 
           IF (ionode) THEN
              WRITE( stdout,*)
@@ -452,42 +450,6 @@
 ! ...     Maximum number of sticks (wave func.)
           nstpwx = maxval( nstpw  )
 
-!          IF( dffts%have_task_groups ) THEN
-            !
-            !  Initialize task groups.
-            !  Note that this call modify dffts adding task group data.
-            !
-!            CALL task_groups_init( dffts )
-            !
-!          END IF
-
-!!$          IF (ionode) THEN
-!!$             WRITE( stdout,*)
-!!$             IF ( nproc > 1 ) THEN
-!!$                WRITE( stdout, '(5X,"Parallelization info")')
-!!$             ELSE
-!!$                WRITE( stdout, '(5X,"G-vector sticks info")')
-!!$             ENDIF
-!!$             WRITE( stdout, '(5X,"--------------------")')
-!!$             WRITE( stdout, '(5X,"sticks:   dense  smooth     PW", &
-!!$                            & 5X,"G-vecs:    dense   smooth      PW")') 
-!!$             IF ( nproc > 1 ) THEN
-!!$                WRITE( stdout,'(5X,"Min",4X,2I8,I7,12X,2I9,I8)') &
-!!$                   minval(nstp), minval(nstps), minval(nstpw), &
-!!$                   minval(sstp), minval(sstps), minval(sstpw)
-!!$                WRITE( stdout,'(5X,"Max",4X,2I8,I7,12X,2I9,I8)') &
-!!$                   maxval(nstp), maxval(nstps), maxval(nstpw), &
-!!$                   maxval(sstp), maxval(sstps), maxval(sstpw)
-!!$             END IF
-!!$             WRITE( stdout,'(5X,"Sum",4X,2I8,I7,12X,2I9,I8)') &
-!!$                sum(nstp), sum(nstps), sum(nstpw), &
-!!$                sum(sstp), sum(sstps), sum(sstpw)
-!!$             ! in the case k=0, the lines above and below differ:
-!!$             ! above all sticks, below only those in the half sphere
-!!$             IF ( .NOT. tk ) &
-!!$                 WRITE( stdout,'(5X,"Tot",4X,2I8,I7)') nst, nsts, nstw
-!!$          ENDIF
-
           DEALLOCATE( ist )
           DEALLOCATE( idx )
 
@@ -558,7 +520,7 @@ SUBROUTINE task_groups_init( dffts )
    !C. Bekas...TASK GROUP RELATED. FFT DATA STRUCTURES ARE ALREADY DEFINED ABOVE
    !-------------------------------------------------------------------------------------
    !dfft%nsw(me) holds the number of z-sticks for the current processor per wave-function
-   !We can either send these in the group with an mpi_allgather...or put the
+   !We can either send these in the group with an mpi_allgather...or put them
    !in the PSIS vector (in special positions) and send them with them.
    !Otherwise we can do this once at the beginning, before the loop.
    !we choose to do the latter one.
@@ -630,28 +592,30 @@ SUBROUTINE task_groups_init_first( dffts )
        pgroup( i ) = i - 1
     ENDDO
     !
-    !LIST OF PROCESSORS IN MY ORBITAL GROUP
+    !LIST OF PROCESSORS IN MY ORBITAL GROUP 
+    !     (processors dealing with my same pw's of different orbitals)
     !
     !  processors in these group have contiguous indexes
     !
-    n1 = ( dffts%mype / dffts%nogrp ) * dffts%nogrp - 1
+    n1 = ( dffts%mype / dffts%nogrp ) * dffts%nogrp 
+    ipos = dffts%mype - n1
     DO i = 1, dffts%nogrp
-       dffts%nolist( i ) = pgroup( n1 + i + 1 )
-       IF( dffts%mype == dffts%nolist( i ) ) ipos = i - 1
+       dffts%nolist( i ) = pgroup( n1 + i )
     ENDDO
     !
     !LIST OF PROCESSORS IN MY PLANE WAVE GROUP
+    !     (processors dealing with different pw's of my same orbital)
     !
-    DO I = 1, dffts%npgrp
+    DO i = 1, dffts%npgrp
        dffts%nplist( i ) = pgroup( ipos + ( i - 1 ) * dffts%nogrp + 1 )
     ENDDO
     !
     !SET UP THE GROUPS
     !
-    !
     !CREATE ORBITAL GROUPS
     !
 #if defined __MPI
+    ! processes with the same color are in the same new communicator
     color = dffts%mype / dffts%nogrp
     key   = MOD( dffts%mype , dffts%nogrp )
     CALL MPI_COMM_SPLIT( dffts%comm, color, key, dffts%ogrp_comm, ierr )
@@ -670,6 +634,7 @@ SUBROUTINE task_groups_init_first( dffts )
     !CREATE PLANEWAVE GROUPS
     !
 #if defined __MPI
+    ! processes with the same color are in the same new communicator
     color = MOD( dffts%mype , dffts%nogrp )
     key   = dffts%mype / dffts%nogrp
     CALL MPI_COMM_SPLIT( dffts%comm, color, key, dffts%pgrp_comm, ierr )
