@@ -18,7 +18,7 @@ SUBROUTINE chdens (filplot,plot_num)
   USE kinds,      ONLY : dp
   USE io_global,  ONLY : stdout, ionode, ionode_id
   USE io_files,   ONLY : nd_nmbr
-  USE mp_global,  ONLY : nproc_pool
+  USE mp_pools,   ONLY : nproc_pool
   USE mp_world,   ONLY : world_comm
   USE mp,         ONLY : mp_bcast
   USE parameters, ONLY : ntypx
@@ -26,9 +26,9 @@ SUBROUTINE chdens (filplot,plot_num)
   USE cell_base,  ONLY : at, bg, celldm, ibrav, alat, omega, tpiba, tpiba2
   USE ions_base,  ONLY : nat, ityp, atm, ntyp => nsp, tau, zv
   USE lsda_mod,   ONLY : nspin
-  USE fft_base,   ONLY : grid_scatter, dfftp, dffts
+  USE fft_base,   ONLY : scatter_grid, dfftp, dffts
   USE fft_interfaces,  ONLY : fwfft
-  USE grid_subroutines,ONLY : realspace_grids_init
+  USE grid_subroutines,ONLY : realspace_grid_init
   USE gvect,      ONLY : ngm, nl, g, gcutm
   USE gvecs,      ONLY : gcutms, doublegrid, dual, ecuts 
   USE recvec_subs,ONLY: ggen 
@@ -273,7 +273,8 @@ SUBROUTINE chdens (filplot,plot_num)
 
      CALL recips (at(1,1), at(1,2), at(1,3), bg(1,1), bg(1,2), bg(1,3) )
      CALL volume (alat, at(1,1), at(1,2), at(1,3), omega)
-     CALL realspace_grids_init ( dfftp, dffts, at, bg, gcutm, gcutms )
+     CALL realspace_grid_init ( dfftp, at, bg, gcutm )
+     CALL realspace_grid_init ( dffts, at, bg, gcutms)
   ENDIF
 
   ALLOCATE  (rhor(dfftp%nr1x*dfftp%nr2x*dfftp%nr3x))
@@ -406,7 +407,7 @@ SUBROUTINE chdens (filplot,plot_num)
         !
      ELSE
         !
-        IF (gamma_only) THEN
+        IF (gamma_only .and. (trim(interpolation) == 'fourier')) THEN
              WRITE(stdout,'(/"BEWARE: plot requiring G-space interpolation",&
                             &" not implemented for Gamma only!",/, &
                             &"SOLUTION: restart this calculation with", &
@@ -417,7 +418,7 @@ SUBROUTINE chdens (filplot,plot_num)
      ENDIF
 #ifdef __MPI
      ALLOCATE(aux(dfftp%nnr))
-     CALL grid_scatter(rhor, aux)
+     CALL scatter_grid(dfftp, rhor, aux)
      psic(:) = cmplx(aux(:), 0.d0,kind=DP)
      DEALLOCATE(aux)
 #else
@@ -500,7 +501,7 @@ SUBROUTINE chdens (filplot,plot_num)
 
      ELSEIF (ionode) THEN
         !
-        ! GOPENMOL FORMAT
+        ! GOPENMOL OR XCRYSDEN FORMAT
         !
         IF (fast3d) THEN
 
@@ -551,7 +552,7 @@ SUBROUTINE plot_1d (nx, m1, x0, e, ngm, g, rhog, alat, iflag, ounit)
   USE kinds, ONLY : DP
   USE constants, ONLY:  pi
   USE io_global, ONLY : stdout, ionode
-  USE mp_global,  ONLY : intra_bgrp_comm
+  USE mp_bands,  ONLY : intra_bgrp_comm
   USE mp,         ONLY : mp_sum
 
   IMPLICIT NONE
@@ -680,7 +681,7 @@ SUBROUTINE plot_2d (nx, ny, m1, m2, x0, e1, e2, ngm, g, rhog, alat, &
   USE kinds, ONLY : DP
   USE constants, ONLY : pi
   USE io_global, ONLY : stdout, ionode
-  USE mp_global,  ONLY : intra_bgrp_comm
+  USE mp_bands,  ONLY : intra_bgrp_comm
   USE mp,         ONLY : mp_sum
   IMPLICIT NONE
   INTEGER :: nx, ny, ngm, nat, ityp (nat), output_format, ounit
@@ -826,7 +827,7 @@ SUBROUTINE plot_2ds (nx, ny, x0, ngm, g, rhog, output_format, ounit)
   USE kinds, ONLY : DP
   USE constants, ONLY:  pi
   USE io_global, ONLY : stdout, ionode
-  USE mp_global,  ONLY : intra_bgrp_comm
+  USE mp_bands,  ONLY : intra_bgrp_comm
   USE mp,         ONLY : mp_sum
   !
   IMPLICIT NONE
@@ -944,7 +945,7 @@ SUBROUTINE plot_3d (alat, at, nat, tau, atm, ityp, ngm, g, rhog, &
   USE kinds, ONLY : DP
   USE constants, ONLY:  pi
   USE io_global, ONLY : stdout, ionode
-  USE mp_global,  ONLY : intra_bgrp_comm
+  USE mp_bands,  ONLY : intra_bgrp_comm
   USE mp,         ONLY : mp_sum
   IMPLICIT NONE
   INTEGER :: nat, ityp (nat), ngm, nx, ny, nz, output_format, ounit
