@@ -164,9 +164,10 @@ PROGRAM pw2bgw
   character (len=256), external :: trimcheck
   character (len=1), external :: lowercase
 
-#ifdef __PARA
-  CALL mp_startup ( world_comm )
+#ifdef __MPI
+  CALL mp_startup ( )
 #endif
+
   CALL environment_start ( codename )
 
   prefix = 'prefix'
@@ -280,6 +281,16 @@ PROGRAM pw2bgw
   if (okpaw) call errore ( 'pw2bgw', 'BGW cannot use PAW.', 4 )
   if (gamma_only) call errore ( 'pw2bgw', 'BGW cannot use gamma-only run.', 5 )
   if (nspin == 4) call errore ( 'pw2bgw', 'BGW cannot use spinors.', 6 )
+  if (real_or_complex == 1 .AND. vxc_flag .AND. vxc_offdiag_nmax > 0) &
+    call errore ( 'pw2bgw', 'Off-diagonal matrix elements of Vxc ' // &
+    'with real wavefunctions are not implemented, compute them in ' // &
+    'Sigma using VXC.', 7)
+
+  ! this is needed to compute igk and store in iunigk
+  ! cannot use gk_sort because for some k-points
+  ! gk_sort generates different igk on every call
+  CALL openfil ( )
+  CALL hinit0 ( )
 
   CALL openfil_pp ( )
 
@@ -388,8 +399,12 @@ PROGRAM pw2bgw
   ENDIF
 
   CALL environment_end ( codename )
+
   CALL stop_pp ( )
 
+  ! this is needed because openfil is called above
+  CALL close_files ( .false. )
+
   STOP
 
 CONTAINS
@@ -404,7 +419,7 @@ SUBROUTINE write_wfng ( output_file_name, real_or_complex, symm_type, &
   USE constants, ONLY : pi, tpi, eps6
   USE fft_base, ONLY : dfftp
   USE gvect, ONLY : ngm, ngm_g, ig_l2g, g, mill, ecutrho
-  USE io_files, ONLY : iunwfc, nwordwfc
+  USE io_files, ONLY : iunwfc, nwordwfc, iunigk
   USE io_global, ONLY : ionode, ionode_id
   USE ions_base, ONLY : nat, atm, ityp, tau
   USE kinds, ONLY : DP
@@ -418,8 +433,8 @@ SUBROUTINE write_wfng ( output_file_name, real_or_complex, symm_type, &
   USE start_k, ONLY : nk1, nk2, nk3, k1, k2, k3
   USE symm_base, ONLY : s, ftau, nsym
   USE wavefunctions_module, ONLY : evc
-  USE wvfct, ONLY : npwx, nbnd, npw, et, wg, g2kin, ecutwfc
-#ifdef __PARA
+  USE wvfct, ONLY : npwx, nbnd, npw, et, wg, g2kin, ecutwfc, igk
+#ifdef __MPI
   USE parallel_include, ONLY : MPI_DOUBLE_COMPLEX
 #endif
 
@@ -622,7 +637,7 @@ SUBROUTINE write_wfng ( output_file_name, real_or_complex, symm_type, &
       et_g ( ib, ik ) = et ( ib, ik )
     ENDDO
   ENDDO
-#ifdef __PARA
+#ifdef __MPI
   CALL poolrecover ( et_g, nb, nk_g, nk_l )
   CALL mp_bcast ( et_g, ionode_id, world_comm )
 #endif
@@ -653,13 +668,14 @@ SUBROUTINE write_wfng ( output_file_name, real_or_complex, symm_type, &
 
     DO ik = 1, nk_l
       DO ib = 1, nb
-        wg_g ( ib, ik ) = wg ( ib, ik ) 
-        IF ( abs ( wk ( ik ) ) .GT. eps6 ) THEN
-          wg_g ( ib, ik ) = wg_g ( ib, ik ) / wk ( ik ) 
+        IF ( wk(ik) == 0.D0 ) THEN
+          wg_g(ib,ik) = wg(ib,ik)
+        ELSE
+          wg_g(ib,ik) = wg(ib,ik) / wk(ik)
         ENDIF
       ENDDO
     ENDDO
-#ifdef __PARA
+#ifdef __MPI
     CALL poolrecover ( wg_g, nb, nk_g, nk_l )
 #endif
     DO ik = 1, nk_g
@@ -695,23 +711,17 @@ SUBROUTINE write_wfng ( output_file_name, real_or_complex, symm_type, &
 
   ALLOCATE ( igk_l2g ( npwx, nk_l ) )
 
-  ALLOCATE ( itmp ( npwx ) )
+  IF ( nk_l > 1 ) REWIND ( iunigk )
   DO ik = 1, nk_l
-    DO i = 1, npwx
-      itmp ( i ) = 0
-    ENDDO
-    npw = npwx
-    CALL gk_sort ( xk ( 1, ik + iks - 1 ), ng_l, g, ecutwfc / tpiba2, &
-      npw, itmp ( 1 ), g2kin )
+    IF ( nk_l > 1 ) READ ( iunigk ) igk
+    npw = ngk ( ik )
     DO ig = 1, npw
-      igk_l2g ( ig, ik ) = ig_l2g ( itmp ( ig ) )
+      igk_l2g ( ig, ik ) = ig_l2g ( igk ( ig ) )
     ENDDO
     DO ig = npw + 1, npwx
       igk_l2g ( ig, ik ) = 0
     ENDDO
-    ngk ( ik ) = npw
   ENDDO
-  DEALLOCATE ( itmp )
 
   ALLOCATE ( ngk_g ( nk_g ) )
 
@@ -904,7 +914,7 @@ SUBROUTINE write_wfng ( output_file_name, real_or_complex, symm_type, &
         DO ig = igwx + 1, ngkdist_g
           wfng_buf ( ig, is ) = ( 0.0D0, 0.0D0 )
         ENDDO
-#ifdef __PARA
+#ifdef __MPI
         CALL mp_barrier ( world_comm )
         CALL MPI_Scatter ( wfng_buf ( :, is ), ngkdist_l, MPI_DOUBLE_COMPLEX, &
         wfng_dist ( :, ib, is ), ngkdist_l, MPI_DOUBLE_COMPLEX, &
@@ -937,7 +947,7 @@ SUBROUTINE write_wfng ( output_file_name, real_or_complex, symm_type, &
       ENDIF
       DO ib = 1, nb
         DO is = 1, ns
-#ifdef __PARA
+#ifdef __MPI
           CALL mp_barrier ( world_comm )
           CALL MPI_Gather ( wfng_dist ( :, ib, is ), ngkdist_l, &
           MPI_DOUBLE_COMPLEX, wfng_buf ( :, is ), ngkdist_l, &
@@ -1424,8 +1434,8 @@ SUBROUTINE calc_rhog (rhog_nvmin, rhog_nvmax)
   USE fft_base, ONLY : dfftp
   USE fft_interfaces, ONLY : fwfft, invfft
   USE gvect, ONLY : ngm, g, nl
-  USE io_files, ONLY : nwordwfc, iunwfc
-  USE klist, ONLY : xk, nkstot
+  USE io_files, ONLY : nwordwfc, iunwfc, iunigk
+  USE klist, ONLY : xk, nkstot, ngk, nks
   USE lsda_mod, ONLY : nspin, isk
   USE mp, ONLY : mp_sum
   USE mp_world, ONLY : world_comm
@@ -1456,10 +1466,11 @@ SUBROUTINE calc_rhog (rhog_nvmin, rhog_nvmax)
   rho%of_r (:, :) = 0.0D0
 
   ! take psi to R-space, compute rho in R-space
+  IF ( nks > 1 ) REWIND ( iunigk )
   DO ik = iks, ike
     is = isk (ik)
-    CALL gk_sort (xk (1, ik - iks + 1), ngm, g, ecutwfc &
-      / tpiba2, npw, igk, g2kin)
+    IF ( nks > 1 ) READ ( iunigk ) igk
+    npw = ngk ( ik - iks + 1 )
     CALL davcio (evc, 2*nwordwfc, iunwfc, ik - iks + 1, -1)
     DO ib = rhog_nvmin, rhog_nvmax
       psic (:) = (0.0D0, 0.0D0)
@@ -1787,7 +1798,7 @@ SUBROUTINE write_vxc0 ( output_file_name, vxc_zero_rho_core )
   RETURN
 
 101 FORMAT ( /, 5X, "--------------------------------------------", &
-             /, 5X, "spin    Re Vxc(G=0) [eV]    Im Vxc(G=0) [eV]", &
+             /, 5X, "spin    Re Vxc(G=0) (eV)    Im Vxc(G=0) (eV)", &
              /, 5X, "--------------------------------------------" )
 102 FORMAT ( 5X, I1, 3X, 2F20.15 )
 103 FORMAT ( 5X, "--------------------------------------------", / )
@@ -1806,9 +1817,9 @@ SUBROUTINE write_vxc_r (output_file_name, diag_nmin, diag_nmax, &
   USE fft_base, ONLY : dfftp
   USE fft_interfaces, ONLY : invfft
   USE gvect, ONLY : ngm, g, nl
-  USE io_files, ONLY : nwordwfc, iunwfc
+  USE io_files, ONLY : nwordwfc, iunwfc, iunigk
   USE io_global, ONLY : ionode
-  USE klist, ONLY : xk, nkstot
+  USE klist, ONLY : xk, nkstot, nks, ngk
   USE lsda_mod, ONLY : nspin, isk
   USE mp, ONLY : mp_sum
   USE mp_pools, ONLY : kunit, my_pool_id, intra_pool_comm, &
@@ -1886,9 +1897,10 @@ SUBROUTINE write_vxc_r (output_file_name, diag_nmin, diag_nmax, &
   ENDIF
   CALL v_xc (rho, rho_core, rhog_core, etxc, vtxc, vxcr)
 
+  IF ( nks > 1 ) REWIND ( iunigk )
   DO ik = iks, ike
-    CALL gk_sort (xk (1, ik - iks + 1), ngm, g, ecutwfc &
-      / tpiba2, npw, igk, g2kin)
+    IF ( nks > 1 ) READ ( iunigk ) igk
+    npw = ngk ( ik - iks + 1 )
     CALL davcio (evc, 2*nwordwfc, iunwfc, ik - iks + 1, -1)
     IF (ndiag .GT. 0) THEN
       DO ib = diag_nmin, diag_nmax
@@ -1997,10 +2009,10 @@ SUBROUTINE write_vxc_g (output_file_name, diag_nmin, diag_nmax, &
   USE fft_interfaces, ONLY : fwfft, invfft
   USE funct, ONLY : exx_is_active
   USE gvect, ONLY : ngm, g, nl
-  USE io_files, ONLY : nwordwfc, iunwfc
+  USE io_files, ONLY : nwordwfc, iunwfc, iunigk
   USE io_global, ONLY : ionode
   USE kinds, ONLY : DP
-  USE klist, ONLY : xk, nkstot
+  USE klist, ONLY : xk, nkstot, nks, ngk
   USE lsda_mod, ONLY : nspin, isk
   USE mp, ONLY : mp_sum
   USE mp_pools, ONLY : kunit, my_pool_id, intra_pool_comm, &
@@ -2079,9 +2091,10 @@ SUBROUTINE write_vxc_g (output_file_name, diag_nmin, diag_nmax, &
   ENDIF
   CALL v_xc (rho, rho_core, rhog_core, etxc, vtxc, vxcr)
 
+  IF ( nks > 1 ) REWIND ( iunigk )
   DO ik = iks, ike
-    CALL gk_sort (xk (1, ik - iks + 1), ngm, g, ecutwfc &
-      / tpiba2, npw, igk, g2kin)
+    IF ( nks > 1 ) READ ( iunigk ) igk
+    npw = ngk ( ik - iks + 1 )
     CALL davcio (evc, 2*nwordwfc, iunwfc, ik - iks + 1, -1)
     IF (ndiag .GT. 0) THEN
       DO ib = diag_nmin, diag_nmax
@@ -2418,6 +2431,7 @@ SUBROUTINE write_vkbg (output_file_name, symm_type, wfng_kgrid, &
   USE constants, ONLY : pi, tpi, eps6
   USE fft_base, ONLY : dfftp
   USE gvect, ONLY : ngm, ngm_g, ig_l2g, g, mill, ecutrho
+  USE io_files, ONLY : iunigk
   USE io_global, ONLY : ionode, ionode_id
   USE ions_base, ONLY : nat, atm, ityp, tau, nsp
   USE kinds, ONLY : DP
@@ -2432,7 +2446,7 @@ SUBROUTINE write_vkbg (output_file_name, symm_type, wfng_kgrid, &
   USE symm_base, ONLY : s, ftau, nsym
   USE uspp, ONLY : nkb, vkb, deeq
   USE uspp_param, ONLY : nhm, nh
-  USE wvfct, ONLY : npwx, npw, g2kin, ecutwfc
+  USE wvfct, ONLY : npwx, npw, g2kin, ecutwfc, igk
 
   IMPLICIT NONE
 
@@ -2598,18 +2612,14 @@ SUBROUTINE write_vkbg (output_file_name, symm_type, wfng_kgrid, &
   ALLOCATE ( igk_l2g ( npwx, nks ) )
   ngk_g = 0
   igk_l2g = 0
-  ALLOCATE ( itmp ( npwx ) )
+  IF ( nks > 1 ) REWIND ( iunigk )
   DO ik = 1, nks
-    itmp = 0
-    npw = npwx
-    CALL gk_sort ( xk ( 1, ik + iks - 1 ), ngm, g, ecutwfc / tpiba2, &
-      npw, itmp ( 1 ), g2kin )
+    IF ( nks > 1 ) READ ( iunigk ) igk
+    npw = ngk ( ik )
     DO ig = 1, npw
-      igk_l2g ( ig, ik ) = ig_l2g ( itmp ( ig ) )
+      igk_l2g ( ig, ik ) = ig_l2g ( igk ( ig ) )
     ENDDO
-    ngk ( ik ) = npw
   ENDDO
-  DEALLOCATE ( itmp )
   DO ik = 1, nks
     ngk_g ( ik + iks - 1 ) = ngk ( ik )
   ENDDO
@@ -2656,6 +2666,7 @@ SUBROUTINE write_vkbg (output_file_name, symm_type, wfng_kgrid, &
 
   ALLOCATE ( igwk ( npwx_g ) )
 
+  IF ( nks > 1 ) REWIND ( iunigk )
   DO i = 1, nkstot
 
     ik = kmap ( i )
@@ -2691,13 +2702,10 @@ SUBROUTINE write_vkbg (output_file_name, symm_type, wfng_kgrid, &
 
     local_pw = 0
     IF ( ik .GE. iks .AND. ik .LE. ike ) THEN
-      ALLOCATE ( itmp ( npwx ) )
-      npw = npwx
-      CALL gk_sort ( xk ( 1, ik ), ngm, g, ecutwfc / tpiba2, &
-        npw, itmp ( 1 ), g2kin )
-      CALL init_us_2 ( npw, itmp, xk ( 1, ik ), vkb )
-      local_pw = ngk ( ik - iks + 1 )
-      DEALLOCATE ( itmp )
+      IF ( nks > 1 ) READ ( iunigk ) igk
+      npw = ngk ( ik - iks + 1 )
+      CALL init_us_2 ( npw, igk, xk ( 1, ik ), vkb )
+      local_pw = npw
     ENDIF
 
     ALLOCATE ( igwf_l2g ( local_pw ) )
@@ -2761,7 +2769,7 @@ SUBROUTINE write_vkbg (output_file_name, symm_type, wfng_kgrid, &
 
       IF ( ionode ) THEN
         WRITE ( unit ) nrecord
-        WRITE ( unit ) igwx
+        WRITE ( unit ) ngk_g ( ik )
         WRITE ( unit ) ( vkb_g ( ig ), ig = 1, igwx )
       ENDIF
 
