@@ -44,11 +44,12 @@ SUBROUTINE sym_band(filband, spin_component, firstk, lastk)
   INTEGER :: iunout, igroup, irap, dim_rap, ios
   INTEGER :: sk(3,3,48), ftauk(3,48), gk(3,48), sk_is(3,3,48), &
        gk_is(3,48), t_revk(48), nsymk, isym, ipol, jpol
-  LOGICAL :: is_complex, is_complex_so, is_symmorphic, is_high_sym, search_sym
+  LOGICAL :: is_complex, is_complex_so, is_symmorphic, search_sym
+  LOGICAL, ALLOCATABLE :: high_symmetry(:)
   REAL(DP), PARAMETER :: accuracy=1.d-4
   COMPLEX(DP) :: d_spink(2,2,48), d_spin_is(2,2,48), zdotc
   COMPLEX(DP),ALLOCATABLE :: times(:,:,:)
-  REAL(DP) :: dxk(3), dkmod, dkmod_save
+  REAL(DP) :: dxk(3), dkmod, dkmod_save, modk1, modk2, k1(3), k2(3), ps
   INTEGER, ALLOCATABLE :: rap_et(:,:), code_group_k(:)
   INTEGER, ALLOCATABLE :: ngroup(:), istart(:,:)
   CHARACTER(len=11) :: group_name
@@ -65,6 +66,8 @@ SUBROUTINE sym_band(filband, spin_component, firstk, lastk)
   ALLOCATE(times(nbnd,24,nkstot))
   ALLOCATE(ngroup(nkstot))
   ALLOCATE(istart(nbnd+1,nkstot))
+  ALLOCATE(high_symmetry(nkstot))
+
   code_group_k=0
   rap_et=-1
   times=(0.0_DP,0.0_DP)
@@ -116,20 +119,18 @@ SUBROUTINE sym_band(filband, spin_component, firstk, lastk)
      !
      IF (noncolin) THEN
         IF (domag) THEN
-           CALL find_band_sym_so(evc,et(1,ik),at,nbnd,npw,nsym_is, &
-                ngm,sk_is,ftau_is,d_spin_is,gk_is,xk(1,ik),igk,nl,dfftp%nr1,dfftp%nr2,&
-                dfftp%nr3,dfftp%nr1x,dfftp%nr2x,dfftp%nr3x,dfftp%nnr,npwx,rap_et(1,ik),times(1,1,ik), &
+           CALL find_band_sym_so(evc,et(1,ik),nsym_is, &
+                sk_is,ftau_is,d_spin_is,gk_is,&
+                rap_et(1,ik),times(1,1,ik), &
                 ngroup(ik),istart(1,ik),accuracy)
         ELSE
-           CALL find_band_sym_so(evc,et(1,ik),at,nbnd,npw,nsymk,ngm, &
-                sk,ftauk,d_spink,gk,xk(1,ik),igk,nl,dfftp%nr1,dfftp%nr2,dfftp%nr3,dfftp%nr1x, &
-                dfftp%nr2x,dfftp%nr3x,dfftp%nnr,npwx,rap_et(1,ik),times(1,1,ik),ngroup(ik),&
+           CALL find_band_sym_so(evc,et(1,ik),nsymk,sk,ftauk,d_spink,&
+                gk,rap_et(1,ik),times(1,1,ik),ngroup(ik),&
                 istart(1,ik),accuracy)
         ENDIF
      ELSE
-        CALL find_band_sym (evc, et(1,ik), at, nbnd, npw, nsymk, ngm, &
-             sk, ftauk, gk, xk(1,ik), igk, nl, dfftp%nr1, dfftp%nr2, dfftp%nr3, dfftp%nr1x, &
-             dfftp%nr2x, dfftp%nr3x, dfftp%nnr, npwx, rap_et(1,ik), times(1,1,ik), ngroup(ik),&
+        CALL find_band_sym (evc, et(1,ik), nsymk, sk, ftauk, gk, &
+             rap_et(1,ik), times(1,1,ik), ngroup(ik),&
              istart(1,ik),accuracy)
      ENDIF
 
@@ -148,7 +149,27 @@ SUBROUTINE sym_band(filband, spin_component, firstk, lastk)
   CALL ipoolrecover(istart,nbnd+1,nkstot,nks)
 #endif
   IF (ionode) THEN
-     is_high_sym=.false.
+     high_symmetry = .FALSE.
+     DO ik=nks1tot,nks2tot
+        IF ( ik==nks1tot .OR. ik==nks2tot ) THEN
+           high_symmetry(ik) = .TRUE.
+        ELSE
+           k1(:) = xk(:,ik) - xk(:,ik-1)
+           k2(:) = xk(:,ik+1) - xk(:,ik)
+           modk1=sqrt( k1(1)*k1(1) + k1(2)*k1(2) + k1(3)*k1(3) )
+           modk2=sqrt( k2(1)*k2(1) + k2(2)*k2(2) + k2(3)*k2(3) )
+           IF (modk1 <1.d-6 .OR. modk2 < 1.d-6) CYCLE
+           ps = ( k1(1)*k2(1) + k1(2)*k2(2) + k1(3)*k2(3) ) / &
+                modk1 / modk2
+           high_symmetry(ik) = (ABS(ps-1.d0) >1.0d-4)
+!
+!  The gamma point is a high symmetry point
+!
+           IF (xk(1,ik)**2+xk(2,ik)**2+xk(3,ik)**2 < 1.0d-9) &
+                             high_symmetry(ik)=.TRUE.
+        END IF
+     END DO
+!
      DO ik=nks1tot, nks2tot
         CALL smallgk (xk(1,ik), at, bg, s, ftau, t_rev, sname, &
              nsym, sk, ftauk, gk, t_revk, snamek, nsymk)
@@ -169,7 +190,6 @@ SUBROUTINE sym_band(filband, spin_component, firstk, lastk)
            WRITE (iunout, '(" &plot_rap nbnd_rap=",i4,", nks_rap=",i4," /")') &
                 nbnd, nks2tot-nks1tot+1
            IF (search_sym) CALL write_group_info(.true.)
-           is_high_sym=.true.
            dxk(:) = xk(:,nks1tot+1) - xk(:,nks1tot)
            dkmod_save = sqrt( dxk(1)**2 + dxk(2)**2 + dxk(3)**2 )
         ELSE
@@ -184,22 +204,29 @@ SUBROUTINE sym_band(filband, spin_component, firstk, lastk)
 !
            dxk(:) = xk(:,ik) - xk(:,ik-1)
            dkmod= sqrt( dxk(1)**2 + dxk(2)**2 + dxk(3)**2 )
-           IF (dkmod < 5.0_DP * dkmod_save) THEN
+           IF (dkmod < 1.D-6) THEN
+              !
+              !   In this case is_high_sym does not change because the point
+              !   is the same
+              high_symmetry(ik)=high_symmetry(ik-1)
+              !
+           ELSE IF (dkmod < 5.0_DP * dkmod_save) THEN
 !
 !    In this case the two points are considered close
 !
-              is_high_sym= ((code_group_k(ik)/=code_group_k(ik-1)) &
-                 .and..not.is_high_sym) 
+              IF ( .NOT. high_symmetry(ik-1) ) &
+                 high_symmetry(ik)= ((code_group_k(ik)/=code_group_k(ik-1)) &
+                                   .OR. high_symmetry(ik) ) 
               IF (dkmod > 1.d-3) dkmod_save=dkmod
            ELSE
 !
 !    Points are distant. They are all high symmetry
 !
-              is_high_sym= .TRUE.
+              high_symmetry(ik) = .TRUE.
            ENDIF
         ENDIF
         WRITE (iunout, '(10x,3f10.6,l5)') xk(1,ik),xk(2,ik),xk(3,ik), &
-             is_high_sym
+             high_symmetry(ik)
         WRITE (iunout, '(10i8)') (rap_et(ibnd,ik), ibnd=1,nbnd)
         IF (.not.search_sym) CYCLE
         IF (noncolin) THEN
@@ -283,13 +310,13 @@ SUBROUTINE sym_band(filband, spin_component, firstk, lastk)
   DEALLOCATE(rap_et)
   DEALLOCATE(ngroup)
   DEALLOCATE(istart)
+  DEALLOCATE(high_symmetry)
   !
   RETURN
 END SUBROUTINE sym_band
 !
-SUBROUTINE find_band_sym (evc,et,at,nbnd,npw,nsym,ngm,s,ftau,gk, &
-     xk,igk,nl,nr1,nr2,nr3,nr1x,nr2x,nr3x,nrxx,npwx, &
-     rap_et,times,ngroup,istart,accuracy)
+SUBROUTINE find_band_sym (evc,et,nsym,s,ftau,gk,rap_et,times,ngroup,&
+                          istart,accuracy)
   !
   !   This subroutine finds the irreducible representations which give
   !   the transformation properties of the wavefunctions.
@@ -302,24 +329,23 @@ SUBROUTINE find_band_sym (evc,et,at,nbnd,npw,nsym,ngm,s,ftau,gk, &
   USE constants,       ONLY : rytoev
   USE rap_point_group, ONLY : code_group, nclass, nelem, elem, which_irr, &
        char_mat, name_rap, name_class, gname
+  USE gvect,           ONLY : ngm, nl
+  USE wvfct,           ONLY : nbnd, npwx, npw, igk
   USE uspp,            ONLY : vkb, nkb, okvan
   USE becmod,          ONLY : bec_type, becp, calbec, &
        allocate_bec_type, deallocate_bec_type
-  USE mp_global,       ONLY : intra_pool_comm
+  USE fft_base,        ONLY : dfftp
+  USE fft_interfaces,  ONLY : invfft
+  USE mp_bands,        ONLY : intra_bgrp_comm
   USE mp,              ONLY : mp_sum
 
   IMPLICIT NONE
 
   REAL(DP), INTENT(in) :: accuracy
 
-  INTEGER :: nr1, nr2, nr3, nr1x, nr2x, nr3x, nrxx, ngm, npw, npwx
-
   INTEGER ::                  &
        nsym,             &
-       nbnd,             &
        rap_et(nbnd),     &
-       igk(npwx),        &
-       nl(ngm),          &
        ftau(3,48),       &
        gk(3,48),         &
        s(3,3,48),        &
@@ -327,8 +353,6 @@ SUBROUTINE find_band_sym (evc,et,at,nbnd,npw,nsym,ngm,s,ftau,gk, &
        istart(nbnd+1)
 
   REAL(DP) ::                 &
-       at(3,3),        &
-       xk(3),          &
        et(nbnd)
 
   COMPLEX(DP) ::  &
@@ -345,17 +369,19 @@ SUBROUTINE find_band_sym (evc,et,at,nbnd,npw,nsym,ngm,s,ftau,gk, &
        irap,      &
        iclass,    &
        shift,     &
-       na, i, j, ig, dimen
+       na, i, j, ig, dimen, nrxx
 
   COMPLEX(DP) :: zdotc
 
   REAL(DP), ALLOCATABLE ::  w1(:)
-  COMPLEX(DP), ALLOCATABLE ::  evcr(:,:), trace(:,:)
+  COMPLEX(DP), ALLOCATABLE ::  evcr(:,:), trace(:,:), psic(:,:)
   !
   !    Divide the bands on the basis of the band degeneracy.
   !
+  nrxx=dfftp%nnr
   ALLOCATE(w1(nbnd))
   ALLOCATE(evcr(npwx,nbnd))
+  ALLOCATE(psic(nrxx,nbnd))
   ALLOCATE(trace(48,nbnd))
   IF (okvan) CALL allocate_bec_type ( nkb, nbnd, becp )
 
@@ -371,6 +397,14 @@ SUBROUTINE find_band_sym (evc,et,at,nbnd,npw,nsym,ngm,s,ftau,gk, &
      ENDIF
   ENDDO
   istart(ngroup+1)=nbnd+1
+!
+!   bring all the bands in real space
+!
+  psic=(0.0_DP,0.0_DP)
+  DO ibnd=1,nbnd
+     psic(nl(igk(1:npw)),ibnd) = evc(1:npw,ibnd)
+     CALL invfft ('Dense', psic(:,ibnd), dfftp)
+  ENDDO
   !
   !  Find the character of one symmetry operation per class
   !
@@ -382,12 +416,11 @@ SUBROUTINE find_band_sym (evc,et,at,nbnd,npw,nsym,ngm,s,ftau,gk, &
      !       rotate the k point.
      !
      !
-     DO ibnd=1,nbnd
-        CALL rotate_psi(evc(1,ibnd),evcr(1,ibnd),s(1,1,irot), &
-             ftau(1,irot),gk(1,irot),nl,igk,nr1,nr2,nr3,nr1x, &
-             nr2x,nr3x,nrxx,ngm,npw)
-
-     ENDDO
+     IF (irot==1) THEN
+        evcr=evc
+     ELSE
+        CALL rotate_all_psi(psic,evcr,s(1,1,irot),ftau(1,irot),gk(1,irot))
+     ENDIF
      !
      !   and apply S if necessary
      !
@@ -410,7 +443,7 @@ SUBROUTINE find_band_sym (evc,et,at,nbnd,npw,nsym,ngm,s,ftau,gk, &
      ENDDO
   ENDDO
   !
-  CALL mp_sum( trace, intra_pool_comm )
+  CALL mp_sum( trace, intra_bgrp_comm )
 
   !DO iclass=1,nclass
   !   write(6,'(i5,3(2f11.8,1x))') iclass,trace(iclass,4),trace(iclass,5), &
@@ -477,132 +510,160 @@ SUBROUTINE find_band_sym (evc,et,at,nbnd,npw,nsym,ngm,s,ftau,gk, &
   DEALLOCATE(trace)
   DEALLOCATE(w1)
   DEALLOCATE(evcr)
+  DEALLOCATE(psic)
   IF (okvan) CALL deallocate_bec_type (becp)
 
   RETURN
 END SUBROUTINE find_band_sym
 
-SUBROUTINE rotate_psi(evc,evcr,s,ftau,gk,nl,igk,nr1,nr2,nr3, &
-     nr1x,nr2x,nr3x,nrxx,ngm,npw)
+
+SUBROUTINE rotate_all_psi(psic,evcr,s,ftau,gk)
 
   USE kinds,     ONLY : DP
   USE constants, ONLY : tpi
-  USE fft_base,  ONLY : cgather_sym, cscatter_sym, dfftp
+  USE gvect,     ONLY : ngm, nl
+  USE wvfct,     ONLY : nbnd, npwx, npw, igk
+  USE fft_base,  ONLY : cgather_sym_many, cscatter_sym_many, dfftp
   USE fft_interfaces, ONLY : fwfft, invfft
+  USE mp_bands,  ONLY : intra_bgrp_comm
+  USE mp,        ONLY : mp_sum
 
   IMPLICIT NONE
 
-  INTEGER :: nr1, nr2, nr3, nr1x, nr2x, nr3x, nrxx, ngm, npw, nbnd
-  INTEGER :: s(3,3), ftau(3), gk(3), nl(ngm), igk(npw)
+  INTEGER :: s(3,3), ftau(3), gk(3)
 
-  COMPLEX(DP), ALLOCATABLE :: psic(:), psir(:)
-  COMPLEX(DP) :: evc(npw), evcr(npw)
+  COMPLEX(DP), ALLOCATABLE :: psir(:)
+  COMPLEX(DP) :: psic(dfftp%nnr,nbnd), evcr(npwx,nbnd)
   COMPLEX(DP) :: phase
   REAL(DP) :: arg
-  INTEGER :: i, j, k, ri, rj, rk, ir, rir, ipol
+  INTEGER :: i, j, k, ri, rj, rk, ir, rir, ipol, ibnd
+  INTEGER :: nr1, nr2, nr3, nr1x, nr2x, nr3x, nrxx
   LOGICAL :: zone_border
+  INTEGER :: start_band, last_band, my_nbnd_proc
+  INTEGER :: start_band_proc(dfftp%nproc), nbnd_proc(dfftp%nproc)
+
 #if defined  (__MPI)
   !
   COMPLEX (DP), ALLOCATABLE :: psir_collect(:)
-  COMPLEX (DP), ALLOCATABLE :: psic_collect(:)
+  COMPLEX (DP), ALLOCATABLE :: psic_collect(:,:)
   !
 #endif
   !
-  ALLOCATE(psic(nrxx))
+  nr1=dfftp%nr1
+  nr2=dfftp%nr2
+  nr3=dfftp%nr3
+  nr1x=dfftp%nr1x
+  nr2x=dfftp%nr2x
+  nr3x=dfftp%nr3x
+  nrxx=dfftp%nnr
+  !
   ALLOCATE(psir(nrxx))
   !
-  zone_border=(gk(1)/=0.or.gk(2)/=0.or.gk(3)/=0)
+  zone_border=(gk(1)/=0.OR.gk(2)/=0.OR.gk(3)/=0)
   !
-  psic = ( 0.D0, 0.D0 )
-  !
-  psic(nl(igk(1:npw))) = evc(1:npw)
-  !
-  CALL invfft ('Dense', psic, dfftp)
+  evcr=  (0.0_DP, 0.0_DP)
   !
 #if defined  (__MPI)
+
+  call divide (intra_bgrp_comm, nbnd, start_band, last_band)
+  start_band_proc=0
+  start_band_proc(dfftp%mype+1)=start_band
+  nbnd_proc=0
+  my_nbnd_proc=last_band-start_band+1
+  nbnd_proc(dfftp%mype+1)=my_nbnd_proc
+  CALL mp_sum(start_band_proc, intra_bgrp_comm)
+  CALL mp_sum(nbnd_proc, intra_bgrp_comm)
   !
-  ALLOCATE (psic_collect(nr1x*nr2x*nr3x))
+  ALLOCATE (psic_collect(nr1x*nr2x*nr3x, my_nbnd_proc))
   ALLOCATE (psir_collect(nr1x*nr2x*nr3x))
   !
-  CALL cgather_sym( psic, psic_collect )
+  CALL cgather_sym_many( psic, psic_collect, nbnd, nbnd_proc, start_band_proc)
   !
-  psir_collect=(0.d0,0.d0)
-  !
-  IF (zone_border) THEN
-     DO k = 1, nr3
-        DO j = 1, nr2
-           DO i = 1, nr1
+  DO ibnd = 1, my_nbnd_proc
+     psir_collect=(0.d0,0.d0)
+     !
+     IF (zone_border) THEN
+        DO k = 1, nr3
+          DO j = 1, nr2
+             DO i = 1, nr1
               CALL ruotaijk (s, ftau, i, j, k, nr1, nr2, nr3, ri, rj, rk )
               ir=i+(j-1)*nr1x+(k-1)*nr1x*nr2x
               rir=ri+(rj-1)*nr1x+(rk-1)*nr1x*nr2x
               arg=tpi*( (gk(1)*(i-1))/dble(nr1)+(gk(2)*(j-1))/dble(nr2)+ &
                    (gk(3)*(k-1))/dble(nr3) )
               phase=cmplx(cos(arg),sin(arg),kind=DP)
-              psir_collect(ir)=psic_collect(rir)*phase
+              psir_collect(ir)=psic_collect(rir,ibnd)*phase
+             ENDDO
+          ENDDO
+       ENDDO
+     ELSE
+        DO k = 1, nr3
+           DO j = 1, nr2
+              DO i = 1, nr1
+                 CALL ruotaijk (s, ftau, i, j, k, nr1, nr2, nr3, ri, rj, rk )
+                 ir=i+(j-1)*nr1x+(k-1)*nr1x*nr2x
+                 rir=ri+(rj-1)*nr1x+(rk-1)*nr1x*nr2x
+                 psir_collect(ir)=psic_collect(rir, ibnd)
+              ENDDO
            ENDDO
         ENDDO
-     ENDDO
-  ELSE
-     DO k = 1, nr3
-        DO j = 1, nr2
-           DO i = 1, nr1
-              CALL ruotaijk (s, ftau, i, j, k, nr1, nr2, nr3, ri, rj, rk )
-              ir=i+(j-1)*nr1x+(k-1)*nr1x*nr2x
-              rir=ri+(rj-1)*nr1x+(rk-1)*nr1x*nr2x
-              psir_collect(ir)=psic_collect(rir)
-           ENDDO
-        ENDDO
-     ENDDO
-  ENDIF
-  !
-  CALL cscatter_sym( psir_collect, psir )
+     ENDIF
+     psic_collect(:,ibnd)=psir_collect(:)
+  ENDDO
   !
+  DO ibnd=1, nbnd
+     CALL cscatter_sym_many( psic_collect, psir, ibnd, nbnd, &
+                                                 nbnd_proc, start_band_proc )
+     !
+     CALL fwfft ('Dense', psir, dfftp)
+     !
+     evcr(1:npw,ibnd) = psir(nl(igk(1:npw)))
+  END DO
   DEALLOCATE (psic_collect)
   DEALLOCATE (psir_collect)
   !
 #else
   psir=(0.d0,0.d0)
-  IF (zone_border) THEN
-     DO k = 1, nr3
-        DO j = 1, nr2
-           DO i = 1, nr1
-              CALL ruotaijk (s, ftau, i, j, k, nr1, nr2, nr3, ri, rj, rk )
-              ir=i+(j-1)*nr1x+(k-1)*nr1x*nr2x
-              rir=ri+(rj-1)*nr1x+(rk-1)*nr1x*nr2x
-              arg=tpi*( (gk(1)*(i-1))/dble(nr1)+(gk(2)*(j-1))/dble(nr2)+ &
-                   (gk(3)*(k-1))/dble(nr3) )
-              phase=cmplx(cos(arg),sin(arg),kind=DP)
-              psir(ir)=psic(rir)*phase
+  DO ibnd=1,nbnd
+     IF (zone_border) THEN
+        DO k = 1, nr3
+           DO j = 1, nr2
+              DO i = 1, nr1
+                 CALL ruotaijk (s, ftau, i, j, k, nr1, nr2, nr3, ri, rj, rk )
+                 ir=i+(j-1)*nr1x+(k-1)*nr1x*nr2x
+                 rir=ri+(rj-1)*nr1x+(rk-1)*nr1x*nr2x
+                 arg=tpi*( (gk(1)*(i-1))/dble(nr1)+(gk(2)*(j-1))/dble(nr2)+ &
+                    (gk(3)*(k-1))/dble(nr3) )
+                 phase=cmplx(cos(arg),sin(arg),kind=DP)
+                 psir(ir)=psic(rir,ibnd)*phase
+              ENDDO
            ENDDO
         ENDDO
-     ENDDO
-  ELSE
-     DO k = 1, nr3
-        DO j = 1, nr2
-           DO i = 1, nr1
-              CALL ruotaijk (s, ftau, i, j, k, nr1, nr2, nr3, ri, rj, rk )
-              ir=i+(j-1)*nr1x+(k-1)*nr1x*nr2x
-              rir=ri+(rj-1)*nr1x+(rk-1)*nr1x*nr2x
-              psir(ir)=psic(rir)
+     ELSE
+        DO k = 1, nr3
+           DO j = 1, nr2
+              DO i = 1, nr1
+                 CALL ruotaijk (s, ftau, i, j, k, nr1, nr2, nr3, ri, rj, rk )
+                 ir=i+(j-1)*nr1x+(k-1)*nr1x*nr2x
+                 rir=ri+(rj-1)*nr1x+(rk-1)*nr1x*nr2x
+                 psir(ir)=psic(rir,ibnd)
+              ENDDO
            ENDDO
         ENDDO
-     ENDDO
-  ENDIF
+     ENDIF
+     CALL fwfft ('Dense', psir, dfftp)
+     !
+     evcr(1:npw,ibnd) = psir(nl(igk(1:npw)))
+  ENDDO
   !
 #endif
   !
-  CALL fwfft ('Dense', psir, dfftp)
-  !
-  evcr(1:npw) = psir(nl(igk(1:npw)))
-  !
-  DEALLOCATE(psic)
   DEALLOCATE(psir)
   !
   RETURN
-END SUBROUTINE rotate_psi
+END SUBROUTINE rotate_all_psi
 
-SUBROUTINE find_band_sym_so (evc,et,at,nbnd,npw,nsym,ngm,s,ftau,d_spin,gk, &
-     xk,igk,nl,nr1,nr2,nr3,nr1x,nr2x,nr3x,nrxx,npwx, &
+SUBROUTINE find_band_sym_so (evc,et,nsym,s,ftau,d_spin,gk, &
      rap_et,times,ngroup,istart,accuracy)
 
   !
@@ -621,34 +682,29 @@ SUBROUTINE find_band_sym_so (evc,et,at,nbnd,npw,nsym,ngm,s,ftau,d_spin,gk, &
        char_mat_so, name_rap_so, name_class_so,      &
        name_class_so1
   USE rap_point_group_is, ONLY : gname_is
+  USE gvect,              ONLY : ngm, nl
+  USE wvfct,              ONLY : nbnd, npwx, npw, igk
   USE spin_orb,           ONLY : domag
   USE uspp,               ONLY : vkb, nkb, okvan
   USE noncollin_module,   ONLY : npol
   USE becmod,             ONLY : bec_type, becp, calbec, allocate_bec_type, deallocate_bec_type
-  USE mp_global,          ONLY : intra_pool_comm
+  USE mp_bands,           ONLY : intra_bgrp_comm
   USE mp,                 ONLY : mp_sum
 
   IMPLICIT NONE
 
   REAL(DP), INTENT(in) :: accuracy
 
-  INTEGER :: nr1, nr2, nr3, nr1x, nr2x, nr3x, nrxx, ngm, npw, npwx
-
   INTEGER ::                  &
        nsym,             &
-       nbnd,             &
        ngroup,           &
        istart(nbnd+1),   &
        rap_et(nbnd),     &
-       igk(npwx),        &
-       nl(ngm),          &
        ftau(3,48),       &
        gk(3,48),         &
        s(3,3,48)
 
   REAL(DP) ::                 &
-       at(3,3),        &
-       xk(3),          &
        et(nbnd)
 
   COMPLEX(DP) ::  &
@@ -708,11 +764,8 @@ SUBROUTINE find_band_sym_so (evc,et,at,nbnd,npw,nsym,ngm,s,ftau,d_spin,gk, &
      !   NB: rotate_psi assumes that s is in the small group of k. It does not
      !       rotate the k point.
      !
-     DO ibnd=1,nbnd
-        CALL rotate_psi_so(evc(1,ibnd),evcr(1,ibnd),s(1,1,irot),        &
-             ftau(1,irot),d_spin(1,1,irot),has_e(1,iclass),gk(1,irot), &
-             nl,igk,npol,nr1,nr2,nr3,nr1x,nr2x,nr3x,nrxx,ngm,npw,npwx)
-     ENDDO
+      CALL rotate_all_psi_so(evc,evcr,s(1,1,irot),        &
+           ftau(1,irot),d_spin(1,1,irot),has_e(1,iclass),gk(1,irot))
      !
      !   and apply S in the US case.
      !
@@ -734,7 +787,7 @@ SUBROUTINE find_band_sym_so (evc,et,at,nbnd,npw,nsym,ngm,s,ftau,d_spin,gk, &
      ENDDO
   ENDDO
   !
-  CALL mp_sum(trace,intra_pool_comm)
+  CALL mp_sum(trace,intra_bgrp_comm)
   !
   !DO iclass=1,nclass
   !   write(6,'(i5,3(2f11.8,1x))') iclass,trace(iclass,1),trace(iclass,2), &
@@ -811,8 +864,7 @@ SUBROUTINE find_band_sym_so (evc,et,at,nbnd,npw,nsym,ngm,s,ftau,d_spin,gk, &
   RETURN
 END SUBROUTINE find_band_sym_so
 
-SUBROUTINE rotate_psi_so(evc_nc,evcr,s,ftau,d_spin,has_e,gk,nl,igk,npol, &
-     nr1,nr2,nr3,nr1x,nr2x,nr3x,nrxx,ngm,npw,npwx)
+SUBROUTINE rotate_all_psi_so(evc_nc,evcr,s,ftau,d_spin,has_e,gk)
   !
   !  This subroutine rotates a spinor wavefunction according to the symmetry
   !  s. d_spin contains the 2x2 rotation matrix in the spin space.
@@ -820,119 +872,164 @@ SUBROUTINE rotate_psi_so(evc_nc,evcr,s,ftau,d_spin,has_e,gk,nl,igk,npol, &
   !
   USE kinds,     ONLY : DP
   USE constants, ONLY : tpi
-  USE fft_base,  ONLY : cgather_sym, cscatter_sym, dfftp
+  USE fft_base,  ONLY : cgather_sym_many, cscatter_sym_many, dfftp
   USE fft_interfaces, ONLY : fwfft, invfft
+  USE gvect,     ONLY : ngm, nl
+  USE wvfct,     ONLY : nbnd, npwx, npw, igk
+  USE noncollin_module, ONLY : npol
+  USE mp_bands,  ONLY : intra_bgrp_comm
+  USE mp,        ONLY : mp_sum
 
   IMPLICIT NONE
 
-  INTEGER :: npol, nr1, nr2, nr3, nr1x, nr2x, nr3x, nrxx, ngm, npw, nbnd, npwx
-  INTEGER :: s(3,3), ftau(3), gk(3), nl(ngm), igk(npw), has_e
+  INTEGER :: s(3,3), ftau(3), gk(3), has_e
+  COMPLEX(DP) :: evc_nc(npwx,2,nbnd), evcr(npwx,2,nbnd), d_spin(2,2)
 
-  COMPLEX(DP), ALLOCATABLE :: psic(:,:), psir(:,:), evcr_save(:,:)
-  COMPLEX(DP) :: evc_nc(npwx,2), evcr(npwx,2), d_spin(2,2)
+  COMPLEX(DP), ALLOCATABLE :: psic(:,:), psir(:), evcr_save(:,:,:)
   COMPLEX(DP) :: phase
-  REAL(DP) :: arg, sum
-  INTEGER :: i, j, k, ri, rj, rk, ir, rir, ipol, jpol
+  REAL(DP) :: arg
+  INTEGER :: i, j, k, ri, rj, rk, ir, rir, ipol, jpol, ibnd
+  INTEGER :: nr1, nr2, nr3, nr1x, nr2x, nr3x, nrxx
   LOGICAL :: zone_border
+  INTEGER :: start_band, last_band, my_nbnd_proc
+  INTEGER :: start_band_proc(dfftp%nproc), nbnd_proc(dfftp%nproc)
   !
 #if defined  (__MPI)
   !
   COMPLEX (DP), ALLOCATABLE :: psir_collect(:)
-  COMPLEX (DP), ALLOCATABLE :: psic_collect(:)
+  COMPLEX (DP), ALLOCATABLE :: psic_collect(:,:)
   !
-  ALLOCATE (psic_collect(nr1x*nr2x*nr3x))
+#endif
+
+  nr1=dfftp%nr1
+  nr2=dfftp%nr2
+  nr3=dfftp%nr3
+  nr1x=dfftp%nr1x
+  nr2x=dfftp%nr2x
+  nr3x=dfftp%nr3x
+  nrxx=dfftp%nnr
+
+#if defined  (__MPI)
+  call divide (intra_bgrp_comm, nbnd, start_band, last_band)
+  start_band_proc=0
+  start_band_proc(dfftp%mype+1)=start_band
+  nbnd_proc=0
+  my_nbnd_proc=last_band-start_band+1
+  nbnd_proc(dfftp%mype+1)=my_nbnd_proc
+  CALL mp_sum(start_band_proc, intra_bgrp_comm)
+  CALL mp_sum(nbnd_proc, intra_bgrp_comm)
+  ALLOCATE (psic_collect(nr1x*nr2x*nr3x,my_nbnd_proc))
   ALLOCATE (psir_collect(nr1x*nr2x*nr3x))
 #endif
   !
-  ALLOCATE(psic(nrxx,npol))
-  ALLOCATE(psir(nrxx,npol))
-  ALLOCATE(evcr_save(npwx,npol))
+  ALLOCATE(psic(nrxx,nbnd))
+  ALLOCATE(psir(nrxx))
+  ALLOCATE(evcr_save(npwx,npol,nbnd))
   !
   zone_border=(gk(1)/=0.or.gk(2)/=0.or.gk(3)/=0)
   !
-  psic = ( 0.D0, 0.D0 )
-  psir = ( 0.D0, 0.D0 )
   !
   DO ipol=1,npol
      !
-     psic(nl(igk(1:npw)),ipol) = evc_nc(1:npw,ipol)
-     CALL invfft ('Dense', psic(:,ipol), dfftp)
+     psic = ( 0.D0, 0.D0 )
+     psir = ( 0.D0, 0.D0 )
+     !
+     DO ibnd=1,nbnd
+        psic(nl(igk(1:npw)),ibnd) = evc_nc(1:npw,ipol,ibnd)
+        CALL invfft ('Dense', psic(:,ibnd), dfftp)
+     ENDDO
      !
 #if defined  (__MPI)
      !
      !
-     CALL cgather_sym( psic(:,ipol), psic_collect )
+     CALL cgather_sym_many( psic, psic_collect, nbnd, nbnd_proc, &
+                                                       start_band_proc  )
      !
      psir_collect=(0.d0,0.d0)
-     !
-     IF (zone_border) THEN
-        DO k = 1, nr3
-           DO j = 1, nr2
-              DO i = 1, nr1
-                 CALL ruotaijk (s, ftau, i, j, k, nr1, nr2, nr3, ri, rj, rk )
-                 ir=i+(j-1)*nr1x+(k-1)*nr1x*nr2x
-                 rir=ri+(rj-1)*nr1x+(rk-1)*nr1x*nr2x
-                 arg=tpi*( (gk(1)*(i-1))/dble(nr1)+(gk(2)*(j-1))/dble(nr2)+ &
-                      (gk(3)*(k-1))/dble(nr3) )
-                 phase=cmplx(cos(arg),sin(arg),kind=DP)
-                 psir_collect(ir)=psic_collect(rir)*phase
+     DO ibnd=1,my_nbnd_proc
+        !
+        IF (zone_border) THEN
+           DO k = 1, nr3
+              DO j = 1, nr2
+                 DO i = 1, nr1
+                    CALL ruotaijk (s, ftau, i, j, k, nr1, nr2, nr3, ri, rj, rk )
+                    ir=i+(j-1)*nr1x+(k-1)*nr1x*nr2x
+                    rir=ri+(rj-1)*nr1x+(rk-1)*nr1x*nr2x
+                    arg=tpi*( (gk(1)*(i-1))/dble(nr1)+(gk(2)*(j-1))/dble(nr2)+ &
+                         (gk(3)*(k-1))/dble(nr3) )
+                    phase=cmplx(cos(arg),sin(arg),kind=DP)
+                    psir_collect(ir)=psic_collect(rir,ibnd)*phase
+                 ENDDO
               ENDDO
            ENDDO
-        ENDDO
-     ELSE
-        DO k = 1, nr3
-           DO j = 1, nr2
-              DO i = 1, nr1
-                 CALL ruotaijk (s, ftau, i, j, k, nr1, nr2, nr3, ri, rj, rk )
-                 ir=i+(j-1)*nr1x+(k-1)*nr1x*nr2x
-                 rir=ri+(rj-1)*nr1x+(rk-1)*nr1x*nr2x
-                 psir_collect(ir)=psic_collect(rir)
+        ELSE
+           DO k = 1, nr3
+              DO j = 1, nr2
+                 DO i = 1, nr1
+                    CALL ruotaijk (s, ftau, i, j, k, nr1, nr2, nr3, ri, rj, rk )
+                    ir=i+(j-1)*nr1x+(k-1)*nr1x*nr2x
+                    rir=ri+(rj-1)*nr1x+(rk-1)*nr1x*nr2x
+                    psir_collect(ir)=psic_collect(rir,ibnd)
+                 ENDDO
               ENDDO
            ENDDO
-        ENDDO
-     ENDIF
-     !
-     CALL cscatter_sym( psir_collect, psir(:,ipol) )
+        ENDIF
+        psic_collect(:,ibnd) = psir_collect(:)
+     ENDDO
+     DO ibnd=1,nbnd
+        !
+        CALL cscatter_sym_many(psic_collect, psir, ibnd, nbnd, nbnd_proc, &
+                               start_band_proc)
+        CALL fwfft ('Dense', psir, dfftp)
+        !
+        evcr_save(1:npw,ipol,ibnd) = psir(nl(igk(1:npw)))
+     ENDDO
      !
 #else
-     IF (zone_border) THEN
-        DO k = 1, nr3
-           DO j = 1, nr2
-              DO i = 1, nr1
-                 CALL ruotaijk (s, ftau, i, j, k, nr1, nr2, nr3, ri, rj, rk )
-                 ir=i+(j-1)*nr1x+(k-1)*nr1x*nr2x
-                 rir=ri+(rj-1)*nr1x+(rk-1)*nr1x*nr2x
-                 arg=tpi*( (gk(1)*(i-1))/dble(nr1)+(gk(2)*(j-1))/dble(nr2)+ &
-                      (gk(3)*(k-1))/dble(nr3) )
-                 phase=cmplx(cos(arg),sin(arg),kind=DP)
-                 psir(ir,ipol)=psic(rir,ipol)*phase
+     DO ibnd=1,nbnd
+        IF (zone_border) THEN
+           DO k = 1, nr3
+              DO j = 1, nr2
+                 DO i = 1, nr1
+                    CALL ruotaijk (s, ftau, i, j, k, nr1, nr2, nr3, ri, rj, rk )
+                    ir=i+(j-1)*nr1x+(k-1)*nr1x*nr2x
+                    rir=ri+(rj-1)*nr1x+(rk-1)*nr1x*nr2x
+                    arg=tpi*( (gk(1)*(i-1))/dble(nr1)+(gk(2)*(j-1))/dble(nr2)+ &
+                         (gk(3)*(k-1))/dble(nr3) )
+                    phase=cmplx(cos(arg),sin(arg),kind=DP)
+                    psir(ir)=psic(rir,ibnd)*phase
+                 ENDDO
               ENDDO
            ENDDO
-        ENDDO
-     ELSE
-        DO k = 1, nr3
-           DO j = 1, nr2
-              DO i = 1, nr1
-                 CALL ruotaijk (s, ftau, i, j, k, nr1, nr2, nr3, ri, rj, rk )
-                 ir=i+(j-1)*nr1x+(k-1)*nr1x*nr2x
-                 rir=ri+(rj-1)*nr1x+(rk-1)*nr1x*nr2x
-                 psir(ir,ipol)=psic(rir,ipol)
+        ELSE
+           DO k = 1, nr3
+              DO j = 1, nr2
+                 DO i = 1, nr1
+                    CALL ruotaijk (s, ftau, i, j, k, nr1, nr2, nr3, ri, rj, rk )
+                    ir=i+(j-1)*nr1x+(k-1)*nr1x*nr2x
+                    rir=ri+(rj-1)*nr1x+(rk-1)*nr1x*nr2x
+                    psir(ir)=psic(rir,ibnd)
+                 ENDDO
               ENDDO
            ENDDO
-        ENDDO
-     ENDIF
+        ENDIF
+        CALL fwfft ('Dense', psir(:), dfftp)
+        !
+        evcr_save(1:npw,ipol,ibnd) = psir(nl(igk(1:npw)))
+     ENDDO
      !
 #endif
      !
-     CALL fwfft ('Dense', psir(:,ipol), dfftp)
-     !
-     evcr_save(1:npw,ipol) = psir(nl(igk(1:npw)),ipol)
      !
   ENDDO
+
   evcr=(0.d0,0.d0)
-  DO ipol=1,npol
-     DO jpol=1,npol
-        evcr(:,ipol)=evcr(:,ipol)+conjg(d_spin(jpol,ipol))*evcr_save(:,jpol)
+  DO ibnd=1,nbnd 
+     DO ipol=1,npol
+        DO jpol=1,npol
+           evcr(:,ipol,ibnd)=evcr(:,ipol,ibnd)+ &
+                conjg(d_spin(jpol,ipol))*evcr_save(:,jpol,ibnd)
+        ENDDO
      ENDDO
   ENDDO
   IF (has_e==-1) evcr=-evcr
@@ -945,7 +1042,7 @@ SUBROUTINE rotate_psi_so(evc_nc,evcr,s,ftau,d_spin,has_e,gk,nl,igk,npol, &
   DEALLOCATE (psir_collect)
 #endif
   RETURN
-END SUBROUTINE rotate_psi_so
+END SUBROUTINE rotate_all_psi_so
 
 SUBROUTINE find_nks1nks2(firstk,lastk,nks1tot,nks1,nks2tot,nks2,spin_component)
   !
@@ -1154,3 +1251,4 @@ SUBROUTINE s_axis_to_cart (s, sr, at, bg)
   ENDDO
   RETURN
 END SUBROUTINE s_axis_to_cart
+
