@@ -21,7 +21,7 @@ PROGRAM wannier_ham
   USE mp_world,         ONLY : world_comm
   USE read_cards_module, ONLY : read_cards
   USE mp_global,     ONLY : mp_startup
-  USE environment,   ONLY : environment_start
+  USE environment,   ONLY : environment_start, environment_end
 
   IMPLICIT NONE
   !
@@ -79,6 +79,8 @@ PROGRAM wannier_ham
 
   IF(u_matrix) CALL wannier_u_matrix(U,J)
 
+  CALL environment_end ( 'WANNIER_HAM')
+
   CALL stop_pp
 
   CALL wannier_clean()
@@ -166,7 +168,7 @@ SUBROUTINE new_hamiltonian(plot_bands)
               ! Hoping integrals
               hamh(i,j,current_spin) = hamh(i,j,current_spin) + &
                  pp(i,n)*cmplx(et(n,ik),0.d0,kind=DP)*conjg(pp(j,n))*wk(ik)*&
-                 cdexp( (0.d0,1.d0)*tpi* (xk(1,ik)*hoping(1) + &
+                 exp( (0.d0,1.d0)*tpi* (xk(1,ik)*hoping(1) + &
                      xk(2,ik)*hoping(2) + xk(3,ik)*hoping(3)) )
               ! Current k-point hamiltonian
               hamk(i,j,ik) = hamk(i,j,ik) + pp(i,n)*conjg(pp(j,n))* &
