@@ -18,13 +18,14 @@ SUBROUTINE electrons()
   USE io_global,            ONLY : stdout, ionode
   USE fft_base,             ONLY : dfftp
   USE gvecs,                ONLY : doublegrid
+  USE gvect,                ONLY : ecutrho
   USE lsda_mod,             ONLY : lsda, nspin, magtot, absmag, isk
   USE ener,                 ONLY : etot, hwf_energy, eband, deband, ehart, &
                                    vtxc, etxc, etxcc, ewld, demet, epaw, &
                                    elondon, ef_up, ef_dw
   USE scf,                  ONLY : rho, rho_core, rhog_core, v, vltot, vrs, &
                                    kedtau, vnew
-  USE control_flags,        ONLY : tr2, niter, conv_elec, restart
+  USE control_flags,        ONLY : tr2, niter, conv_elec, restart, lmd
   USE io_files,             ONLY : iunwfc, iunmix, nwordwfc, output_drho, &
                                    iunres, iunefield, seqopn
   USE buffers,              ONLY : save_buffer, close_buffer
@@ -56,32 +57,30 @@ SUBROUTINE electrons()
   INTEGER :: &
       idum,         &! dummy counter on iterations
       iter,         &! counter on iterations
+      printout,     &
       ik, ios
   REAL(DP) :: &
       tr2_min,     &! estimated error on energy coming from diagonalization
       tr2_final     ! final threshold for exx minimization 
                     ! when using adaptive thresholds.
-  LOGICAL :: &
-      first, no_printout, exst
+  LOGICAL :: first, exst
   !
   !
   iter = 0
   first = .true.
   tr2_final = tr2
-  no_printout = dft_is_hybrid()
+  IF ( dft_is_hybrid() ) THEN
+     printout = 0  ! do not print etot and energy components at each scf step
+  ELSE IF ( lmd ) THEN
+     printout = 1  ! print etot, not energy components at each scf step
+  ELSE
+     printout = 2  ! print etot and energy components at each scf step
+  END IF
   IF (dft_is_hybrid() .AND. adapt_thr ) tr2= tr2_init
   fock0 = 0.D0
   fock1 = 0.D0
   IF (.NOT. exx_is_active () ) fock2 = 0.D0
   !
-  ! these routines can be used to patch quantities that are dependent
-  ! on the ions and cell parameters
-  !
-  CALL plugin_init_ions()
-  CALL plugin_init_cell()
-  !
-  CALL plugin_init_potential()
-  !
   !%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   !%%%%%%%%%%%%%%%%%%%%  Iterate hybrid functional  %%%%%%%%%%%%%%%%%%%%%
   !%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -130,7 +129,7 @@ SUBROUTINE electrons()
      ! ... Self-consistency loop. For hybrid functionals the exchange potential
      ! ... is calculated with the orbitals at previous step (none at first step)
      !
-     CALL electrons_scf ( no_printout )
+     CALL electrons_scf ( printout )
      !
      IF ( .NOT. dft_is_hybrid() ) RETURN
      !
@@ -139,6 +138,8 @@ SUBROUTINE electrons()
      IF ( stopped_by_user .OR. .NOT. conv_elec ) THEN
         conv_elec=.FALSE.
         IF ( .NOT. first) THEN
+           WRITE(stdout,'(5x,"Calculation (EXX) stopped during iteration #", &
+                        & i6)') iter
            CALL seqopn (iunres, 'restart_e', 'formatted', exst)
            WRITE (iunres, *) iter-1, tr2, dexx
            WRITE (iunres, *) fock0, fock1, fock2
@@ -225,6 +226,8 @@ SUBROUTINE electrons()
      WRITE( stdout,'(/5x,"EXX: now go back to refine exchange calculation")')
      !
      IF ( check_stop_now() ) THEN
+        WRITE(stdout,'(5x,"Calculation (EXX) stopped after iteration #", &
+                        & i6)') iter
         conv_elec=.FALSE.
         CALL seqopn (iunres, 'restart_e', 'formatted', exst)
         WRITE (iunres, *) iter, tr2, dexx
@@ -257,7 +260,7 @@ SUBROUTINE electrons()
 END SUBROUTINE electrons
 !
 !----------------------------------------------------------------------------
-SUBROUTINE electrons_scf ( no_printout )
+SUBROUTINE electrons_scf ( printout )
   !----------------------------------------------------------------------------
   !
   ! ... This routine is a driver of the self-consistent cycle.
@@ -265,8 +268,8 @@ SUBROUTINE electrons_scf ( no_printout )
   ! ... Hamiltonian, the routine sum_band to compute the charge density,
   ! ... the routine v_of_rho to compute the new potential and the routine
   ! ... mix_rho to mix input and output charge densities.
-  ! ... It prints on output the total energy and its decomposition in
-  ! ... the separate contributions (unless no_printout is .true.)
+  ! ... If printout > 0, prints on output the total energy;
+  ! ... if printout > 1, also prints decomposition into energy contributions
   !
   USE kinds,                ONLY : DP
   USE check_stop,           ONLY : check_stop_now, stopped_by_user
@@ -279,13 +282,13 @@ SUBROUTINE electrons_scf ( no_printout )
   USE gvect,                ONLY : ngm, gstart, nl, nlm, g, gg, gcutm
   USE gvecs,                ONLY : doublegrid, ngms
   USE klist,                ONLY : xk, wk, nelec, ngk, nks, nkstot, lgauss, &
-                                   two_fermi_energies
+                                   two_fermi_energies, tot_charge
   USE lsda_mod,             ONLY : lsda, nspin, magtot, absmag, isk
   USE vlocal,               ONLY : strf
   USE wvfct,                ONLY : nbnd, et, npwx, ecutwfc
   USE ener,                 ONLY : etot, hwf_energy, eband, deband, ehart, &
                                    vtxc, etxc, etxcc, ewld, demet, epaw, &
-                                   elondon, ef_up, ef_dw, exdm
+                                   elondon, ef_up, ef_dw, exdm, ef
   USE scf,                  ONLY : scf_type, scf_type_COPY, bcast_scf_type,&
                                    create_scf_type, destroy_scf_type, &
                                    open_mix_file, close_mix_file, &
@@ -321,14 +324,15 @@ SUBROUTINE electrons_scf ( no_printout )
   USE paw_symmetry,         ONLY : PAW_symmetrize_ddd
   USE uspp_param,           ONLY : nh, nhm ! used for PAW
   USE dfunct,               ONLY : newd
-  USE esm,                  ONLY : do_comp_esm, esm_printpot
+  USE esm,                  ONLY : do_comp_esm, esm_printpot, esm_ewald
+  USE fcp_variables,        ONLY : lfcpopt, lfcpdyn
   USE iso_c_binding,        ONLY : c_int
   !
   USE plugin_variables,     ONLY : plugin_etot
   !
   IMPLICIT NONE
   !
-  LOGICAL, INTENT (IN) :: no_printout
+  INTEGER, INTENT (IN) :: printout
   !
   ! ... a few local variables
   !
@@ -382,8 +386,12 @@ SUBROUTINE electrons_scf ( no_printout )
   !
   ! ... calculates the ewald contribution to total energy
   !
-  ewld = ewald( alat, nat, nsp, ityp, zv, at, bg, tau, &
+  IF ( do_comp_esm ) THEN
+     ewld = esm_ewald()
+  ELSE
+     ewld = ewald( alat, nat, nsp, ityp, zv, at, bg, tau, &
                 omega, g, gg, ngm, gcutm, gstart, gamma_only, strf )
+  END IF
   !
   IF ( llondon ) THEN
      elondon = energy_london ( alat , nat , ityp , at ,bg , tau )
@@ -688,8 +696,9 @@ SUBROUTINE electrons_scf ( no_printout )
      IF ( lelfield ) etot = etot + en_el
      ! not sure about the HWF functional in the above case
      IF( textfor ) THEN
-        eext =  compute_eextfor()
+        eext = alat*compute_eextfor()
         etot = etot + eext
+        hwf_energy = hwf_energy + eext
      END IF
      IF (llondon) THEN
         etot = etot + elondon
@@ -712,11 +721,16 @@ SUBROUTINE electrons_scf ( no_printout )
         hwf_energy = hwf_energy + etotefield
      END IF
      !
+     IF ( lfcpopt .or. lfcpdyn ) THEN
+        etot = etot + ef * tot_charge
+        hwf_energy = hwf_energy + ef * tot_charge
+     ENDIF
+     !
      ! ... adds possible external contribution from plugins to the energy
      !
      etot = etot + plugin_etot 
      !
-     IF ( .NOT. no_printout ) CALL print_energies ( )
+     CALL print_energies ( printout )
      !
      IF ( conv_elec ) THEN
         !
@@ -1025,13 +1039,14 @@ SUBROUTINE electrons_scf ( no_printout )
      END FUNCTION calc_pol
      !
      !-----------------------------------------------------------------------
-     SUBROUTINE print_energies ( )
+     SUBROUTINE print_energies ( printout )
        !-----------------------------------------------------------------------
        !
        USE constants, ONLY : eps8
-       USE control_flags, ONLY : lmd
+       INTEGER, INTENT (IN) :: printout
        !
-       IF ( ( conv_elec .OR. MOD( iter, iprint ) == 0 ) .AND. .NOT. lmd ) THEN
+       IF ( printout == 0 ) RETURN
+       IF ( ( conv_elec .OR. MOD(iter,iprint) == 0 ) .AND. printout > 1 ) THEN
           !
           IF ( dr2 > eps8 ) THEN
              WRITE( stdout, 9081 ) etot, hwf_energy, dr2
@@ -1046,9 +1061,7 @@ SUBROUTINE electrons_scf ( no_printout )
           IF ( llondon ) WRITE ( stdout , 9074 ) elondon
           IF ( lxdm )    WRITE ( stdout , 9075 ) exdm
           IF ( ts_vdw )  WRITE ( stdout , 9076 ) 2.0d0*EtsvdW
-          !
-          IF ( textfor)             WRITE( stdout, &
-               '(/5x,"Energy of the external Forces = ", F18.8)' ) eext
+          IF ( textfor)  WRITE ( stdout , 9077 ) eext
           IF ( tefield )            WRITE( stdout, 9061 ) etotefield
           IF ( lda_plus_u )         WRITE( stdout, 9065 ) eth
           IF ( ABS (descf) > eps8 ) WRITE( stdout, 9069 ) descf
@@ -1059,7 +1072,13 @@ SUBROUTINE electrons_scf ( no_printout )
           !
           IF ( lgauss ) WRITE( stdout, 9070 ) demet
           !
-       ELSE IF ( conv_elec .AND. lmd ) THEN
+          ! ... With Fictitious charge particle (FCP), etot is the grand
+          ! ... potential energy Omega = E - muN, -muN is the potentiostat
+          ! ... contribution.
+          !
+          IF ( lfcpopt .or. lfcpdyn ) WRITE( stdout, 9072 ) ef*tot_charge
+          !
+       ELSE IF ( conv_elec ) THEN
           !
           IF ( dr2 > eps8 ) THEN
              WRITE( stdout, 9081 ) etot, hwf_energy, dr2
@@ -1107,10 +1126,12 @@ SUBROUTINE electrons_scf ( no_printout )
 9069 FORMAT( '     scf correction            =',F17.8,' Ry' )
 9070 FORMAT( '     smearing contrib. (-TS)   =',F17.8,' Ry' )
 9071 FORMAT( '     Magnetic field            =',3F12.7,' Ry' )
+9072 FORMAT( '     pot.stat. contrib. (-muN) =',F17.8,' Ry' )
 9073 FORMAT( '     lambda                    =',F11.2,' Ry' )
 9074 FORMAT( '     Dispersion Correction     =',F17.8,' Ry' )
 9075 FORMAT( '     Dispersion XDM Correction =',F17.8,' Ry' )
 9076 FORMAT( '     Dispersion T-S Correction =',F17.8,' Ry' )
+9077 FORMAT( '     External forces energy    =',F17.8,' Ry' )
 9080 FORMAT(/'     total energy              =',0PF17.8,' Ry' &
             /'     Harris-Foulkes estimate   =',0PF17.8,' Ry' &
             /'     estimated scf accuracy    <',0PF17.8,' Ry' )
