@@ -1,5 +1,5 @@
 
-! Copyright (C) 2002-2011 Quantum ESPRESSO group
+! Copyright (C) 2002-2015 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -16,7 +16,7 @@ SUBROUTINE iosys()
   ! ...  those in input_parameters, are locally renamed by adding a "_"
   !
   USE kinds,         ONLY : DP
-  USE funct,         ONLY : dft_has_finite_size_correction, &
+  USE funct,         ONLY : dft_is_hybrid, dft_has_finite_size_correction, &
                             set_finite_size_volume, get_inlc 
   USE funct,         ONLY: set_exx_fraction, set_screening_parameter
   USE control_flags, ONLY: adapt_thr, tr2_init, tr2_multi
@@ -38,9 +38,10 @@ SUBROUTINE iosys()
                             lorbm_     => lorbm, &
                             efield_    => efield, &
                             nberrycyc_ => nberrycyc, &
-                            efield_cart_ => efield_cart
+                            efield_cart_ => efield_cart, &
+                            phase_control
   !
-  USE cell_base,     ONLY : at, alat, omega, &
+  USE cell_base,     ONLY : at, alat, omega, bg, &
                             cell_base_init, init_dofree
   !
   USE ions_base,     ONLY : if_pos, ityp, tau, extfor, &
@@ -63,6 +64,14 @@ SUBROUTINE iosys()
                               nraise_     => nraise, &
                               refold_pos_ => refold_pos
   !
+  USE fcp_variables, ONLY : lfcpopt_ => lfcpopt, &
+                            lfcpdyn_ => lfcpdyn, &
+                            fcp_mu_ => fcp_mu, &
+                            fcp_mass_ => fcp_mass, &
+                            fcp_temperature, &
+                            fcp_relax_step_ => fcp_relax_step, &
+                            fcp_relax_crit_ => fcp_relax_crit
+  !
   USE extfield,      ONLY : tefield_  => tefield, &
                             dipfield_ => dipfield, &
                             edir_     => edir, &
@@ -108,7 +117,8 @@ SUBROUTINE iosys()
                            esm_bc_ => esm_bc, &
                            esm_nfit_ => esm_nfit, &
                            esm_efield_ => esm_efield, &
-                           esm_w_ => esm_w
+                           esm_w_ => esm_w, &
+                           esm_a_ => esm_a
   !
   USE a2F,           ONLY : la2F_ => la2F
   !
@@ -201,7 +211,8 @@ SUBROUTINE iosys()
                                gdir, nppstr, wf_collect,lelfield,lorbm,efield, &
                                nberrycyc, lkpoint_dir, efield_cart, lecrpa,    &
                                vdw_table_name, memory, tqmmm,                  &
-                               lcalc_z2, z2_m_threshold, z2_z_threshold
+                               lcalc_z2, z2_m_threshold, z2_z_threshold,       &
+                               efield_phase
 
   !
   ! ... SYSTEM namelist
@@ -226,11 +237,15 @@ SUBROUTINE iosys()
                                angle1, angle2, constrained_magnetization,     &
                                B_field, fixed_magnetization, report, lspinorb,&
                                starting_spin_angle, assume_isolated,spline_ps,&
-                               vdw_corr, london, london_s6, london_rcut,      &
+                               vdw_corr, london, london_s6, london_rcut, london_c6, &
                                ts_vdw, ts_vdw_isolated, ts_vdw_econv_thr,     &
                                xdm, xdm_a1, xdm_a2,                           &
                                one_atom_occupations,                          &
-                               esm_bc, esm_efield, esm_w, esm_nfit
+                               esm_bc, esm_efield, esm_w, esm_nfit, esm_a,    &
+                               lfcpopt, lfcpdyn, fcp_mu, fcp_mass, fcp_tempw, & 
+                               fcp_relax_step, fcp_relax_crit,                &
+                               space_group, uniqueb, origin_choice,           &
+                               rhombohedral
   !
   ! ... ELECTRONS namelist
   !
@@ -244,12 +259,13 @@ SUBROUTINE iosys()
   !
   ! ... IONS namelist
   !
-  USE input_parameters, ONLY : phase_space, ion_dynamics, ion_positions, tolp, &
+  USE input_parameters, ONLY : ion_dynamics, ion_positions, tolp, &
                                tempw, delta_t, nraise, ion_temperature,        &
                                refold_pos, remove_rigid_rot, upscale,          &
                                pot_extrapolation,  wfc_extrapolation,          &
                                w_1, w_2, trust_radius_max, trust_radius_min,   &
-                               trust_radius_ini, bfgs_ndim
+                               trust_radius_ini, bfgs_ndim, rd_pos, sp_pos, &
+                               rd_for, rd_if_pos => if_pos, lsg
   !
   ! ... CELL namelist
   !
@@ -270,19 +286,20 @@ SUBROUTINE iosys()
   !
   USE constraints_module,    ONLY : init_constraint
   USE read_namelists_module, ONLY : read_namelists, sm_not_set
-  USE london_module,         ONLY : init_london, lon_rcut, scal6
+  USE london_module,         ONLY : init_london, lon_rcut, scal6, in_c6
   USE xdm_module,            ONLY : init_xdm, a1i, a2i
   USE tsvdw_module,          ONLY : vdw_isolated, vdw_econv_thr
   USE us,                    ONLY : spline_ps_ => spline_ps
   !
-  USE input_parameters,       ONLY : deallocate_input_parameters
+  USE input_parameters,      ONLY : deallocate_input_parameters
+  USE wyckoff,               ONLY : nattot, sup_spacegroup
   !
   IMPLICIT NONE
   !
   CHARACTER(LEN=256), EXTERNAL :: trimcheck
   INTEGER, EXTERNAL :: read_config_from_file
   !
-  INTEGER  :: ia, nt, inlc, ierr
+  INTEGER  :: ia, nt, inlc, ibrav_sg, ierr
   LOGICAL  :: exst, parallelfs
   REAL(DP) :: theta, phi
   !
@@ -458,7 +475,7 @@ SUBROUTINE iosys()
             '(5x,"Presently no symmetry can be used with electric field",/)' )
   ENDIF
   IF ( tefield .and. tstress ) THEN
-     tstress = .false.
+     lstres = .false.
      WRITE( stdout, &
             '(5x,"Presently stress not available with electric field",/)' )
   ENDIF
@@ -764,7 +781,15 @@ SUBROUTINE iosys()
   CASE( 'restart' )
      !
      restart = .true.
+     IF ( TRIM(startingwfc) /= 'file' ) THEN
+        CALL infomsg('input','WARNING: "startingwfc" set to '//TRIM(startingwfc)//' may spoil restart')
+     END IF
+     IF ( TRIM(startingpot) /= 'file' ) THEN
+        CALL infomsg('input','WARNING: "startingpot" set to '//TRIM(startingpot)//' may spoil restart')
+        startingpot = 'file'
+     END IF
      IF ( trim( ion_positions ) == 'from_input' ) THEN
+        CALL infomsg('input','WARNING: restarting from positions as given in input')
         startingconfig = 'input'
      ELSE
         startingconfig = 'file'
@@ -800,7 +825,15 @@ SUBROUTINE iosys()
      !
   CASE DEFAULT
      !
-     io_level = 0
+     ! In the scf case, it is usually convenient to write to RAM;
+     ! otherwise it is preferrable to write to disk, since the number
+     ! of k-points can be large, leading to large RAM requirements
+     !
+     IF ( lscf ) THEN
+        io_level = 0
+     ELSE
+        io_level = 1
+     END IF
      !
   END SELECT
   !
@@ -1018,11 +1051,13 @@ SUBROUTINE iosys()
      iverbosity = 0
   END SELECT
   !
-  IF ( lberry .OR. lelfield ) THEN
+  IF ( lberry .OR. lelfield .OR. lorbm ) THEN
      IF ( npool > 1 ) CALL errore( 'iosys', &
           'Berry Phase/electric fields not implemented with pools', 1 )
      IF ( lgauss .OR. ltetra ) CALL errore( 'iosys', &
           'Berry Phase/electric fields only for insulators!', 1 )
+     IF ( lmovecell ) CALL errore( 'iosys', &
+          'Berry Phase/electric fields not implemented with variable cell', 1 )
   END IF
   !
   ! ... Copy values from input module to PW internals
@@ -1038,6 +1073,17 @@ SUBROUTINE iosys()
   efield_     = efield
   nberrycyc_  = nberrycyc
   efield_cart_ = efield_cart
+  SELECT CASE(efield_phase)
+     CASE( 'none' )
+        phase_control=0
+     CASE ('write')
+        phase_control=1
+     CASE ('read')
+        phase_control=2
+     CASE DEFAULT
+        CALL errore( 'iosys', &
+          'Unknown efield_phase', 1 )
+  END SELECT
   tqr_        = tqr
   real_space_ = real_space
   !
@@ -1107,18 +1153,6 @@ SUBROUTINE iosys()
   nofrac                  = force_symmorphic
   nbnd_                   = nbnd
   !
-  x_gamma_extrapolation_ = x_gamma_extrapolation
-  !
-  nqx1_ = nqx1
-  nqx2_ = nqx2
-  nqx3_ = nqx3
-  !
-  exxdiv_treatment_ = trim(exxdiv_treatment)
-  yukawa_   = yukawa
-  ecutvcut_ = ecutvcut
-  ecutfock_ = ecutfock
-  !
-  vdw_table_name_  = vdw_table_name
   !
   diago_full_acc_ = diago_full_acc
   starting_wfc    = startingwfc
@@ -1148,13 +1182,6 @@ SUBROUTINE iosys()
   w_1_              = w_1
   w_2_              = w_2
   !
-  ! ... ESM
-  !
-  esm_bc_ = esm_bc
-  esm_efield_ = esm_efield
-  esm_w_ = esm_w
-  esm_nfit_ = esm_nfit
-  !
   IF (trim(occupations) /= 'from_input') one_atom_occupations_=.false.
   !
   !  ... initialize variables for vdW (dispersions) corrections
@@ -1204,6 +1231,7 @@ SUBROUTINE iosys()
   IF ( llondon) THEN
      lon_rcut    = london_rcut
      scal6       = london_s6
+     in_c6(:)    = london_c6(:)
   END IF
   IF ( lxdm ) THEN
      a1i = xdm_a1
@@ -1214,6 +1242,29 @@ SUBROUTINE iosys()
      vdw_econv_thr= ts_vdw_econv_thr
   END IF
   !
+  !  calculate all the atomic positions if only the inequivalent ones
+  !  have been given.
+  !  NB: ibrav is an output of this routine
+  !
+  IF (space_group /= 0 .AND. .NOT. lsg ) &
+     CALL errore('input','space_group requires crystal_sg atomic &
+                                                   &coordinates',1 )
+  IF (lsg) THEN
+     IF (space_group==0) &
+        CALL errore('input','The option crystal_sg requires the space group &
+                                                   &number',1 )
+        
+     CALL sup_spacegroup(rd_pos,sp_pos,rd_for,rd_if_pos,space_group,nat,&
+              uniqueb,rhombohedral,origin_choice,ibrav_sg)
+     IF (ibrav==-1) THEN
+        ibrav=ibrav_sg
+     ELSEIF (ibrav /= ibrav_sg) THEN
+        CALL errore ('input','Input ibrav not compatible with space group &
+                                                   &number',1 )
+     ENDIF
+     nat_=nattot
+  ENDIF
+  !
   ! QM/MM specific parameters
   !
   IF (.NOT. tqmmm) CALL qmmm_config( mode=-1 )
@@ -1244,15 +1295,66 @@ SUBROUTINE iosys()
       !
   END SELECT
   !
+  IF ( do_comp_mt .AND. lstres ) THEN
+     lstres = .false.
+     WRITE( stdout, &
+          '(5x,"Stress calculation not meaningful in isolated systems",/)' )
+  END IF
+  !
+  ! ... ESM
+  !
+  esm_bc_ = esm_bc
+  esm_efield_ = esm_efield
+  esm_w_ = esm_w
+  esm_nfit_ = esm_nfit 
+  esm_a_ = esm_a
+  !
+  IF ( esm_bc .EQ. 'bc4' ) THEN
+    IF ( ABS(esm_w) .LT. 1.D-8 ) THEN
+      CALL errore ('iosys','esm_w too small',1)
+    ELSEIF ( esm_w .GT. 0.D0 ) THEN
+      CALL errore ('iosys','positive esm_w not allowed for bc4',1)
+    ENDIF
+    IF ( esm_a .LT. 1.D-4 ) THEN
+      CALL errore ('iosys','smoothness parameter for bc4 too small',1)
+    ELSEIF ( esm_a .GT. 10.D0 ) THEN
+      CALL errore ('iosys','smoothness parameter for bc4 too big',1)
+    ENDIF
+  ENDIF
+  !
+  ! ... FCP
+  !
+  lfcpopt_        = lfcpopt
+  lfcpdyn_        = lfcpdyn
+  fcp_mu_         = fcp_mu
+  fcp_mass_       = fcp_mass
+  fcp_temperature = fcp_tempw
+  !
+  IF ( lfcpopt .or. lfcpdyn ) THEN
+     IF ( .not. do_comp_esm ) THEN
+        CALL errore ('iosys','FCP optimise/dynamics currently not available without ESM',1)
+     ENDIF
+     IF ( trim( calculation ).NE.'relax'.AND.trim( calculation ).NE.'md')THEN
+        CALL errore ('iosys',"FCP optimise/dynamics only available with calculation = 'relax' and 'md'",1)
+     ENDIF
+  ENDIF
+  !
+  IF ( fcp_temperature == 0.0_DP ) &
+     fcp_temperature = temperature
+  fcp_relax_step_ = fcp_relax_step
+  fcp_relax_crit_ = fcp_relax_crit
+  !
   CALL plugin_read_input()
   !
   ! ... read following cards
   !
+
   ALLOCATE( ityp( nat_ ) )
   ALLOCATE( tau(    3, nat_ ) )
   ALLOCATE( force(  3, nat_ ) )
   ALLOCATE( if_pos( 3, nat_ ) )
   ALLOCATE( extfor( 3, nat_ ) )
+
   IF ( tfixed_occ ) THEN
      IF ( nspin_ == 4 ) THEN
         ALLOCATE( f_inp( nbnd_, 1 ) )
@@ -1285,11 +1387,16 @@ SUBROUTINE iosys()
      wfc_dir = tmp_dir
   ENDIF
   !
+!   IF ( lmovecell ) THEN
+  at_old    = at
+  omega_old = omega
+!   ENDIF
+  !
   ! ... Read atomic positions and unit cell from data file, if needed,
   ! ... overwriting what has just been read before from input
   !
   ierr = 1
-  IF ( startingconfig == 'file' ) ierr = read_config_from_file()
+  IF ( startingconfig == 'file' )   ierr = read_config_from_file(nat, at_old,omega_old, lmovecell, at, bg, omega, tau)
   !
   ! ... read_config_from_file returns 0 if structure successfully read
   ! ... Atomic positions (tau) must be converted to internal units
@@ -1348,7 +1455,31 @@ SUBROUTINE iosys()
   !
   CALL readpp ( input_dft )
   !
-  ! Set variables for hybrid functional HSE
+  ! ... set parameters of hybrid functionals
+  !
+  x_gamma_extrapolation_ = x_gamma_extrapolation
+  !
+  nqx1_ = nqx1
+  nqx2_ = nqx2
+  nqx3_ = nqx3
+  !
+  exxdiv_treatment_ = trim(exxdiv_treatment)
+  yukawa_   = yukawa
+  ecutvcut_ = ecutvcut
+  !
+  IF(ecutfock <= 0.0_DP) THEN
+     ! default case
+     ecutfock_ = 4.0_DP*ecutwfc
+  ELSE
+     IF(ecutfock < ecutwfc .OR. ecutfock > ecutrho) CALL errore('iosys', &
+          'ecutfock can not be < ecutwfc or > ecutrho!', 1) 
+     ecutfock_ = ecutfock
+  END IF
+  IF ( lstres .AND. dft_is_hybrid() .AND. npool > 1 )  CALL errore('iosys', &
+         'stress for hybrid functionals not available with pools', 1)
+  !
+  ! ... must be done AFTER dft is read from PP files and initialized
+  ! ... or else the two following parameters will be overwritten
   !
   IF (exx_fraction >= 0.0_DP) CALL set_exx_fraction (exx_fraction)
   IF (screening_parameter >= 0.0_DP) &
@@ -1356,10 +1487,9 @@ SUBROUTINE iosys()
   !
   ! ... read the vdw kernel table if needed
   !
+  vdw_table_name_  = vdw_table_name
   inlc = get_inlc()
-  if (inlc > 0) then
-      call initialize_kernel_table(inlc)
-  endif
+  IF (inlc > 0) CALL initialize_kernel_table(inlc)
   !
   ! ... if DFT finite size corrections are needed, define the appropriate volume
   !
@@ -1370,9 +1500,9 @@ SUBROUTINE iosys()
   ! ... and initialize a few other variables
   !
   IF ( lmovecell ) THEN
-     !
-     at_old    = at
-     omega_old = omega
+     ! The next two lines have been moved before the call to read_config_from_file:
+     !      at_old    = at
+     !      omega_old = omega
      IF ( cell_factor_ <= 0.D0 ) cell_factor_ = 1.2D0
      !
      IF ( cmass <= 0.D0 ) &
@@ -1419,8 +1549,6 @@ SUBROUTINE iosys()
   IF ( TRIM(wfc_dir) /= TRIM(tmp_dir) ) &
      CALL check_tempdir( wfc_dir, exst, parallelfs )
 
-  ! CALL restart_from_file()
-  !
   RETURN
   !
 END SUBROUTINE iosys
@@ -1432,13 +1560,16 @@ SUBROUTINE read_cards_pw ( psfile, tau_format )
   USE kinds,              ONLY : DP
   USE input_parameters,   ONLY : atom_label, atom_pfile, atom_mass, taspc, &
                                  tapos, rd_pos, atomic_positions, if_pos,  &
-                                 sp_pos, f_inp, rd_for, tavel, sp_vel, rd_vel
+                                 sp_pos, f_inp, rd_for, tavel, sp_vel, rd_vel, &
+                                 lsg
   USE dynamics_module,    ONLY : vel
   USE cell_base,          ONLY : at, ibrav
   USE ions_base,          ONLY : nat, ntyp => nsp, ityp, tau, atm, extfor
   USE fixed_occ,          ONLY : tfixed_occ, f_inp_ => f_inp
   USE ions_base,          ONLY : if_pos_ =>  if_pos, amass, fixatom
   USE control_flags,      ONLY : textfor, tv0rd
+  USE wyckoff,            ONLY : nattot, tautot, ityptot, extfortot, &
+                                 if_postot, clean_spacegroup
   !
   IMPLICIT NONE
   !
@@ -1473,13 +1604,22 @@ SUBROUTINE read_cards_pw ( psfile, tau_format )
   textfor = .false.
   IF( any( rd_for /= 0.0_DP ) ) textfor = .true.
   !
-  DO ia = 1, nat
-     !
-     tau(:,ia) = rd_pos(:,ia)
-     ityp(ia)  = sp_pos(ia)
-     extfor(:,ia) = rd_for(:,ia)
-     !
-  ENDDO
+  IF (lsg) THEN
+     tau(:,:)=tautot(:,:)
+     ityp(:) = ityptot(:)
+     extfor(:,:) = extfortot(:,:)
+     if_pos_(:,:) = if_postot(:,:)
+     CALL clean_spacegroup()
+  ELSE 
+     DO ia = 1, nat
+        !
+        tau(:,ia) = rd_pos(:,ia)
+        ityp(ia)  = sp_pos(ia)
+        extfor(:,ia) = rd_for(:,ia)
+        if_pos_(:,ia) = if_pos(:,ia)
+        !
+     ENDDO
+  ENDIF
   !
   ! ... check for initial velocities read from input file
   !
@@ -1498,7 +1638,6 @@ SUBROUTINE read_cards_pw ( psfile, tau_format )
   ! ... if_pos whose value is 0 when the coordinate is to be kept fixed, 1
   ! ... otherwise. 
   !
-  if_pos_(:,:) = if_pos(:,1:nat)
   fixatom = COUNT( if_pos_(1,:)==0 .AND. if_pos_(2,:)==0 .AND. if_pos_(3,:)==0 )
   !
   tau_format = trim( atomic_positions )
