@@ -25,14 +25,14 @@ MODULE pw_restart
                           qexml_write_spin, qexml_write_magnetization, &
                           qexml_write_xc, qexml_write_exx, qexml_write_occ, &
                           qexml_write_bz,qexml_write_para, qexml_write_bands_info, &
-                          qexml_write_bands_pw, qexml_wfc_filename, &
+                          qexml_write_bands_pw, qexml_write_esm, qexml_wfc_filename, &
                           default_fmt_version => qexml_default_version, &
                           qexml_save_history, qexml_kpoint_dirname, &
                           qexml_read_header, qexml_read_cell, qexml_read_moving_cell, &
                           qexml_read_planewaves, qexml_read_ions, qexml_read_spin, &
                           qexml_read_magnetization, qexml_read_xc, qexml_read_occ, qexml_read_bz, &
                           qexml_read_bands_info, qexml_read_bands_pw, qexml_read_symmetry, &
-                          qexml_read_efield, qexml_read_para, qexml_read_exx
+                          qexml_read_efield, qexml_read_para, qexml_read_exx, qexml_read_esm
   !
   USE xml_io_base, ONLY :  rho_binary,read_wfc, write_wfc, create_directory
   !
@@ -83,7 +83,8 @@ MODULE pw_restart
       !
       USE control_flags,        ONLY : istep, twfcollect, conv_ions, &
                                        lscf, lkpoint_dir, gamma_only, &
-                                       tqr, noinv, do_makov_payne, smallmem
+                                       tqr, noinv, do_makov_payne, smallmem, &
+                                       llondon, lxdm, ts_vdw
       USE realus,               ONLY : real_space
       USE global_version,       ONLY : version_number
       USE cell_base,            ONLY : at, bg, alat, tpiba, tpiba2, &
@@ -132,16 +133,18 @@ MODULE pw_restart
       USE mp_bands,             ONLY : nproc_bgrp, me_bgrp, root_bgrp, &
                                        intra_bgrp_comm, inter_bgrp_comm, &
                                        nbgrp, ntask_groups
-      USE mp_pots,              ONLY : nproc_pot
       USE mp_diag,              ONLY : nproc_ortho
       USE funct,                ONLY : get_exx_fraction, dft_is_hybrid, &
                                        get_gau_parameter, &
                                        get_screening_parameter, exx_is_active
       USE exx,                  ONLY : x_gamma_extrapolation, nq1, nq2, nq3, &
-                                       exxdiv_treatment, yukawa, ecutvcut
+                                       exxdiv_treatment, yukawa, ecutvcut, ecutfock
       USE cellmd,               ONLY : lmovecell, cell_factor 
       USE martyna_tuckerman,    ONLY : do_comp_mt
-      USE esm,                  ONLY : do_comp_esm
+      USE esm,                  ONLY : do_comp_esm, esm_nfit, esm_efield, esm_w, &
+                                       esm_a, esm_bc
+      USE london_module,        ONLY : scal6, lon_rcut
+      USE tsvdw_module,         ONLY : vdw_isolated
       
       !
       IMPLICIT NONE
@@ -432,14 +435,23 @@ MODULE pw_restart
                         HUBBARD_J0 = Hubbard_J0, HUBBARD_BETA = Hubbard_beta, &
                         HUBBARD_ALPHA = Hubbard_alpha, &
                         INLC = inlc, VDW_TABLE_NAME = vdw_table_name, &
-                        PSEUDO_DIR = pseudo_dir, DIRNAME = dirname)
+                        PSEUDO_DIR = pseudo_dir, DIRNAME = dirname,   &
+                        LLONDON = llondon, LONDON_S6 = scal6,         &
+                        LONDON_RCUT = lon_rcut, LXDM = lxdm,          &
+                        TS_VDW = ts_vdw, VDW_ISOLATED = vdw_isolated )
 
 
          IF ( dft_is_hybrid() ) CALL qexml_write_exx &
                        ( x_gamma_extrapolation, nq1, nq2, nq3, &
                          exxdiv_treatment, yukawa, ecutvcut, &
                          get_exx_fraction(), get_gau_parameter(), &
-                         get_screening_parameter(), exx_is_active() )
+                         get_screening_parameter(), exx_is_active(), ecutfock )
+         !
+!-------------------------------------------------------------------------------
+! ... ESM
+!-------------------------------------------------------------------------------
+         !
+         CALL qexml_write_esm( esm_nfit, esm_efield, esm_w, esm_a, esm_bc )
          !
 !-------------------------------------------------------------------------------
 ! ... OCCUPATIONS
@@ -462,8 +474,8 @@ MODULE pw_restart
 !-------------------------------------------------------------------------------
          !
          !
-         CALL qexml_write_para( kunit, nproc, nproc_pool, nproc_image, ntask_groups,&
-                          nproc_pot, nproc_bgrp, nproc_ortho )
+         CALL qexml_write_para( kunit, nproc, nproc_pool, nproc_image, &
+                                ntask_groups, nproc_bgrp, nproc_ortho )
          !
 !-------------------------------------------------------------------------------
 ! ... CHARGE DENSITY
@@ -669,20 +681,20 @@ MODULE pw_restart
           CHARACTER(LEN=256)  :: filename
           INTEGER :: ispin,ik_eff
           !
-          ! ... wavefunctions
+          ! ... wavefunctions - do not read if already in memory (nsk==1)
+          ! ...                 read only if on this pool (iks <= ik <= ike )
+          !
+          IF ( ( nks > 1 ) .AND. ( ik >= iks ) .AND. ( ik <= ike ) ) THEN
+              CALL get_buffer ( evc, nwordwfc, iunwfc, (ik-iks+1) )
+          END IF
           !
           IF ( nspin == 2 ) THEN
              !
-             ! ... beware: with pools, this is correct only on ionode
+             ! ... beware: with pools, isk(ik) has the correct value for 
+             ! ... all k-points only on first pool (ionode proc is ok)
              !
              ispin = isk(ik)
              !
-             IF ( ( ik >= iks ) .AND. ( ik <= ike ) ) THEN
-                !
-                CALL get_buffer ( evc, nwordwfc, iunwfc, (ik-iks+1) )
-                !
-             END IF
-             !
              IF ( ionode ) THEN
                 !
                 filename = qexml_wfc_filename( ".", 'evc', ik, ispin, &
@@ -705,6 +717,9 @@ MODULE pw_restart
              !
              ispin = isk(ik_eff)
              !
+             ! ... LSDA: now read minority wavefunctions (if not already
+             ! ... in memory and if they are on this pool)
+             !
              IF ( ( nks > 1 ) .AND. ( ik_eff >= iks ) .AND. ( ik_eff <= ike ) ) THEN
                 !
                 CALL get_buffer ( evc, nwordwfc, iunwfc, (ik_eff-iks+1) )
@@ -730,12 +745,6 @@ MODULE pw_restart
                              ionode, root_pool, intra_pool_comm, inter_pool_comm, intra_image_comm )
              !
           ELSE
-             !
-             IF ( ( nks > 1 ) .AND. ( ik >= iks ) .AND. ( ik <= ike ) ) THEN
-                !
-                CALL get_buffer( evc, nwordwfc, iunwfc, (ik-iks+1) )
-                !
-             END IF
              !
              IF ( noncolin ) THEN
                 !
@@ -754,7 +763,9 @@ MODULE pw_restart
                       !
                    END IF
                    !
-                   !!! TEMP
+                   ! TEMP  spin-up and spin-down spinor components are written
+                   ! TEMP  to different files, like in LSDA - not a smart way
+                   !
                    nkl=(ipol-1)*npwx+1
                    nkr= ipol   *npwx
                    CALL write_wfc( iunout, ik, nkstot, kunit, ipol, npol,   &
@@ -815,7 +826,7 @@ MODULE pw_restart
       LOGICAL            :: lcell, lpw, lions, lspin, linit_mag, &
                             lxc, locc, lbz, lbs, lwfc, lheader,          &
                             lsymm, lrho, lefield, ldim, &
-                            lef, lexx
+                            lef, lexx, lesm
       !
       LOGICAL            :: need_qexml
       INTEGER            :: tmp
@@ -852,6 +863,7 @@ MODULE pw_restart
       lefield = .FALSE.
       lef     = .FALSE.
       lexx    = .FALSE.
+      lesm    = .FALSE.
       !
       SELECT CASE( what )
       CASE( 'header' )
@@ -886,6 +898,20 @@ MODULE pw_restart
          lwfc  = .TRUE.
          need_qexml = .TRUE.
          !
+      CASE( 'nowavenobs' )
+         !
+         lcell   = .TRUE.
+         lpw     = .TRUE.
+         lions   = .TRUE.
+         lspin   = .TRUE.
+         linit_mag   = .TRUE.
+         lxc     = .TRUE.
+         locc    = .TRUE.
+         lbz     = .TRUE.
+         lsymm   = .TRUE.
+         lefield = .TRUE.
+         need_qexml = .TRUE.
+
       CASE( 'nowave' )
          !
          lcell   = .TRUE.
@@ -943,6 +969,11 @@ MODULE pw_restart
          lexx       = .TRUE.
          need_qexml = .TRUE.
          !
+      CASE( 'esm' )
+         !
+         lesm       = .TRUE.
+         need_qexml = .TRUE.
+         !
       END SELECT
       !
       IF ( .NOT. lheader .AND. .NOT. qexml_version_init) &
@@ -1102,7 +1133,7 @@ MODULE pw_restart
          !
          CALL read_ef( ierr )
          IF ( ierr > 0 ) THEN
-            errmsg='error reading Fermi energy in xml data file'
+            errmsg='error reading Fermi energy and number of electrons in xml data file'
             GOTO 100
          END IF
          !
@@ -1116,6 +1147,15 @@ MODULE pw_restart
          END IF
          !
       END IF
+      IF ( lesm ) THEN
+         !
+         CALL read_esm( ierr )
+         IF ( ierr > 0 ) THEN
+            errmsg='error reading ESM restart data in xml data file'
+            GOTO 100
+         END IF
+         !
+      END IF
       !
       IF (ionode .AND. need_qexml) THEN
          !
@@ -1195,8 +1235,7 @@ MODULE pw_restart
       USE mp_pools,         ONLY : kunit
       USE mp_global,        ONLY : nproc_file, nproc_pool_file, &
                                    nproc_image_file, ntask_groups_file, &
-                                   nproc_pot_file, nproc_bgrp_file, &
-                                   nproc_ortho_file
+                                   nproc_bgrp_file, nproc_ortho_file
       !
       IMPLICIT NONE
       !
@@ -1255,7 +1294,7 @@ MODULE pw_restart
          !
          CALL qexml_read_para( KUNIT=kunit, NPROC=nproc_file, NPROC_POOL=nproc_pool_file, &
               NPROC_IMAGE=nproc_image_file, NTASK_GROUPS = ntask_groups_file, &
-              NPROC_POT=nproc_pot_file, NPROC_BGRP=nproc_bgrp_file, NPROC_ORTHO=nproc_ortho_file, FOUND=found, IERR=ierr )
+              NPROC_BGRP=nproc_bgrp_file, NPROC_ORTHO=nproc_ortho_file, FOUND=found, IERR=ierr )
          IF ( ierr /= 0) GOTO 100
          !
          IF ( .NOT. found ) THEN
@@ -1265,7 +1304,6 @@ MODULE pw_restart
             nproc_pool_file=1
             nproc_image_file=1
             ntask_groups_file=1
-            nproc_pot_file=1
             nproc_bgrp_file=1
             nproc_ortho_file=1
             !
@@ -1304,7 +1342,6 @@ MODULE pw_restart
       CALL mp_bcast( nproc_pool_file,    ionode_id, intra_image_comm )
       CALL mp_bcast( nproc_image_file,   ionode_id, intra_image_comm )
       CALL mp_bcast( ntask_groups_file,  ionode_id, intra_image_comm )
-      CALL mp_bcast( nproc_pot_file,     ionode_id, intra_image_comm )
       CALL mp_bcast( nproc_bgrp_file,    ionode_id, intra_image_comm )
       CALL mp_bcast( nproc_ortho_file,   ionode_id, intra_image_comm )
       !
@@ -1849,6 +1886,9 @@ MODULE pw_restart
                             Hubbard_l, Hubbard_U, Hubbard_J, Hubbard_alpha, &
                             Hubbard_J0, Hubbard_beta, U_projection
       USE kernel_table, ONLY : vdw_table_name
+      USE control_flags,ONLY : llondon, lxdm, ts_vdw
+      USE london_module,ONLY : scal6, lon_rcut
+      USE tsvdw_module, ONLY : vdw_isolated
       !
       IMPLICIT NONE
       !
@@ -1869,7 +1909,8 @@ MODULE pw_restart
          CALL qexml_read_xc( dft_name, lda_plus_u, lda_plus_u_kind, U_projection,&
                              Hubbard_lmax, Hubbard_l, nsp_, Hubbard_U, Hubbard_J, &
                              Hubbard_J0, Hubbard_alpha, Hubbard_beta, &
-                             inlc, vdw_table_name,  ierr )
+                             inlc, vdw_table_name, llondon, scal6, &
+                             lon_rcut, lxdm, ts_vdw, vdw_isolated, ierr )
          !
       END IF
       !
@@ -2546,10 +2587,10 @@ MODULE pw_restart
     SUBROUTINE read_ef( ierr )
       !------------------------------------------------------------------------
       !
-      ! ... this routine reads only the Fermi energy
+      ! ... this routine reads the Fermi energy and the number of electrons
       !
       USE ener,  ONLY : ef, ef_up, ef_dw
-      USE klist, ONLY : two_fermi_energies
+      USE klist, ONLY : two_fermi_energies, nelec
       !
       IMPLICIT NONE
       INTEGER, INTENT(OUT) :: ierr
@@ -2558,7 +2599,8 @@ MODULE pw_restart
       !
       IF ( ionode ) THEN
          !
-         CALL qexml_read_bands_info( EF = ef , TWO_FERMI_ENERGIES=two_fermi_energies, EF_UP=ef_up, EF_DW=ef_dw, IERR=ierr )
+         CALL qexml_read_bands_info( EF = ef, EF_UP=ef_up, EF_DW=ef_dw, &
+            TWO_FERMI_ENERGIES=two_fermi_energies, NELEC=nelec, IERR=ierr )
          !
       END IF
       !
@@ -2583,6 +2625,7 @@ MODULE pw_restart
       CALL mp_bcast( ef, ionode_id, intra_image_comm )
       CALL mp_bcast( ef_up, ionode_id, intra_image_comm )
       CALL mp_bcast( ef_dw, ionode_id, intra_image_comm )
+      CALL mp_bcast( nelec, ionode_id, intra_image_comm )
       !
       RETURN
       !
@@ -2597,7 +2640,7 @@ MODULE pw_restart
       USE funct,                ONLY : set_exx_fraction, set_screening_parameter, &
                                        set_gau_parameter, enforce_input_dft, start_exx
       USE exx,                  ONLY : x_gamma_extrapolation, nq1, nq2, nq3, &
-                                       exxdiv_treatment, yukawa, ecutvcut
+                                       exxdiv_treatment, yukawa, ecutvcut, ecutfock
       IMPLICIT NONE
       !
       INTEGER,          INTENT(OUT) :: ierr
@@ -2609,7 +2652,7 @@ MODULE pw_restart
               NQX1=nq1, NQX2=nq2, NQX3=nq3, EXXDIV_TREATMENT=exxdiv_treatment, &
               YUKAWA = yukawa, ECUTVCUT=ecutvcut, EXX_FRACTION=exx_fraction, &
               SCREENING_PARAMETER=screening_parameter, GAU_PARAMETER=gau_parameter, &
-              EXX_IS_ACTIVE=exx_is_active, FOUND=found, IERR=ierr )
+              EXX_IS_ACTIVE=exx_is_active, ECUTFOCK=ecutfock, FOUND=found, IERR=ierr )
          !
       ENDIF
       !
@@ -2631,6 +2674,7 @@ MODULE pw_restart
       CALL mp_bcast( screening_parameter, ionode_id, intra_image_comm )
       CALL mp_bcast( gau_parameter, ionode_id, intra_image_comm )
       CALL mp_bcast( exx_is_active, ionode_id, intra_image_comm )
+      CALL mp_bcast( ecutfock, ionode_id, intra_image_comm )
       !
       CALL set_exx_fraction(exx_fraction)
       CALL set_screening_parameter(screening_parameter)
@@ -2642,6 +2686,39 @@ MODULE pw_restart
     END SUBROUTINE read_exx
     !
     !------------------------------------------------------------------------
+    SUBROUTINE read_esm( ierr )
+      !------------------------------------------------------------------------
+      !
+      ! ... this routine reads only nelec and ef
+      !
+      USE esm, ONLY : esm_nfit, esm_efield, esm_w, esm_a, esm_bc
+      !
+      IMPLICIT NONE
+      INTEGER, INTENT(OUT) :: ierr
+      !
+      ! ... then selected tags are read from the other sections
+      !
+      IF ( ionode ) THEN
+         !
+         CALL qexml_read_esm( ESM_NFIT = esm_nfit, ESM_EFIELD = esm_efield, &
+           ESM_W = esm_w, ESM_A = esm_a, ESM_BC = esm_bc, IERR=ierr )
+         !
+      END IF
+      !
+      CALL mp_bcast( ierr, ionode_id, intra_image_comm )
+      IF ( ierr > 0 ) RETURN
+      !
+      CALL mp_bcast( esm_nfit,    ionode_id, intra_image_comm )
+      CALL mp_bcast( esm_efield,  ionode_id, intra_image_comm )
+      CALL mp_bcast( esm_w,       ionode_id, intra_image_comm )
+      CALL mp_bcast( esm_a,       ionode_id, intra_image_comm )
+      CALL mp_bcast( esm_bc,      ionode_id, intra_image_comm )
+      !
+      RETURN
+      !
+    END SUBROUTINE read_esm
+    !
+    !------------------------------------------------------------------------
     SUBROUTINE read_( dirname, ierr )
       !------------------------------------------------------------------------
       !
