@@ -33,7 +33,7 @@ SUBROUTINE setup()
   ! ...    electric-field, LDA+U calculations, and for parallelism
   !
   USE kinds,              ONLY : DP
-  USE constants,          ONLY : eps8, rytoev 
+  USE constants,          ONLY : eps8, rytoev, fpi
   USE parameters,         ONLY : npk
   USE io_global,          ONLY : stdout
   USE io_files,           ONLY : tmp_dir, prefix, xmlpun, delete_if_present
@@ -41,10 +41,10 @@ SUBROUTINE setup()
   USE cell_base,          ONLY : at, bg, alat, tpiba, tpiba2, ibrav, omega
   USE ions_base,          ONLY : nat, tau, ntyp => nsp, ityp, zv
   USE basis,              ONLY : starting_pot, natomwfc
-  USE gvect,              ONLY : gcutm
+  USE gvect,              ONLY : gcutm, ecutrho
   USE fft_base,           ONLY : dfftp
   USE fft_base,           ONLY : dffts
-  USE grid_subroutines,   ONLY : realspace_grids_init
+  USE grid_subroutines,   ONLY : realspace_grid_init
   USE gvecs,              ONLY : doublegrid, gcutms, dual
   USE klist,              ONLY : xk, wk, nks, nelec, degauss, lgauss, &
                                  lxkcry, nkstot, &
@@ -77,10 +77,12 @@ SUBROUTINE setup()
                                  angle1, angle2, bfield, ux, nspin_lsda, &
                                  nspin_gga, nspin_mag
   USE pw_restart,         ONLY : pw_readfile
-  USE exx,                ONLY : exx_grid_init, exx_div_check
+  USE exx,                ONLY : ecutfock, exx_grid_init, exx_div_check
   USE funct,              ONLY : dft_is_meta, dft_is_hybrid, dft_is_gradient
   USE paw_variables,      ONLY : okpaw
   USE cellmd,             ONLY : lmovecell  
+  USE control_flags,      ONLY : restart
+  USE fcp_variables,      ONLY : lfcpopt, lfcpdyn
   !
   IMPLICIT NONE
   !
@@ -92,27 +94,36 @@ SUBROUTINE setup()
   !
   ! ... okvan/okpaw = .TRUE. : at least one pseudopotential is US/PAW
   !
-  okvan = ANY( upf(:)%tvanp )
+  okvan = ANY( upf(1:ntyp)%tvanp )
   okpaw = ANY( upf(1:ntyp)%tpawp )
   !
   ! ... check for features not implemented with US-PP or PAW
   !
   IF ( okvan .OR. okpaw ) THEN
      IF ( dft_is_meta() ) CALL errore( 'setup', &
-                          'US/PAW and Meta-GGA not yet implemented', 1 )
-     IF ( noncolin .AND. lberry)  CALL errore( 'iosys', &
-       'Noncolinear Berry Phase/electric not implemented with USPP', 1 )
-     IF  (ts_vdw ) CALL errore ('iosys',&
-                    'Tkatchenko-Scheffler not implemented with USPP',1)
+                               'Meta-GGA not implemented with USPP/PAW', 1 )
+     IF ( noncolin .AND. lberry)  CALL errore( 'setup', &
+       'Noncolinear Berry Phase/electric not implemented with USPP/PAW', 1 )
+     IF  (ts_vdw ) CALL errore ('setup',&
+                   'Tkatchenko-Scheffler not implemented with USPP/PAW', 1 )
+     IF ( lorbm ) CALL errore( 'setup', &
+                  'Orbital Magnetization not implemented with USPP/PAW', 1 )
   END IF
 
   IF ( dft_is_hybrid() ) THEN
      IF (.NOT. lscf) CALL errore( 'setup ', &
-                         'HYBRID XC not allowed in non-scf calculations', 1 )
+                         'hybrid XC not allowed in non-scf calculations', 1 )
      IF ( ANY (upf(1:ntyp)%nlcc) ) CALL infomsg( 'setup ', 'BEWARE:' // &
                & ' nonlinear core correction is not consistent with hybrid XC')
-     IF (lmovecell) CALL errore('setup','Variable cell and EXX not tested!',1)
-     IF (noncolin) no_t_rev=.true.
+     IF (lmovecell) CALL errore('setup','Variable cell and hybrid XC not tested',1)
+     IF (okpaw) CALL errore('setup','PAW and hybrid XC not tested',1)
+     IF (okvan) THEN
+        IF (ecutfock /= 4*ecutwfc) CALL infomsg &
+           ('setup','Warning: US/PAW use ecutfock=4*ecutwfc, ecutfock ignored')
+        IF ( noncolin ) CALL errore &
+           ('setup','Noncolinear hybrid XC for USPP not implemented',1)
+     END IF
+     IF ( noncolin ) no_t_rev=.true.
   END IF
   !
   ! ... Compute the ionic charge for each atom type and the total ionic charge
@@ -132,6 +143,16 @@ SUBROUTINE setup()
   !
   nelec = ionic_charge - tot_charge
   !
+  IF ( lfcpopt .AND. restart ) THEN
+     CALL pw_readfile( 'fcpopt', ierr )
+     tot_charge = ionic_charge - nelec
+  END IF
+  !
+  IF ( lfcpdyn .AND. restart ) THEN
+     CALL pw_readfile( 'fcpdyn', ierr )
+     tot_charge = ionic_charge - nelec
+  END IF
+  !
   ! ... magnetism-related quantities
   !
   ALLOCATE( m_loc( 3, nat ) )
@@ -384,7 +405,14 @@ SUBROUTINE setup()
   !
   ! ... calculate dimensions of the FFT grid
   !
-  CALL realspace_grids_init ( dfftp, dffts, at, bg, gcutm, gcutms )
+  CALL realspace_grid_init ( dfftp, at, bg, gcutm )
+  IF ( gcutms == gcutm ) THEN
+     ! ... No double grid, the two grids are the same
+     dffts%nr1 = dfftp%nr1 ; dffts%nr2 = dfftp%nr2 ; dffts%nr3 = dfftp%nr3
+     dffts%nr1x= dfftp%nr1x; dffts%nr2x= dfftp%nr2x; dffts%nr3x= dfftp%nr3x
+  ELSE
+     CALL realspace_grid_init ( dffts, at, bg, gcutms)
+  END IF
   !
   !  ... generate transformation matrices for the crystal point group
   !  ... First we generate all the symmetry matrices of the Bravais lattice
