@@ -1,5 +1,5 @@
 ! 
-! Copyright (C) 2001-2007 Quantum ESPRESSO group
+! Copyright (C) 2001-2013 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -12,6 +12,7 @@ SUBROUTINE wfcinit()
   !
   ! ... This routine computes an estimate of the starting wavefunctions
   ! ... from superposition of atomic wavefunctions and/or random wavefunctions.
+  ! ... It also open needed files or memory buffers
   !
   USE io_global,            ONLY : stdout
   USE basis,                ONLY : natomwfc, starting_wfc
@@ -19,34 +20,67 @@ SUBROUTINE wfcinit()
   USE klist,                ONLY : xk, nks, ngk
   USE control_flags,        ONLY : io_level, lscf
   USE fixed_occ,            ONLY : one_atom_occupations
-  USE ldaU,                 ONLY : swfcatom, lda_plus_u, U_projection
+  USE ldaU,                 ONLY : lda_plus_u, U_projection, wfcU
   USE lsda_mod,             ONLY : lsda, current_spin, isk
-  USE io_files,             ONLY : nwordwfc, nwordatwfc, iunwfc, iunigk, iunsat
-  USE buffers,              ONLY : get_buffer, save_buffer
+  USE io_files,             ONLY : nwordwfc, nwordwfcU, iunhub, iunwfc, iunigk
+  USE buffers,              ONLY : open_buffer, get_buffer, save_buffer
   USE uspp,                 ONLY : nkb, vkb
   USE wavefunctions_module, ONLY : evc
   USE wvfct,                ONLY : nbnd, npw, current_k, igk
-  USE wannier_new,              ONLY : use_wannier
+  USE wannier_new,          ONLY : use_wannier
+  USE pw_restart,           ONLY : pw_readfile
   !
   IMPLICIT NONE
   !
-  INTEGER :: ik
+  INTEGER :: ik, ierr
+  LOGICAL :: exst_mem, exst_file
   !
   !
   CALL start_clock( 'wfcinit' )
   !
-  ! ... Needed for LDA+U
+  ! ... Orthogonalized atomic functions needed for LDA+U and other cases
   !
-  IF ( use_wannier .OR. one_atom_occupations ) CALL orthoatwfc()
-  IF ( lda_plus_u ) THEN
-     IF ( U_projection == 'pseudo' ) THEN
-         WRITE( stdout,*) 'Beta functions used for LDA+U Projector'
-     ELSE
-         CALL orthoatwfc()
-     ENDIF
-  ENDIF
+  IF ( use_wannier .OR. one_atom_occupations ) CALL orthoatwfc ( use_wannier )
+  IF ( lda_plus_u ) CALL orthoUwfc()
   !
-  ! ... state what is going to happen
+  ! ... open files/buffer for wavefunctions (nwordwfc set in openfil)
+  ! ... io_level > 1 : open file, otherwise: open buffer
+  !
+  CALL open_buffer( iunwfc, 'wfc', nwordwfc, io_level, exst_mem, exst_file )
+  !
+  ! ... now the various possible wavefunction initializations
+  ! ... first a check: is "tmp_dir"/"prefix".wfc found on disk?
+  !
+  IF ( TRIM(starting_wfc) == 'file' .AND. .NOT. exst_file) THEN
+     !
+     ! ... "tmp_dir"/"prefix".wfc not found on disk: try to read
+     ! ... wavefunctions in "collected" format from "prefix".save/, 
+     ! ... rewrite them (in pw_readfile) using the internal format
+     !
+     ierr = 1
+     CALL pw_readfile( 'wave', ierr )
+     IF ( ierr > 0 ) THEN
+        WRITE( stdout, '(5X,"Cannot read wfc : file not found")' )
+        starting_wfc = 'atomic+random'
+     END IF
+     !
+     ! ... workaround: with k-point parallelization and 1 k-point per pool,
+     ! ... pw_readfile does not leave evc properly initialized on all pools
+     !
+     IF ( nks == 1 ) CALL get_buffer( evc, nwordwfc, iunwfc, 1 )
+     !
+  ELSE IF ( TRIM(starting_wfc) == 'file' .AND. exst_file) THEN
+     !
+     ! ... wavefunctions are read from file (or buffer) in routine 
+     ! ... c_bands, but not if there is a single k-point. In such
+     ! ... a case, we read wavefunctions (directly from file in 
+     ! ... order to avoid a useless buffer allocation) here
+     !
+     IF ( nks == 1 ) CALL davcio ( evc, 2*nwordwfc, iunwfc, nks, -1 )
+     !
+  END IF
+  !
+  ! ... state what will happen
   !
   IF ( TRIM(starting_wfc) == 'file' ) THEN
      !
@@ -55,20 +89,21 @@ SUBROUTINE wfcinit()
   ELSE IF ( starting_wfc == 'atomic' ) THEN
      !
      IF ( natomwfc >= nbnd ) THEN
-        !
         WRITE( stdout, '(5X,"Starting wfc are ",I4," atomic wfcs")' ) natomwfc
-        !
      ELSE
-        !
         WRITE( stdout, '(5X,"Starting wfc are ",I4," atomic + ", &
              &           I4," random wfc")' ) natomwfc, nbnd-natomwfc
-        !
      END IF
      !
-  ELSE IF ( TRIM(starting_wfc) == 'atomic+random' ) THEN
+  ELSE IF ( TRIM(starting_wfc) == 'atomic+random' .AND. natomwfc > 0) THEN
      !
-     WRITE( stdout, '(5X,"Starting wfc are ",I4," randomized atomic wfcs")' ) &
+     IF ( natomwfc >= nbnd ) THEN
+        WRITE( stdout, '(5X,"Starting wfc are ",I4," randomized atomic wfcs")')&
              natomwfc
+     ELSE
+        WRITE( stdout, '(5X,"Starting wfc are ",I4," randomized atomic wfcs + "&
+             &          ,I4," random wfc")' ) natomwfc, nbnd-natomwfc
+     END IF
      !
   ELSE
      !
@@ -76,28 +111,13 @@ SUBROUTINE wfcinit()
      !
   END IF
   !
-  ! ... for non-scf calculations, the starting wavefunctions are not 
-  ! ... calculated here but immediately before diagonalization
+  ! ... exit here if starting from file or for non-scf calculations.
+  ! ... In the latter case the starting wavefunctions are not 
+  ! ... calculated here but just before diagonalization (to reduce I/O)
   !
-  IF ( .NOT. lscf .AND. .NOT. lelfield ) THEN
+  IF (  ( .NOT. lscf .AND. .NOT. lelfield ) .OR. TRIM(starting_wfc) == 'file' ) THEN
      !
      CALL stop_clock( 'wfcinit' )
-     !
-     RETURN
-     !
-  END IF
-  !
-  IF ( TRIM(starting_wfc) == 'file' ) THEN
-     !
-     ! ... wavefunctions are to be read from file: store wavefunction into
-     ! ... memory if c_bands will not do it (for a single k-point);
-     ! ... return and do nothing otherwise (c_bands will read wavefunctions)
-     !
-     IF ( nks == 1 .AND. (io_level < 2) ) &
-        CALL get_buffer ( evc, nwordwfc, iunwfc, 1 )
-     !
-     CALL stop_clock( 'wfcinit' )
-     !
      RETURN
      !
   END IF
@@ -121,10 +141,10 @@ SUBROUTINE wfcinit()
      !
      IF ( nkb > 0 ) CALL init_us_2( npw, igk, xk(1,ik), vkb )
      !
-     ! ... LDA+U: read atomic wavefunctions for U term in Hamiltonian
+     ! ... Needed for LDA+U
      !
-     IF ( lda_plus_u .AND. (U_projection .NE. 'pseudo') ) &
-         CALL davcio( swfcatom, nwordatwfc, iunsat, ik, - 1 )
+     IF ( nks > 1 .AND. lda_plus_u .AND. (U_projection .NE. 'pseudo') ) &
+        CALL get_buffer( wfcU, nwordwfcU, iunhub, ik )
      !
      ! ... calculate starting wavefunctions
      !
@@ -138,7 +158,6 @@ SUBROUTINE wfcinit()
   END DO
   !
   CALL stop_clock( 'wfcinit' )
-  !
   RETURN
   !
 END SUBROUTINE wfcinit
@@ -163,7 +182,7 @@ SUBROUTINE init_wfc ( ik )
   USE noncollin_module,     ONLY : npol
   USE wavefunctions_module, ONLY : evc
   USE random_numbers,       ONLY : randy
-  USE mp_global,            ONLY : intra_bgrp_comm
+  USE mp_bands,             ONLY : intra_bgrp_comm
   USE control_flags,        ONLY : gamma_only
   !
   IMPLICIT NONE
@@ -214,7 +233,6 @@ SUBROUTINE init_wfc ( ik )
             !
             DO ipol = 1, npol
                !
-!$OMP PARALLEL DO DEFAULT(SHARED) PRIVATE(ig, rr, arg)
                DO ig = 1, npw
                   !
                   rr  = randy()
@@ -224,7 +242,6 @@ SUBROUTINE init_wfc ( ik )
                      ( 1.0_DP + 0.05_DP * CMPLX( rr*COS(arg), rr*SIN(arg) ,kind=DP) ) 
                   !
                END DO
-!$OMP END PARALLEL DO
                !
             END DO
             !
@@ -243,7 +260,6 @@ SUBROUTINE init_wfc ( ik )
         ! 
         wfcatom(:,ipol,ibnd) = (0.0_dp, 0.0_dp)
         !
-!$OMP PARALLEL DO DEFAULT(SHARED) PRIVATE(ig, rr, arg)
         DO ig = 1, npw
            !
            rr  = randy()
@@ -255,7 +271,6 @@ SUBROUTINE init_wfc ( ik )
                          ( xk(2,ik) + g(2,igk(ig)) )**2 + &
                          ( xk(3,ik) + g(3,igk(ig)) )**2 + 1.0_DP )
         END DO
-!$OMP END PARALLEL DO
         !
      END DO
      !
