@@ -1,7 +1,7 @@
 #!/bin/sh -x
 
 tempdir=$HOME/Downloads
-version=5.1
+version=5.2.1
 
 # make sure there is no locale setting creating unneeded differences.
 LC_ALL=C
@@ -11,7 +11,7 @@ mkdir $tempdir
 cd $tempdir
 /bin/rm -rf espresso/ espresso-$version
 # get the svn copy
-svn checkout http://qeforge.qe-forge.org/svn/q-e/trunk/espresso
+svn checkout http://qeforge.qe-forge.org/svn/q-e/tags/QE-5-2-1/espresso
 mv espresso/ espresso-$version/
 
 cd espresso-$version
@@ -47,9 +47,10 @@ make doc
 make tar-gui PWGUI_VERSION=$version 
 tar -xzvf PWgui-$version.tgz
 /bin/rm PWgui-$version.tgz
-#
+
 cd ..
-tar -cvzf PWgui-$version.tar.gz    espresso-$version/PWgui-$version
+
+# core espresso
 
 tar -czvf espresso-$version.tar.gz espresso-$version/archive \
                                    espresso-$version/clib \
@@ -74,6 +75,7 @@ tar -czvf espresso-$version.tar.gz espresso-$version/archive \
 # Packages, ready for automatic unpacking
 
 cd espresso-$version
+tar -cvzf ../PWgui-$version.tar.gz    PWgui-$version
 tar -czvf ../PHonon-$version.tar.gz   PHonon PlotPhon QHA
 tar -czvf ../neb-$version.tar.gz      NEB
 tar -czvf ../pwcond-$version.tar.gz   PWCOND
