@@ -93,15 +93,15 @@ for dir in $dirs; do
 	$TOPDIR/moduledep.sh $DEPENDS > make.depend
 	$TOPDIR/includedep.sh $DEPENDS >> make.depend
 
-        # handle special cases
-        sed '/@\/cineca\/prod\/hpm\/include\/f_hpm.h@/d' \
-            make.depend > make.depend.tmp
-        sed '/@iso_c_binding@/d;/@ifcore@/d' make.depend.tmp > make.depend
+        # handle special cases: hardware-specific monitoring tools
+        sed '/@\/cineca\/prod\/hpm\/include\/f_hpm.h@/d;/@ifcore@/d' make.depend > make.depend.tmp
+        # handle special cases: modules for C-fortran binding, system utilities
+        sed '/@iso_c_binding@/d;/@f90_unix_env@/d' make.depend.tmp > make.depend
 
         if test "$DIR" = "Modules"
         then
-            sed '/@mpi@/d' make.depend > make.depend.tmp
-            sed '/@elpa1@/d' make.depend.tmp > make.depend
+            sed '/@mpi@/d;/@elpa1@/d' make.depend > make.depend.tmp
+            sed '/@mkl_dfti/d;/@fftw3.f/d' make.depend.tmp > make.depend
         fi
 
         if test "$DIR" = "clib"
@@ -112,14 +112,8 @@ for dir in $dirs; do
 
         if test "$DIR" = "PW/src" || test "$DIR" = "TDDFPT/src"
         then
-            sed '/@environ_base@/d'  make.depend > make.depend.tmp
-            sed '/@environ_input@/d' make.depend.tmp > make.depend
-            sed '/@environ_info@/d'  make.depend > make.depend.tmp
-            sed '/@environ_init@/d'  make.depend.tmp > make.depend
-            sed '/@environ_main@/d'  make.depend > make.depend.tmp
-            sed '/@environ_mp@/d'    make.depend.tmp > make.depend
-            sed '/@solvent_tddfpt@/d' make.depend > make.depend.tmp
-            mv make.depend.tmp make.depend
+            sed '/@environ_/d;/@solvent_tddfpt@/d' make.depend > make.depend.tmp
+            sed '/fft_defs.h@/d' make.depend.tmp > make.depend
         fi
 
         rm -f make.depend.tmp
