@@ -72,12 +72,12 @@ PROGRAM interpolate
   real (8) :: xmin, dx
   CHARACTER (len=256) :: filein(2), fileout
   PRINT '('' '')'
-  PRINT '('' Iterpolate an UPF pseudopotential to a different radial mesh'')'
+  PRINT '('' Interpolate an UPF pseudopotential to a different radial mesh'')'
   PRINT '('' '')'
   !
   is=2
   PRINT '('' Read the pseudo to be converted '')'
-  PRINT '('' Input PP file in UPF format > '',$)'
+  PRINT '('' Input PP file in UPF format > ''), advance="NO"'
   READ (5, '(a)', end = 20, err = 20) filein(is)
   OPEN(unit=iunps,file=filein(is),status='old',form='formatted',iostat=ios)
   IF (ios/=0) STOP
@@ -88,7 +88,7 @@ PROGRAM interpolate
   !
 10 CONTINUE
   PRINT '('' radial mesh : r(i) = exp ( xmin + (i-1) *dx )/ Z_ion '')'
-  WRITE(*,'(a,$)') " xmin, dx [typical values -7.0, 0.0125 ] > "
+  WRITE(*,'(a), advance="NO"') " xmin, dx [typical values -7.0, 0.0125 ] > "
   READ (*,*) xmin, dx
 
   CALL interpolate_ps(filein,xmin,dx)
@@ -97,7 +97,7 @@ PROGRAM interpolate
   PRINT '(''Output PP file in UPF format :  '',a)', fileout
 
   OPEN(unit=2,file=fileout,status='unknown',form='formatted')
-  CALL write_upf(2)
+  CALL write_upf_v1(2)
   CLOSE (unit=2)
 
 20 STOP
@@ -136,7 +136,7 @@ SUBROUTINE interpolate_ps(filein,xmin,dx)
   USE splinelib
   USE funct, ONLY : set_dft_from_name, get_iexch, get_icorr, get_igcx, get_igcc
   IMPLICIT NONE
-  real (8), INTENT(IN) :: dx, xmin
+  real (8), INTENT(in) :: dx, xmin
   INTEGER :: i, j, ib
   CHARACTER (len=256) :: filein(2)
   CHARACTER (len=5) :: dxlabel, xminlabel
@@ -146,7 +146,7 @@ SUBROUTINE interpolate_ps(filein,xmin,dx)
   interpolate = .false.
   !
   WRITE(dxlabel,'(f5.4)') dx
-  WRITE(xminlabel,'(f5.2)')xmin 
+  WRITE(xminlabel,'(f5.2)')xmin
   !pp_info
   upf_rel = -1
   upf_rcloc = 0.d0
@@ -168,11 +168,11 @@ SUBROUTINE interpolate_ps(filein,xmin,dx)
 
   zp(1)   = zp(2)
   mesh(1) = (log(r(mesh(2),2) * zp(2) ) - xmin ) /dx + 1
-  do i=1,mesh(1)
+  DO i=1,mesh(1)
      r(i,1) = exp(xmin+dble(i-1)*dx)/zp(1)
      rab(i,1) = r(i,1) * dx
-  end do
-  write (*,*) xmin, dx, mesh(1),zp(1)
+  ENDDO
+  WRITE (*,*) xmin, dx, mesh(1),zp(1)
 
   IF (mesh(1)/=mesh(2) ) THEN
      WRITE (*,*) " pseudopotentials have different mesh "
@@ -217,7 +217,7 @@ SUBROUTINE interpolate_ps(filein,xmin,dx)
   !pp_nlcc
   ALLOCATE( upf_rho_atc(upf_mesh) )
   IF (interpolate) THEN
-     WRITE (*,'(a,$)') "interpolate rho_atc"
+     WRITE (*,'(a), advance="NO"') "interpolate rho_atc"
      aux2(1,1:mesh(2)) = rho_atc(1:mesh(2),2)
      CALL dosplineint( r(1:mesh(2),2), aux2, upf_r(1:upf_mesh), aux1 )
      rho_atc(1:upf_mesh,2) = aux1(1,1:upf_mesh)
@@ -228,7 +228,7 @@ SUBROUTINE interpolate_ps(filein,xmin,dx)
   !pp_local
   ALLOCATE( upf_vloc0(upf_mesh) )
   IF (interpolate) THEN
-     WRITE (*,'(a,$)') " interpolate vloc0"
+     WRITE (*,'(a), advance="NO"') " interpolate vloc0"
      aux2(1,1:mesh(2)) =  vloc0(1:mesh(2),2)
 
      CALL dosplineint( r(1:mesh(2),2), aux2, upf_r(1:upf_mesh), aux1 )
@@ -259,7 +259,7 @@ SUBROUTINE interpolate_ps(filein,xmin,dx)
   DO i=1,nbeta(2)
      ib  = ib + 1
      IF (interpolate) THEN
-     WRITE (*,'(a,$)') " interpolate betar"
+     WRITE (*,'(a), advance="NO"') " interpolate betar"
         aux2(1,1:mesh(2)) = betar(1:mesh(2),i,2)
         CALL dosplineint( r(1:mesh(2),2), aux2, upf_r(1:upf_mesh), aux1 )
         betar(1:upf_mesh,i,2) = aux1(1,1:upf_mesh)
@@ -303,7 +303,7 @@ SUBROUTINE interpolate_ps(filein,xmin,dx)
      DO j=1,nbeta(2)
         upf_qqq(i,j) = qqq(i, j, 2)
         IF (interpolate) THEN
-     WRITE (*,'(a,$)') " interpolate qfunc"
+     WRITE (*,'(a), advance="NO"') " interpolate qfunc"
            aux2(1,1:mesh(2) ) = qfunc(1:mesh(2),i,j,2)
            CALL dosplineint( r(1:mesh(2),2), aux2, upf_r(1:upf_mesh), aux1 )
            qfunc(1:upf_mesh,i,j,2) = aux1(1,1:upf_mesh)
@@ -329,7 +329,7 @@ SUBROUTINE interpolate_ps(filein,xmin,dx)
 
   DO i=1,ntwfc(2)
      IF (interpolate) THEN
-        WRITE (*,'(a,$)') " interpolate chi"
+        WRITE (*,'(a), advance="NO"') " interpolate chi"
         aux2(1,1:mesh(2)) = chi(1:mesh(2),i,2)
         CALL dosplineint( r(1:mesh(2),2), aux2, upf_r(1:upf_mesh), aux1 )
         chi(1:upf_mesh,i,2) = aux1(1,1:upf_mesh)
@@ -343,7 +343,7 @@ SUBROUTINE interpolate_ps(filein,xmin,dx)
 
   ALLOCATE (upf_rho_at(upf_mesh) )
   IF (interpolate) THEN
-     WRITE (*,'(a,$)') " interpolate rho_at"
+     WRITE (*,'(a), advance="NO"') " interpolate rho_at"
      aux2(1,1:mesh(2)) = rho_at(1:mesh(2),2)
      CALL dosplineint( r(1:mesh(2),2), aux2, upf_r(1:upf_mesh), aux1 )
      rho_at(1:upf_mesh,2) = aux1(1,1:upf_mesh)
